searchData={"items":[{"type":"task","title":"mix websockex_nova.check_handlers","doc":"Checks all WebsockexNova.Defaults.* handler modules to ensure they implement all required callbacks for their respective behaviors.\nPrints missing callbacks or confirms all are correct.","ref":"Mix.Tasks.WebsockexNova.CheckHandlers.html"},{"type":"module","title":"WebsockexNova","doc":"WebsockexNova is a robust WebSocket client library for Elixir with a pluggable adapter architecture.","ref":"WebsockexNova.html"},{"type":"module","title":"Architecture Overview - WebsockexNova","doc":"WebsockexNova employs a \"thin adapter\" architecture that separates concerns through:\n\n1. **Behavioral Interfaces**: Well-defined behaviors for various aspects of WebSocket handling\n2. **Default Implementations**: Ready-to-use default implementations of these behaviors\n3. **Platform Adapters**: Thin adapters that bridge to specific platforms/services\n4. **Connection Management**: Process-based connection handling with ownership semantics\n\nThis modular design allows for maximum flexibility while minimizing boilerplate code.","ref":"WebsockexNova.html#module-architecture-overview"},{"type":"module","title":"Key Components - WebsockexNova","doc":"* **Connection**: The core GenServer process managing the WebSocket lifecycle\n* **Client**: A convenient API for interacting with connections\n* **Behaviors**: Interfaces for connection, message, authentication, error handling, etc.\n* **Defaults**: Ready-to-use implementations of all behaviors\n* **Platform Adapters**: Thin adapters for specific WebSocket services","ref":"WebsockexNova.html#module-key-components"},{"type":"module","title":"Basic Usage - WebsockexNova","doc":"```elixir\n# Start a connection to the Echo service\n{:ok, conn} = WebsockexNova.Connection.start_link(\n  adapter: WebsockexNova.Platform.Echo.Adapter\n)\n\n# Send a message and get the response\n{:text, response} = WebsockexNova.Client.send_text(conn, \"Hello\")\n```","ref":"WebsockexNova.html#module-basic-usage"},{"type":"module","title":"Using with Custom Handlers - WebsockexNova","doc":"```elixir\n# Start a connection with custom handlers\n{:ok, conn} = WebsockexNova.Connection.start_link(\n  adapter: WebsockexNova.Platform.Echo.Adapter,\n  message_handler: MyApp.MessageHandler,\n  connection_handler: MyApp.ConnectionHandler\n)\n```","ref":"WebsockexNova.html#module-using-with-custom-handlers"},{"type":"module","title":"Creating a Client Module - WebsockexNova","doc":"```elixir\ndefmodule MyApp.WebSocketClient do\n  use GenServer\n\n  def start_link(opts \\\\ []) do\n    GenServer.start_link(__MODULE__, opts)\n  end\n\n  def init(opts) do\n    # Start the WebSocket connection\n    {:ok, conn} = WebsockexNova.Connection.start_link(\n      adapter: WebsockexNova.Platform.Echo.Adapter\n    )\n\n    {:ok, %{conn: conn}}\n  end\n\n  # API functions\n  def send_message(client, message) do\n    GenServer.call(client, {:send, message})\n  end\n\n  # Callbacks\n  def handle_call({:send, message}, _from, %{conn: conn} = state) do\n    result = WebsockexNova.Client.send_text(conn, message)\n    {:reply, result, state}\n  end\nend\n```","ref":"WebsockexNova.html#module-creating-a-client-module"},{"type":"module","title":"Implementing Custom Handlers - WebsockexNova","doc":"Each aspect of WebSocket communication can be customized by implementing one of\nthe behaviors in `WebsockexNova.Behaviors`:\n\n```elixir\ndefmodule MyApp.MessageHandler do\n  @behaviour WebsockexNova.Behaviors.MessageHandler\n\n  @impl true\n  def init(opts) do\n    {:ok, %{messages: []}}\n  end\n\n  @impl true\n  def handle_message(frame_type, data, state) do\n    IO.puts(\"Received #{frame_type} message: #{inspect(data)}\")\n    new_state = update_in(state.messages, &[{frame_type, data} | &1])\n    {:ok, new_state}\n  end\nend\n```","ref":"WebsockexNova.html#module-implementing-custom-handlers"},{"type":"module","title":"Creating a Platform Adapter - WebsockexNova","doc":"To support a new WebSocket service, implement the `WebsockexNova.Platform.Adapter` behavior:\n\n```elixir\ndefmodule MyApp.CustomAdapter do\n  use WebsockexNova.Platform.Adapter,\n    default_host: \"api.example.com\",\n    default_port: 443,\n    default_path: \"/websocket\"\n\n  @impl true\n  def handle_platform_message(message, state) do\n    # Custom message handling\n    {:reply, {:text, \"Processed: #{inspect(message)}\"}, state}\n  end\n\n  @impl true\n  def encode_auth_request(credentials) do\n    {:text, Jason.encode!(%{\n      type: \"auth\",\n      key: credentials.api_key,\n      secret: credentials.api_secret\n    })}\n  end\n\n  @impl true\n  def encode_subscription_request(channel, params) do\n    {:text, Jason.encode!(%{\n      type: \"subscribe\",\n      channel: channel,\n      params: params\n    })}\n  end\n\n  @impl true\n  def encode_unsubscription_request(channel) do\n    {:text, Jason.encode!(%{\n      type: \"unsubscribe\",\n      channel: channel\n    })}\n  end\nend\n```","ref":"WebsockexNova.html#module-creating-a-platform-adapter"},{"type":"module","title":"Available Behaviors - WebsockexNova","doc":"* `ConnectionHandler`: Handle connection lifecycle events\n* `MessageHandler`: Process incoming WebSocket messages\n* `SubscriptionHandler`: Manage channel subscriptions\n* `AuthHandler`: Handle authentication\n* `ErrorHandler`: Process error scenarios\n* `RateLimitHandler`: Implement rate limiting\n* `LoggingHandler`: Provide logging functionality\n* `MetricsCollector`: Collect metrics about WebSocket operations\n\nEach behavior has a corresponding default implementation in the `WebsockexNova.Defaults` namespace.","ref":"WebsockexNova.html#module-available-behaviors"},{"type":"module","title":"WebsockexNova.Adapter","doc":"Macro for building WebsockexNova adapters with minimal boilerplate.\n\nThis macro provides a convenient way to create platform-specific WebSocket adapters\nby automatically implementing all required behaviors with sensible defaults. You only\nneed to override the specific behaviors needed for your use case.","ref":"WebsockexNova.Adapter.html"},{"type":"module","title":"Basic Usage - WebsockexNova.Adapter","doc":"defmodule MyApp.MyAdapter do\n      use WebsockexNova.Adapter\n\n      # Override only what you need:\n      @impl WebsockexNova.Behaviors.MessageHandler\n      def handle_message(message, state) do\n        # Custom message handling\n        {:ok, decoded_message, state}\n      end\n    end","ref":"WebsockexNova.Adapter.html#module-basic-usage"},{"type":"module","title":"What this macro does - WebsockexNova.Adapter","doc":"- Declares all core `@behaviour` attributes (AuthHandler, ConnectionHandler, etc.)\n- Injects default implementations for all callbacks, delegating to `WebsockexNova.Defaults.*`\n- Allows you to override any callback by implementing it in your module\n- Imports necessary aliases for convenient access to behaviors and defaults","ref":"WebsockexNova.Adapter.html#module-what-this-macro-does"},{"type":"module","title":"Examples - WebsockexNova.Adapter","doc":"","ref":"WebsockexNova.Adapter.html#module-examples"},{"type":"module","title":"Simple adapter with custom connection info - WebsockexNova.Adapter","doc":"defmodule MyApp.EchoAdapter do\n      use WebsockexNova.Adapter\n\n      @impl WebsockexNova.Behaviors.ConnectionHandler\n      def connection_info(opts) do\n        {:ok, Map.merge(%{\n          host: \"echo.websocket.org\",\n          port: 443,\n          path: \"/\",\n          transport: :tls\n        }, opts)}\n      end\n    end","ref":"WebsockexNova.Adapter.html#module-simple-adapter-with-custom-connection-info"},{"type":"module","title":"Financial trading adapter with authentication - WebsockexNova.Adapter","doc":"defmodule MyApp.TradingAdapter do\n      use WebsockexNova.Adapter\n\n      @impl WebsockexNova.Behaviors.ConnectionHandler\n      def connection_info(opts) do\n        {:ok, Map.merge(%{\n          host: \"api.exchange.com\",\n          port: 443,\n          path: \"/ws/v2\",\n          transport: :tls\n        }, opts)}\n      end\n\n      @impl WebsockexNova.Behaviors.AuthHandler\n      def authenticate(conn, auth_config, state) do\n        # Send authentication message\n        auth_msg = %{\n          method: \"auth\",\n          params: %{\n            api_key: auth_config.api_key,\n            api_secret: auth_config.api_secret\n          }\n        }\n        WebsockexNova.Client.send_message(conn, auth_msg)\n        {:ok, state}\n      end\n\n      @impl WebsockexNova.Behaviors.MessageHandler\n      def handle_message(%{\"method\" => \"auth\", \"result\" => true}, state) do\n        # Authentication successful\n        {:ok, %{authenticated: true}, Map.put(state, :authenticated, true)}\n      end\n      def handle_message(message, state) do\n        # Decode JSON messages\n        with {:ok, decoded} <- Jason.decode(message) do\n          {:ok, decoded, state}\n        else\n          _ -> {:error, :invalid_json, state}\n        end\n      end\n    end","ref":"WebsockexNova.Adapter.html#module-financial-trading-adapter-with-authentication"},{"type":"module","title":"Gaming adapter with custom subscription handling - WebsockexNova.Adapter","doc":"defmodule MyApp.GameAdapter do\n      use WebsockexNova.Adapter\n      alias WebsockexNova.Message.SubscriptionManager\n\n      @impl WebsockexNova.Behaviors.SubscriptionHandler\n      def handle_subscription(channel, opts, conn, state) do\n        # Custom subscription format for game events\n        subscribe_msg = %{\n          action: \"subscribe\",\n          channel: channel,\n          params: opts || %{}\n        }\n        \n        case WebsockexNova.Client.send_message(conn, subscribe_msg) do\n          {:ok, _} ->\n            manager = Map.get(state, :subscription_manager, SubscriptionManager.new())\n            updated_manager = SubscriptionManager.add_subscription(manager, channel, opts)\n            {:ok, %{}, Map.put(state, :subscription_manager, updated_manager)}\n          error ->\n            error\n        end\n      end\n\n      @impl WebsockexNova.Behaviors.MessageHandler\n      def handle_message(%{\"type\" => \"game_event\"} = message, state) do\n        # Handle game-specific events\n        {:ok, message, state}\n      end\n      def handle_message(message, state) do\n        DefaultMessageHandler.handle_message(message, state)\n      end\n    end","ref":"WebsockexNova.Adapter.html#module-gaming-adapter-with-custom-subscription-handling"},{"type":"module","title":"Available Behaviors - WebsockexNova.Adapter","doc":"The macro automatically implements these behaviors with default implementations:\n\n- `WebsockexNova.Behaviors.AuthHandler` - Authentication flow\n- `WebsockexNova.Behaviors.ConnectionHandler` - Connection lifecycle\n- `WebsockexNova.Behaviors.ErrorHandler` - Error handling strategies\n- `WebsockexNova.Behaviors.MessageHandler` - Message processing\n- `WebsockexNova.Behaviors.SubscriptionHandler` - Channel subscriptions\n\nNote: LoggingHandler, MetricsCollector, and RateLimitHandler are not included by default\nsince they're typically configured at the client level rather than the adapter level.","ref":"WebsockexNova.Adapter.html#module-available-behaviors"},{"type":"module","title":"Tips - WebsockexNova.Adapter","doc":"1. Only override the behaviors you need to customize\n2. Call the default implementations when you want to extend rather than replace behavior\n3. Use pattern matching in message handlers for efficient message routing\n4. Store adapter-specific state in the adapter_state field of the connection\n\nSee the `WebsockexNova.Examples.AdapterDeribit` module for a complete real-world example.","ref":"WebsockexNova.Adapter.html#module-tips"},{"type":"module","title":"WebsockexNova.Auth.AuthFlow","doc":"Manages authentication flows for WebSocket connections.\n\nThe `AuthFlow` module provides standardized functions for:\n\n- Initial authentication when a connection is established\n- Reauthentication when tokens expire or authentication fails\n- Processing authentication responses from the server\n- Determining when reauthentication is needed","ref":"WebsockexNova.Auth.AuthFlow.html"},{"type":"module","title":"Authentication Flow - WebsockexNova.Auth.AuthFlow","doc":"1. When a connection is established, `authenticate/2` is called\n2. The AuthHandler generates authentication data via `generate_auth_data/1`\n3. Authentication data is sent to the server\n4. Server responds with success or error\n5. `handle_auth_response/2` processes the response and updates state\n6. During connection, `check_reauthentication/2` checks if auth renewal is needed","ref":"WebsockexNova.Auth.AuthFlow.html#module-authentication-flow"},{"type":"module","title":"Example Usage - WebsockexNova.Auth.AuthFlow","doc":"```elixir\n# During initial connection\n{:ok, state} = AuthFlow.authenticate(state, &send_frame_fn/4)\n\n# When a message is received that might be an auth response\n{:ok, updated_state} = AuthFlow.handle_auth_response(message, state)\n\n# Periodically check if reauthentication is needed\ncase AuthFlow.check_reauthentication(state, &send_frame_fn/4) do\n  {:ok, state} -> state # No reauthentication needed\n  {:reauthenticated, state} -> state # Reauthentication initiated\nend\n```","ref":"WebsockexNova.Auth.AuthFlow.html#module-example-usage"},{"type":"function","title":"WebsockexNova.Auth.AuthFlow.authenticate/2","doc":"Initiates the authentication process using the configured AuthHandler.\n\nThis function:\n1. Retrieves the AuthHandler from the state\n2. Calls generate_auth_data/1 to get authentication data\n3. Sends the authentication data to the server\n4. Returns the updated state\n\nThe actual authentication response will be processed later via handle_auth_response/2.","ref":"WebsockexNova.Auth.AuthFlow.html#authenticate/2"},{"type":"function","title":"Parameters - WebsockexNova.Auth.AuthFlow.authenticate/2","doc":"- `state`: The current state containing authentication handler configuration\n- `send_frame_fn`: Function to send frames to the WebSocket server,\n  with signature `(conn, frame_type, frame_data, stream_ref) -> :ok | {:error, reason}`","ref":"WebsockexNova.Auth.AuthFlow.html#authenticate/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Auth.AuthFlow.authenticate/2","doc":"- `{:ok, updated_state}`: Authentication data sent successfully\n- `{:error, reason, updated_state}`: Error generating authentication data","ref":"WebsockexNova.Auth.AuthFlow.html#authenticate/2-returns"},{"type":"function","title":"WebsockexNova.Auth.AuthFlow.check_reauthentication/2","doc":"Checks if reauthentication is needed and initiates the process if necessary.\n\nThis function:\n1. Retrieves the AuthHandler from the state\n2. Calls needs_reauthentication?/1 to check if reauthentication is needed\n3. If needed, initiates the authentication process","ref":"WebsockexNova.Auth.AuthFlow.html#check_reauthentication/2"},{"type":"function","title":"Parameters - WebsockexNova.Auth.AuthFlow.check_reauthentication/2","doc":"- `state`: The current state containing authentication handler configuration\n- `send_frame_fn`: Function to send frames to the WebSocket server,\n  with signature `(conn, frame_type, frame_data, stream_ref) -> :ok | {:error, reason}`","ref":"WebsockexNova.Auth.AuthFlow.html#check_reauthentication/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Auth.AuthFlow.check_reauthentication/2","doc":"- `{:ok, state}`: No reauthentication needed\n- `{:reauthenticated, state}`: Reauthentication initiated\n- `{:error, reason, state}`: Error during reauthentication","ref":"WebsockexNova.Auth.AuthFlow.html#check_reauthentication/2-returns"},{"type":"function","title":"WebsockexNova.Auth.AuthFlow.handle_auth_response/2","doc":"Handles an authentication response from the server.\n\nThis function:\n1. Retrieves the AuthHandler from the state\n2. Calls handle_auth_response/2 to process the authentication response\n3. Updates the state with the result","ref":"WebsockexNova.Auth.AuthFlow.html#handle_auth_response/2"},{"type":"function","title":"Parameters - WebsockexNova.Auth.AuthFlow.handle_auth_response/2","doc":"- `response`: The response message received from the server\n- `state`: The current state containing authentication handler configuration","ref":"WebsockexNova.Auth.AuthFlow.html#handle_auth_response/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Auth.AuthFlow.handle_auth_response/2","doc":"- `{:ok, updated_state}`: Authentication successful or message processed\n- `{:error, reason, updated_state}`: Authentication failed","ref":"WebsockexNova.Auth.AuthFlow.html#handle_auth_response/2-returns"},{"type":"behaviour","title":"WebsockexNova.Behaviors.AuthHandler","doc":"Defines a behavior for handling WebSocket authentication flows.\n\nThe `AuthHandler` behavior provides a standardized interface for authentication\noperations in WebSocket connections, including:\n\n- Generating authentication data to send to the server\n- Processing authentication responses from the server\n- Determining when reauthentication is needed","ref":"WebsockexNova.Behaviors.AuthHandler.html"},{"type":"behaviour","title":"Binary Data Support - WebsockexNova.Behaviors.AuthHandler","doc":"WebsockexNova supports both structured map data and raw binary data for authentication:\n\n- `generate_auth_data/1` can return either a map or binary data\n- This flexibility allows implementing various authentication protocols\n- Binary authentication data is useful for token-based or custom binary protocols\n- Map-based authentication is convenient for JSON or structured payloads","ref":"WebsockexNova.Behaviors.AuthHandler.html#module-binary-data-support"},{"type":"behaviour","title":"Architecture - WebsockexNova.Behaviors.AuthHandler","doc":"WebsockexNova uses a thin adapter architecture for authentication, allowing applications\nto implement service-specific authentication logic while the core library handles\nconnection and message flow.","ref":"WebsockexNova.Behaviors.AuthHandler.html#module-architecture"},{"type":"behaviour","title":"Authentication Flow - WebsockexNova.Behaviors.AuthHandler","doc":"1. The client needs to authenticate (initial connection or reauthentication)\n2. `generate_auth_data/1` is called to create authentication payload\n3. The authentication message is sent to the server\n4. Server responds with success or error\n5. `handle_auth_response/2` processes the response and updates state\n6. During connection, `needs_reauthentication?/1` periodically checks if auth renewal is needed","ref":"WebsockexNova.Behaviors.AuthHandler.html#module-authentication-flow"},{"type":"behaviour","title":"Implementation Example - WebsockexNova.Behaviors.AuthHandler","doc":"```elixir\ndefmodule MyApp.ApiAuthHandler do\n  @behaviour WebsockexNova.Behaviors.AuthHandler\n\n  @impl true\n  def generate_auth_data(state) do\n    # Get API credentials from state\n    %{api_key: api_key, secret: secret} = state.credentials\n\n    # Generate timestamp and signature\n    timestamp = System.system_time(:second)\n    signature = generate_hmac_signature(secret, timestamp)\n\n    # Create auth payload\n    auth_data = %{\n      type: \"auth\",\n      api_key: api_key,\n      timestamp: timestamp,\n      signature: signature\n    }\n\n    {:ok, auth_data, state}\n  end\n\n  @impl true\n  def handle_auth_response(%{\"type\" => \"auth_success\", \"expires_at\" => expires_at}, state) do\n    # Update state with authentication status and expiration\n    updated_state =\n      state\n      |> Map.put(:auth_status, :authenticated)\n      |> Map.put(:auth_expires_at, expires_at)\n\n    {:ok, updated_state}\n  end\n\n  @impl true\n  def handle_auth_response(%{\"type\" => \"auth_error\", \"reason\" => reason}, state) do\n    # Update state with error information\n    updated_state =\n      state\n      |> Map.put(:auth_status, :failed)\n      |> Map.put(:auth_error, reason)\n\n    {:error, reason, updated_state}\n  end\n\n  @impl true\n  def handle_auth_response(_response, state) do\n    # Ignore unrelated messages\n    {:ok, state}\n  end\n\n  @impl true\n  def needs_reauthentication?(state) do\n    cond do\n      state.auth_status == :failed -> true\n      not Map.has_key?(state, :auth_expires_at) -> false\n      state.auth_expires_at   true\n      true -> false\n    end\n  end\n\n  defp generate_hmac_signature(secret, timestamp) do\n    # Implementation of signature generation\n  end\nend\n```","ref":"WebsockexNova.Behaviors.AuthHandler.html#module-implementation-example"},{"type":"type","title":"WebsockexNova.Behaviors.AuthHandler.auth_data/0","doc":"Authentication payload to send to the server","ref":"WebsockexNova.Behaviors.AuthHandler.html#t:auth_data/0"},{"type":"type","title":"WebsockexNova.Behaviors.AuthHandler.auth_response/0","doc":"Authentication response from the server","ref":"WebsockexNova.Behaviors.AuthHandler.html#t:auth_response/0"},{"type":"callback","title":"WebsockexNova.Behaviors.AuthHandler.authenticate/3","doc":"Authenticates using the provided credentials and current state.","ref":"WebsockexNova.Behaviors.AuthHandler.html#c:authenticate/3"},{"type":"callback","title":"Parameters - WebsockexNova.Behaviors.AuthHandler.authenticate/3","doc":"- credentials: The credentials to use for authentication (map or struct)\n- state: The current state map\n- stream_ref: The stream reference for the WebSocket connection","ref":"WebsockexNova.Behaviors.AuthHandler.html#c:authenticate/3-parameters"},{"type":"callback","title":"Returns - WebsockexNova.Behaviors.AuthHandler.authenticate/3","doc":"- `{:ok, updated_state}`: Authentication successful\n- `{:error, reason, updated_state}`: Authentication failed","ref":"WebsockexNova.Behaviors.AuthHandler.html#c:authenticate/3-returns"},{"type":"callback","title":"WebsockexNova.Behaviors.AuthHandler.generate_auth_data/1","doc":"Generates authentication data to be sent to the WebSocket server.\n\nThis function is called when authentication is needed, either during initial\nconnection or when reauthentication is required.","ref":"WebsockexNova.Behaviors.AuthHandler.html#c:generate_auth_data/1"},{"type":"callback","title":"Parameters - WebsockexNova.Behaviors.AuthHandler.generate_auth_data/1","doc":"- `state`: The current state map, expected to contain authentication credentials","ref":"WebsockexNova.Behaviors.AuthHandler.html#c:generate_auth_data/1-parameters"},{"type":"callback","title":"Returns - WebsockexNova.Behaviors.AuthHandler.generate_auth_data/1","doc":"- `{:ok, auth_data, state}`: Authentication data generated successfully\n  - `auth_data`: A map or binary containing authentication payload to send to the server\n  - `state`: Potentially updated state after generating authentication data\n- `{:error, reason, state}`: Failed to generate authentication data\n  - `reason`: Atom or string describing the error\n  - `state`: Potentially updated state after the error","ref":"WebsockexNova.Behaviors.AuthHandler.html#c:generate_auth_data/1-returns"},{"type":"callback","title":"WebsockexNova.Behaviors.AuthHandler.handle_auth_response/2","doc":"Processes an authentication response received from the WebSocket server.\n\nThis function is called when a message that might be an authentication response\nis received. It should identify if the message is authentication-related and\nupdate the state accordingly.","ref":"WebsockexNova.Behaviors.AuthHandler.html#c:handle_auth_response/2"},{"type":"callback","title":"Parameters - WebsockexNova.Behaviors.AuthHandler.handle_auth_response/2","doc":"- `response`: The response message from the server, typically a map\n- `state`: The current state map","ref":"WebsockexNova.Behaviors.AuthHandler.html#c:handle_auth_response/2-parameters"},{"type":"callback","title":"Returns - WebsockexNova.Behaviors.AuthHandler.handle_auth_response/2","doc":"- `{:ok, updated_state}`: Authentication successful or message processed\n  - `updated_state`: State updated with authentication information\n- `{:error, reason, updated_state}`: Authentication failed\n  - `reason`: Atom or string describing the error\n  - `updated_state`: State updated with error information","ref":"WebsockexNova.Behaviors.AuthHandler.html#c:handle_auth_response/2-returns"},{"type":"callback","title":"WebsockexNova.Behaviors.AuthHandler.needs_reauthentication?/1","doc":"Determines if reauthentication is needed based on the current state.\n\nThis function is called periodically to check if authentication needs to be renewed,\ntypically due to token expiration or previous authentication failure.","ref":"WebsockexNova.Behaviors.AuthHandler.html#c:needs_reauthentication?/1"},{"type":"callback","title":"Parameters - WebsockexNova.Behaviors.AuthHandler.needs_reauthentication?/1","doc":"- `state`: The current state map, containing authentication status information","ref":"WebsockexNova.Behaviors.AuthHandler.html#c:needs_reauthentication?/1-parameters"},{"type":"callback","title":"Returns - WebsockexNova.Behaviors.AuthHandler.needs_reauthentication?/1","doc":"- `true`: Reauthentication is needed\n- `false`: Current authentication is still valid","ref":"WebsockexNova.Behaviors.AuthHandler.html#c:needs_reauthentication?/1-returns"},{"type":"type","title":"WebsockexNova.Behaviors.AuthHandler.state/0","doc":"Handler state","ref":"WebsockexNova.Behaviors.AuthHandler.html#t:state/0"},{"type":"behaviour","title":"WebsockexNova.Behaviors.ClientBehavior","doc":"Behavior defining the contract for WebSocket client operations.\n\nThis behavior ensures a consistent API for WebSocket client operations across\ndifferent implementations and enables proper mocking in tests.","ref":"WebsockexNova.Behaviors.ClientBehavior.html"},{"type":"callback","title":"WebsockexNova.Behaviors.ClientBehavior.authenticate/3","doc":"Authenticates with the WebSocket server.","ref":"WebsockexNova.Behaviors.ClientBehavior.html#c:authenticate/3"},{"type":"callback","title":"WebsockexNova.Behaviors.ClientBehavior.close/1","doc":"Closes the WebSocket connection.","ref":"WebsockexNova.Behaviors.ClientBehavior.html#c:close/1"},{"type":"callback","title":"WebsockexNova.Behaviors.ClientBehavior.connect/2","doc":"Connects to a WebSocket server using the specified adapter.","ref":"WebsockexNova.Behaviors.ClientBehavior.html#c:connect/2"},{"type":"callback","title":"WebsockexNova.Behaviors.ClientBehavior.ping/2","doc":"Sends a ping message to the WebSocket server.","ref":"WebsockexNova.Behaviors.ClientBehavior.html#c:ping/2"},{"type":"callback","title":"WebsockexNova.Behaviors.ClientBehavior.register_callback/2","doc":"Registers a process to receive notifications from the connection.","ref":"WebsockexNova.Behaviors.ClientBehavior.html#c:register_callback/2"},{"type":"callback","title":"WebsockexNova.Behaviors.ClientBehavior.send_frame/2","doc":"Sends a raw WebSocket frame.","ref":"WebsockexNova.Behaviors.ClientBehavior.html#c:send_frame/2"},{"type":"callback","title":"WebsockexNova.Behaviors.ClientBehavior.send_json/3","doc":"Sends a JSON message.","ref":"WebsockexNova.Behaviors.ClientBehavior.html#c:send_json/3"},{"type":"callback","title":"WebsockexNova.Behaviors.ClientBehavior.send_text/3","doc":"Sends a text message.","ref":"WebsockexNova.Behaviors.ClientBehavior.html#c:send_text/3"},{"type":"callback","title":"WebsockexNova.Behaviors.ClientBehavior.status/2","doc":"Gets the current connection status.","ref":"WebsockexNova.Behaviors.ClientBehavior.html#c:status/2"},{"type":"callback","title":"WebsockexNova.Behaviors.ClientBehavior.subscribe/3","doc":"Subscribes to a channel or topic.","ref":"WebsockexNova.Behaviors.ClientBehavior.html#c:subscribe/3"},{"type":"callback","title":"WebsockexNova.Behaviors.ClientBehavior.unregister_callback/2","doc":"Unregisters a process from receiving notifications.","ref":"WebsockexNova.Behaviors.ClientBehavior.html#c:unregister_callback/2"},{"type":"callback","title":"WebsockexNova.Behaviors.ClientBehavior.unsubscribe/3","doc":"Unsubscribes from a channel or topic.","ref":"WebsockexNova.Behaviors.ClientBehavior.html#c:unsubscribe/3"},{"type":"behaviour","title":"WebsockexNova.Behaviors.ConnectionHandler","doc":"Behaviour for handling WebSocket connection lifecycle events.\n\nThe ConnectionHandler defines callbacks for managing the complete lifecycle of a WebSocket\nconnection, from initial connection parameters through disconnection. It serves as the\nfoundation for platform-specific connection management.","ref":"WebsockexNova.Behaviors.ConnectionHandler.html"},{"type":"behaviour","title":"Architecture - WebsockexNova.Behaviors.ConnectionHandler","doc":"ConnectionHandler is responsible for:\n- Providing connection parameters (host, port, path, transport options)\n- Handling connection establishment events\n- Managing disconnection events\n- Preparing outgoing frames\n- Building and sending ping frames\n- Processing incoming pong responses\n\nThe handler maintains its state as a map throughout the connection lifecycle.","ref":"WebsockexNova.Behaviors.ConnectionHandler.html#module-architecture"},{"type":"behaviour","title":"Callback Flow - WebsockexNova.Behaviors.ConnectionHandler","doc":"1. `init/1` - Called once when the handler is initialized\n2. `connection_info/1` - Provides connection parameters for establishing connection\n3. `handle_connect/2` - Called when WebSocket connection is established\n4. `prepare_frame/3` - Called before sending any frame (for encryption, encoding, etc.)\n5. `ping/2` - Called to build ping frames (if custom format needed)\n6. `handle_pong/2` - Called when pong frames are received\n7. `handle_disconnect/2` - Called on disconnection","ref":"WebsockexNova.Behaviors.ConnectionHandler.html#module-callback-flow"},{"type":"behaviour","title":"Reconnection Policy - WebsockexNova.Behaviors.ConnectionHandler","doc":"Important: ConnectionHandler does NOT handle reconnection logic.\n- Never return `{:reconnect, state}` from any callback\n- Reconnection decisions are made exclusively by the ErrorHandler\n- On disconnect, return `{:ok, state}` or `{:stop, reason, state}`","ref":"WebsockexNova.Behaviors.ConnectionHandler.html#module-reconnection-policy"},{"type":"behaviour","title":"Implementation Example - WebsockexNova.Behaviors.ConnectionHandler","doc":"defmodule MyApp.CustomConnectionHandler do\n      @behaviour WebsockexNova.Behaviors.ConnectionHandler\n\n      @impl true\n      def init(opts) do\n        state = %{\n          options: opts,\n          connected_at: nil,\n          ping_interval: opts[:ping_interval] || 30_000\n        }\n        {:ok, state}\n      end\n\n      @impl true\n      def connection_info(opts) do\n        # Merge runtime options with defaults\n        conn_info = %{\n          host: opts[:host] || \"api.example.com\",\n          port: opts[:port] || 443,\n          path: opts[:path] || \"/ws/v1\",\n          transport: :tls,\n          transport_opts: %{\n            verify: :verify_peer,\n            cacerts: :certifi.cacerts(),\n            server_name_indication: to_charlist(opts[:host] || \"api.example.com\")\n          },\n          timeout: opts[:timeout] || 10_000\n        }\n        {:ok, conn_info}\n      end\n\n      @impl true\n      def handle_connect(conn_info, state) do\n        # Connection established, update state\n        updated_state = Map.merge(state, %{\n          connected_at: DateTime.utc_now(),\n          conn_info: conn_info\n        })\n        {:ok, updated_state}\n      end\n\n      @impl true\n      def prepare_frame(frame_data, frame_type, state) do\n        # Optionally transform frame data before sending\n        # e.g., add encryption, compression, or custom encoding\n        case frame_type do\n          :text ->\n            # Maybe add message ID or timestamp\n            prepared = Jason.encode!(%{\n              id: UUID.uuid4(),\n              timestamp: System.system_time(:millisecond),\n              data: frame_data\n            })\n            {:ok, prepared, state}\n          \n          :binary ->\n            # Binary frames might need different preparation\n            {:ok, frame_data, state}\n          \n          _ ->\n            # Control frames usually pass through unchanged\n            {:ok, frame_data, state}\n        end\n      end\n\n      @impl true\n      def ping(state) do\n        # Build custom ping frame if needed\n        ping_data = %{\n          timestamp: System.system_time(:millisecond),\n          sequence: Map.get(state, :ping_sequence, 0)\n        }\n        \n        updated_state = Map.update(state, :ping_sequence, 1, &(&1 + 1))\n        {:ok, Jason.encode!(ping_data), updated_state}\n      end\n\n      @impl true\n      def handle_pong(frame_data, state) do\n        # Process pong response, maybe calculate latency\n        with {:ok, pong_data} <- Jason.decode(frame_data) do\n          latency = System.system_time(:millisecond) - pong_data[\"timestamp\"]\n          updated_state = Map.put(state, :last_latency, latency)\n          {:ok, updated_state}\n        else\n          _ ->\n            # Malformed pong, but don't crash\n            {:ok, state}\n        end\n      end\n\n      @impl true\n      def handle_disconnect(reason, state) do\n        # Clean up resources, log disconnection\n        # Remember: Don't return {:reconnect, state} here!\n        case reason do\n          :normal ->\n            {:ok, Map.put(state, :disconnected_at, DateTime.utc_now())}\n          \n          {:error, :timeout} ->\n            # Log timeout but let ErrorHandler decide on reconnection\n            {:ok, Map.put(state, :last_error, :timeout)}\n          \n          other ->\n            # Unexpected disconnection\n            {:ok, Map.put(state, :last_error, other)}\n        end\n      end\n    end","ref":"WebsockexNova.Behaviors.ConnectionHandler.html#module-implementation-example"},{"type":"behaviour","title":"Connection Info Structure - WebsockexNova.Behaviors.ConnectionHandler","doc":"The `connection_info/1` callback should return a map with these keys:\n- `:host` - WebSocket server hostname (required)\n- `:port` - Port number (required)\n- `:path` - WebSocket path (required)\n- `:transport` - Either `:tcp` or `:tls` (required)\n- `:transport_opts` - Transport-specific options map (optional)\n- `:timeout` - Connection timeout in milliseconds (optional)\n- `:protocols` - List of WebSocket subprotocols (optional)\n- `:headers` - Additional HTTP headers for upgrade (optional)","ref":"WebsockexNova.Behaviors.ConnectionHandler.html#module-connection-info-structure"},{"type":"behaviour","title":"Tips - WebsockexNova.Behaviors.ConnectionHandler","doc":"1. Keep connection state separate from application state\n2. Use prepare_frame/3 for consistent message formatting\n3. Implement proper error handling in all callbacks\n4. Consider implementing heartbeat/ping logic for connection health\n5. Log important connection events for debugging\n6. Don't implement reconnection logic in this behavior\n\nSee `WebsockexNova.Defaults.DefaultConnectionHandler` for a reference implementation.","ref":"WebsockexNova.Behaviors.ConnectionHandler.html#module-tips"},{"type":"type","title":"WebsockexNova.Behaviors.ConnectionHandler.conn_info/0","doc":"Connection info map","ref":"WebsockexNova.Behaviors.ConnectionHandler.html#t:conn_info/0"},{"type":"callback","title":"WebsockexNova.Behaviors.ConnectionHandler.connection_info/1","doc":"Returns connection information for establishing a WebSocket connection.\nReturns:\n  - `{:ok, conn_info}`\n  - `{:error, reason}`","ref":"WebsockexNova.Behaviors.ConnectionHandler.html#c:connection_info/1"},{"type":"type","title":"WebsockexNova.Behaviors.ConnectionHandler.disconnect_reason/0","doc":"Disconnect reason","ref":"WebsockexNova.Behaviors.ConnectionHandler.html#t:disconnect_reason/0"},{"type":"type","title":"WebsockexNova.Behaviors.ConnectionHandler.frame_type/0","doc":"Frame type","ref":"WebsockexNova.Behaviors.ConnectionHandler.html#t:frame_type/0"},{"type":"callback","title":"WebsockexNova.Behaviors.ConnectionHandler.handle_connect/2","doc":"Handle connection establishment.\nReturns:\n  - `{:ok, state}`\n  - `{:reply, frame_type, data, state}`\n  - `{:close, code, reason, state}`\n  - `{:stop, reason, state}`","ref":"WebsockexNova.Behaviors.ConnectionHandler.html#c:handle_connect/2"},{"type":"callback","title":"WebsockexNova.Behaviors.ConnectionHandler.handle_disconnect/2","doc":"Handle disconnect event.\nReturns:\n  - `{:ok, state}`\n  - `{:stop, reason, state}`","ref":"WebsockexNova.Behaviors.ConnectionHandler.html#c:handle_disconnect/2"},{"type":"callback","title":"WebsockexNova.Behaviors.ConnectionHandler.handle_frame/3","doc":"Handle a received WebSocket frame.\nReturns:\n  - `{:ok, state}`\n  - `{:reply, frame_type, binary(), state}`\n  - `{:close, code, reason, state}`\n  | {:stop, reason, state}","ref":"WebsockexNova.Behaviors.ConnectionHandler.html#c:handle_frame/3"},{"type":"callback","title":"WebsockexNova.Behaviors.ConnectionHandler.handle_timeout/1","doc":"Optional: handle connection timeout.\nReturns:\n  - `{:ok, state}`\n  - `{:stop, reason, state}`","ref":"WebsockexNova.Behaviors.ConnectionHandler.html#c:handle_timeout/1"},{"type":"callback","title":"WebsockexNova.Behaviors.ConnectionHandler.init/1","doc":"Initialize the handler's state.","ref":"WebsockexNova.Behaviors.ConnectionHandler.html#c:init/1"},{"type":"callback","title":"WebsockexNova.Behaviors.ConnectionHandler.ping/2","doc":"Ping the connection or stream.\nReturns:\n  - `{:ok, state}`\n  - `{:error, reason, state}`","ref":"WebsockexNova.Behaviors.ConnectionHandler.html#c:ping/2"},{"type":"type","title":"WebsockexNova.Behaviors.ConnectionHandler.state/0","doc":"Handler state","ref":"WebsockexNova.Behaviors.ConnectionHandler.html#t:state/0"},{"type":"callback","title":"WebsockexNova.Behaviors.ConnectionHandler.status/2","doc":"Query the status of the connection or stream.\nReturns:\n  - `{:ok, status, state}`\n  - `{:error, reason, state}`","ref":"WebsockexNova.Behaviors.ConnectionHandler.html#c:status/2"},{"type":"behaviour","title":"WebsockexNova.Behaviors.ErrorHandler","doc":"Behaviour for error handlers.\n\nThe ErrorHandler is the **single source of truth** for error handling and reconnection\npolicies in WebsockexNova. It centralizes all error-related decisions, including whether\nto retry operations, reconnect after failures, or stop the connection entirely.","ref":"WebsockexNova.Behaviors.ErrorHandler.html"},{"type":"behaviour","title":"Responsibilities - WebsockexNova.Behaviors.ErrorHandler","doc":"- The error handler is the **single source of truth** for reconnection policy (max attempts, backoff, etc.)\n- Tracks reconnection attempts and delays in its own state\n- All reconnection decisions are delegated here from the connection manager and connection handler\n- The transport adapter is responsible only for the mechanics of (re)connecting\n- Handles error classification for metrics and monitoring\n- Provides consistent error logging across the system","ref":"WebsockexNova.Behaviors.ErrorHandler.html#module-responsibilities"},{"type":"behaviour","title":"Callbacks - WebsockexNova.Behaviors.ErrorHandler","doc":"- `handle_error/3` — Handles errors and returns actions (`:ok`, `:stop`, `:retry`, `:reconnect`)\n- `should_reconnect?/3` — Returns `{true, delay}` or `{false, _}` based on error, attempt, and state\n- `increment_reconnect_attempts/1` — (Optional) Increments the attempt count in the handler state\n- `reset_reconnect_attempts/1` — (Optional) Resets the attempt count in the handler state\n- `log_error/3` — Logs errors with context\n- `classify_error/2` — (Optional) Classifies errors for reporting/metrics\n\nAll state is a map. All arguments and return values are explicit and documented.","ref":"WebsockexNova.Behaviors.ErrorHandler.html#module-callbacks"},{"type":"behaviour","title":"Implementation Example - WebsockexNova.Behaviors.ErrorHandler","doc":"defmodule MyApp.CustomErrorHandler do\n      @behaviour WebsockexNova.Behaviors.ErrorHandler\n      require Logger\n\n      @impl true\n      def handle_error(error, context, state) do\n        # Classify the error for appropriate handling\n        error_type = classify_error(error, context)\n        \n        case error_type do\n          :network_error ->\n            # Network errors should trigger reconnection\n            {:reconnect, state}\n          \n          :authentication_error ->\n            # Auth errors should stop the connection\n            {:stop, {:authentication_failed, error}, state}\n          \n          :rate_limit ->\n            # Rate limits should retry with exponential backoff\n            delay = calculate_backoff(state.retry_count || 0)\n            updated_state = Map.update(state, :retry_count, 1, &(&1 + 1))\n            {:retry, delay, updated_state}\n          \n          :timeout ->\n            # Timeouts might retry with shorter delay\n            {:retry, 1000, state}\n          \n          :protocol_error ->\n            # Protocol errors usually mean incompatibility\n            {:stop, {:protocol_error, error}, state}\n          \n          _ ->\n            # Unknown errors default to reconnection\n            {:reconnect, state}\n        end\n      end\n\n      @impl true\n      def should_reconnect?(error, attempt, state) do\n        # Define reconnection policy\n        max_attempts = state[:max_reconnect_attempts] || 10\n        base_delay = state[:base_reconnect_delay] || 1_000\n        max_delay = state[:max_reconnect_delay] || 60_000\n        \n        if attempt >= max_attempts do\n          # Too many attempts, stop reconnecting\n          {false, nil}\n        else\n          # Calculate exponential backoff with jitter\n          delay = calculate_exponential_backoff(attempt, base_delay, max_delay)\n          {true, delay}\n        end\n      end\n\n      @impl true\n      def increment_reconnect_attempts(state) do\n        Map.update(state, :reconnect_attempts, 1, &(&1 + 1))\n      end\n\n      @impl true\n      def reset_reconnect_attempts(state) do\n        state\n        |> Map.put(:reconnect_attempts, 0)\n        |> Map.put(:retry_count, 0)\n      end\n\n      @impl true\n      def log_error(error, context, state) do\n        error_type = classify_error(error, context)\n        attempts = Map.get(state, :reconnect_attempts, 0)\n        \n        Logger.error(\\\"\\\"\\\"\n        WebSocket error occurred:\n          Type: \\#{error_type}\n          Error: \\#{inspect(error)}\n          Context: \\#{inspect(context)}\n          Reconnection attempts: \\#{attempts}\n        \\\"\\\"\\\")\n      end\n\n      @impl true\n      def classify_error(error, context) do\n        cond do\n          # Network-related errors\n          match?({:error, :nxdomain}, error) -> :network_error\n          match?({:error, :econnrefused}, error) -> :network_error\n          match?({:error, :ehostunreach}, error) -> :network_error\n          match?({:error, :timeout}, error) -> :timeout\n          \n          # SSL/TLS errors\n          match?({:ssl_error, _}, error) -> :ssl_error\n          match?({:tls_alert, _}, error) -> :ssl_error\n          \n          # Authentication errors\n          match?({:error, {:auth_failed, _}}, error) -> :authentication_error\n          match?(%{\"error\" => \"unauthorized\"}, error) -> :authentication_error\n          \n          # Rate limiting\n          match?(%{\"error\" => \"rate_limit_exceeded\"}, error) -> :rate_limit\n          match?({:error, :too_many_requests}, error) -> :rate_limit\n          \n          # Protocol errors\n          match?({:error, {:invalid_message, _}}, error) -> :protocol_error\n          match?({:error, :invalid_frame}, error) -> :protocol_error\n          \n          # Connection state errors\n          is_map(context) and Map.get(context, :during) == \"handshake\" -> :handshake_error\n          is_map(context) and Map.get(context, :during) == \"upgrade\" -> :upgrade_error\n          \n          # Default classification\n          true -> :unknown_error\n        end\n      end\n\n      # Private helpers\n      \n      defp calculate_backoff(retry_count) do\n        # Simple exponential backoff: 1s, 2s, 4s, 8s, etc.\n        base_delay = 1_000\n        max_delay = 30_000\n        \n        delay = base_delay * :math.pow(2, retry_count)\n        min(round(delay), max_delay)\n      end\n\n      defp calculate_exponential_backoff(attempt, base_delay, max_delay) do\n        # Exponential backoff with jitter\n        delay = base_delay * :math.pow(2, attempt)\n        delay_with_jitter = delay * (0.5 + :rand.uniform())\n        \n        min(round(delay_with_jitter), max_delay)\n      end\n    end","ref":"WebsockexNova.Behaviors.ErrorHandler.html#module-implementation-example"},{"type":"behaviour","title":"Context Map Structure - WebsockexNova.Behaviors.ErrorHandler","doc":"The context map typically contains:\n- `:during` - Phase when error occurred (\"connect\", \"handshake\", \"upgrade\", \"message\", etc.)\n- `:operation` - Specific operation that failed\n- `:gun_pid` - Gun process PID if available\n- `:stream_ref` - WebSocket stream reference\n- `:conn_info` - Connection information map\n- `:metadata` - Additional error metadata","ref":"WebsockexNova.Behaviors.ErrorHandler.html#module-context-map-structure"},{"type":"behaviour","title":"Error Classification - WebsockexNova.Behaviors.ErrorHandler","doc":"Common error types to consider:\n- `:network_error` - Connection failures, DNS issues\n- `:timeout` - Operation timeouts\n- `:authentication_error` - Auth failures\n- `:rate_limit` - Rate limiting errors\n- `:protocol_error` - WebSocket protocol violations\n- `:ssl_error` - TLS/SSL errors\n- `:handshake_error` - WebSocket handshake failures\n- `:unknown_error` - Unclassified errors","ref":"WebsockexNova.Behaviors.ErrorHandler.html#module-error-classification"},{"type":"behaviour","title":"Reconnection Strategies - WebsockexNova.Behaviors.ErrorHandler","doc":"1. **Exponential Backoff**: Delay doubles with each attempt\n2. **Linear Backoff**: Fixed delay between attempts\n3. **Fibonacci Backoff**: Delays follow Fibonacci sequence\n4. **Jittered Backoff**: Add randomness to prevent thundering herd\n5. **Circuit Breaker**: Stop reconnecting after threshold","ref":"WebsockexNova.Behaviors.ErrorHandler.html#module-reconnection-strategies"},{"type":"behaviour","title":"Tips - WebsockexNova.Behaviors.ErrorHandler","doc":"1. Keep error classification comprehensive but simple\n2. Use exponential backoff with jitter for reconnection\n3. Set reasonable maximum reconnection attempts\n4. Log errors with sufficient context for debugging\n5. Consider different policies for different error types\n6. Reset attempt counters on successful reconnection\n7. Implement circuit breakers for persistent failures\n\n**Note**: The `:reconnect` return value from `handle_error/3` is deprecated.\nUse `should_reconnect?/3` for all reconnection policy decisions.\n\nSee `WebsockexNova.Defaults.DefaultErrorHandler` for a reference implementation.","ref":"WebsockexNova.Behaviors.ErrorHandler.html#module-tips"},{"type":"callback","title":"WebsockexNova.Behaviors.ErrorHandler.classify_error/2","doc":"Optional callback for classifying errors.\nReturns:\n  - The error category (any term, typically an atom like :transient or :critical)","ref":"WebsockexNova.Behaviors.ErrorHandler.html#c:classify_error/2"},{"type":"type","title":"WebsockexNova.Behaviors.ErrorHandler.context/0","doc":"Context information about the error","ref":"WebsockexNova.Behaviors.ErrorHandler.html#t:context/0"},{"type":"type","title":"WebsockexNova.Behaviors.ErrorHandler.delay/0","doc":"Delay in milliseconds for retry/reconnect operations","ref":"WebsockexNova.Behaviors.ErrorHandler.html#t:delay/0"},{"type":"type","title":"WebsockexNova.Behaviors.ErrorHandler.error/0","doc":"Error type or data","ref":"WebsockexNova.Behaviors.ErrorHandler.html#t:error/0"},{"type":"callback","title":"WebsockexNova.Behaviors.ErrorHandler.handle_error/3","doc":"Handle an error and determine the appropriate action.\nReturns:\n  - `{:ok, state}`\n  - `{:retry, delay, state}`\n  - `{:stop, reason, state}`\n  - `{:reconnect, state}` (DEPRECATED: use `should_reconnect?/3` for policy)","ref":"WebsockexNova.Behaviors.ErrorHandler.html#c:handle_error/3"},{"type":"callback","title":"WebsockexNova.Behaviors.ErrorHandler.increment_reconnect_attempts/1","doc":"Optional callback to increment the reconnection attempt count in the handler state.\nReturns the updated state.","ref":"WebsockexNova.Behaviors.ErrorHandler.html#c:increment_reconnect_attempts/1"},{"type":"callback","title":"WebsockexNova.Behaviors.ErrorHandler.log_error/3","doc":"Log an error with appropriate context.\nReturns:\n  - `:ok`","ref":"WebsockexNova.Behaviors.ErrorHandler.html#c:log_error/3"},{"type":"callback","title":"WebsockexNova.Behaviors.ErrorHandler.reset_reconnect_attempts/1","doc":"Optional callback to reset the reconnection attempt count in the handler state.\nReturns the updated state.","ref":"WebsockexNova.Behaviors.ErrorHandler.html#c:reset_reconnect_attempts/1"},{"type":"callback","title":"WebsockexNova.Behaviors.ErrorHandler.should_reconnect?/3","doc":"Determine if reconnection should be attempted.\nThis is the single source of truth for reconnection policy.\nReturns:\n  - `{true, delay}`\n  - `{false, _}`","ref":"WebsockexNova.Behaviors.ErrorHandler.html#c:should_reconnect?/3"},{"type":"type","title":"WebsockexNova.Behaviors.ErrorHandler.state/0","doc":"Handler state","ref":"WebsockexNova.Behaviors.ErrorHandler.html#t:state/0"},{"type":"behaviour","title":"WebsockexNova.Behaviors.LoggingHandler","doc":"Defines the behavior for standardized, configurable logging in WebsockexNova.\n\nThe LoggingHandler behavior enables pluggable, context-aware logging for connection, message, and error events. This allows applications to customize log levels, formats, and destinations, supporting both structured and unstructured logging.","ref":"WebsockexNova.Behaviors.LoggingHandler.html"},{"type":"behaviour","title":"Usage - WebsockexNova.Behaviors.LoggingHandler","doc":"All Gun pipeline modules, helpers, and test/mock handlers now use the logging handler for connection, message, and error events. If a logging handler is not present in the connection state, logging falls back to Elixir's Logger.","ref":"WebsockexNova.Behaviors.LoggingHandler.html#module-usage"},{"type":"behaviour","title":"Configuring a Custom Logging Handler - WebsockexNova.Behaviors.LoggingHandler","doc":"To use a custom logging handler, add it to your connection state:\n\n    state = %{logging_handler: MyApp.LoggingHandler, ...}\n\nYour handler module must implement the `WebsockexNova.Behaviors.LoggingHandler` behavior.","ref":"WebsockexNova.Behaviors.LoggingHandler.html#module-configuring-a-custom-logging-handler"},{"type":"behaviour","title":"Implementing the LoggingHandler Behavior - WebsockexNova.Behaviors.LoggingHandler","doc":"defmodule MyApp.LoggingHandler do\n      @behaviour WebsockexNova.Behaviors.LoggingHandler\n\n      @impl true\n      def log_connection_event(event, context, state) do\n        # Custom connection event logging\n        :ok\n      end\n\n      @impl true\n      def log_message_event(event, context, state) do\n        # Custom message event logging\n        :ok\n      end\n\n      @impl true\n      def log_error_event(event, context, state) do\n        # Custom error event logging\n        :ok\n      end\n    end","ref":"WebsockexNova.Behaviors.LoggingHandler.html#module-implementing-the-logginghandler-behavior"},{"type":"behaviour","title":"Example: Plugging in a Custom Handler - WebsockexNova.Behaviors.LoggingHandler","doc":"state = %{logging_handler: MyApp.LoggingHandler, ...}\n    # Pass this state to your connection pipeline","ref":"WebsockexNova.Behaviors.LoggingHandler.html#module-example-plugging-in-a-custom-handler"},{"type":"behaviour","title":"Example: Capturing Log Events in Tests - WebsockexNova.Behaviors.LoggingHandler","doc":"defmodule MyTest.LoggingHandler do\n      @behaviour WebsockexNova.Behaviors.LoggingHandler\n      def log_connection_event(event, context, state), do: send(self(), {:log, :connection, event, context}); :ok\n      def log_message_event(event, context, state), do: send(self(), {:log, :message, event, context}); :ok\n      def log_error_event(event, context, state), do: send(self(), {:log, :error, event, context}); :ok\n    end\n\n    # In your test setup:\n    state = %{logging_handler: MyTest.LoggingHandler, ...}","ref":"WebsockexNova.Behaviors.LoggingHandler.html#module-example-capturing-log-events-in-tests"},{"type":"behaviour","title":"Callbacks - WebsockexNova.Behaviors.LoggingHandler","doc":"* `log_connection_event/3` - Log a connection lifecycle event\n* `log_message_event/3` - Log a message send/receive event\n* `log_error_event/3` - Log an error event with context","ref":"WebsockexNova.Behaviors.LoggingHandler.html#module-callbacks"},{"type":"behaviour","title":"Configuration - WebsockexNova.Behaviors.LoggingHandler","doc":"Implementations should support configuration for log level (e.g., :debug, :info, :warn, :error) and log format (e.g., :plain, :json). These can be provided via handler state or options.","ref":"WebsockexNova.Behaviors.LoggingHandler.html#module-configuration"},{"type":"behaviour","title":"Example - WebsockexNova.Behaviors.LoggingHandler","doc":"```elixir\ndefmodule MyApp.LoggingHandler do\n  @behaviour WebsockexNova.Behaviors.LoggingHandler\n\n  @impl true\n  def log_connection_event(event, context, state) do\n    # Custom logging logic\n    :ok\n  end\n\n  @impl true\n  def log_message_event(event, context, state) do\n    # Custom logging logic\n    :ok\n  end\n\n  @impl true\n  def log_error_event(event, context, state) do\n    # Custom logging logic\n    :ok\n  end\nend\n```","ref":"WebsockexNova.Behaviors.LoggingHandler.html#module-example"},{"type":"type","title":"WebsockexNova.Behaviors.LoggingHandler.context/0","doc":"Context information about the event (e.g., connection details, message payload, error info).","ref":"WebsockexNova.Behaviors.LoggingHandler.html#t:context/0"},{"type":"type","title":"WebsockexNova.Behaviors.LoggingHandler.event/0","doc":"Event type for connection, message, or error events.","ref":"WebsockexNova.Behaviors.LoggingHandler.html#t:event/0"},{"type":"callback","title":"WebsockexNova.Behaviors.LoggingHandler.log_connection_event/3","doc":"Log a connection lifecycle event (e.g., connect, disconnect, reconnect).\nReturns:\n  - `:ok`","ref":"WebsockexNova.Behaviors.LoggingHandler.html#c:log_connection_event/3"},{"type":"callback","title":"WebsockexNova.Behaviors.LoggingHandler.log_error_event/3","doc":"Log an error event with context.\nReturns:\n  - `:ok`","ref":"WebsockexNova.Behaviors.LoggingHandler.html#c:log_error_event/3"},{"type":"type","title":"WebsockexNova.Behaviors.LoggingHandler.log_format/0","doc":"Log format (:plain, :json, or custom atom).","ref":"WebsockexNova.Behaviors.LoggingHandler.html#t:log_format/0"},{"type":"type","title":"WebsockexNova.Behaviors.LoggingHandler.log_level/0","doc":"Log level (:debug, :info, :warn, :error).","ref":"WebsockexNova.Behaviors.LoggingHandler.html#t:log_level/0"},{"type":"callback","title":"WebsockexNova.Behaviors.LoggingHandler.log_message_event/3","doc":"Log a message event (send/receive).\nReturns:\n  - `:ok`","ref":"WebsockexNova.Behaviors.LoggingHandler.html#c:log_message_event/3"},{"type":"type","title":"WebsockexNova.Behaviors.LoggingHandler.state/0","doc":"Handler state","ref":"WebsockexNova.Behaviors.LoggingHandler.html#t:state/0"},{"type":"behaviour","title":"WebsockexNova.Behaviors.MessageHandler","doc":"Defines the behavior for handling WebSocket messages.\n\nThe MessageHandler behavior is part of WebsockexNova's thin adapter architecture,\nallowing client applications to customize message processing while maintaining a\nclean separation from transport concerns.","ref":"WebsockexNova.Behaviors.MessageHandler.html"},{"type":"behaviour","title":"Binary Data Support - WebsockexNova.Behaviors.MessageHandler","doc":"WebsockexNova supports both structured map data and raw binary data throughout\nthe message handling pipeline:\n\n- `handle_message/2` can receive and process both maps and binary data\n- `encode_message/2` can return either maps or binary payloads\n- Binary data is preserved in its original form when appropriate\n- Implementations can choose how to handle different data formats\n\nThis flexibility allows WebsockexNova to work with any WebSocket protocol,\nwhether it uses structured data like JSON or raw binary formats.","ref":"WebsockexNova.Behaviors.MessageHandler.html#module-binary-data-support"},{"type":"behaviour","title":"Thin Adapter Pattern - WebsockexNova.Behaviors.MessageHandler","doc":"As part of the thin adapter architecture:\n\n1. This behavior focuses exclusively on message processing logic\n2. The connection layer delegates message handling responsibilities to implementations\n3. Your implementation can use domain-specific message types and validation rules\n4. The adapter handles encoding/decoding between your domain types and the wire format","ref":"WebsockexNova.Behaviors.MessageHandler.html#module-thin-adapter-pattern"},{"type":"behaviour","title":"Delegation Flow - WebsockexNova.Behaviors.MessageHandler","doc":"The message handling delegation flow works as follows:\n\n1. Raw frames are received by the connection handler\n2. Text/binary frames are passed to your `handle_message/2` callback\n3. Your implementation processes the message according to your application's needs\n4. If you need to send a response, the adapter handles the encoding back to wire format","ref":"WebsockexNova.Behaviors.MessageHandler.html#module-delegation-flow"},{"type":"behaviour","title":"Implementation Example - WebsockexNova.Behaviors.MessageHandler","doc":"```elixir\ndefmodule MyApp.ChatMessageHandler do\n  @behaviour WebsockexNova.Behaviors.MessageHandler\n\n  @impl true\n  def handle_message(%{\"type\" => \"chat_message\", \"text\" => text, \"user\" => user}, state) do\n    # Process a chat message\n    IO.puts(\"\\#{user}: \\#{text}\")\n\n    # Send an acknowledgment\n    {:reply, {:ack, %{message_id: state.last_message_id}}, state}\n  end\n\n  @impl true\n  def handle_message(%{\"type\" => \"presence_update\", \"user\" => user, \"status\" => status}, state) do\n    # Process a presence update\n    new_state = update_in(state.users[user], fn _ -> status end)\n    {:ok, new_state}\n  end\n\n  @impl true\n  def validate_message(message) when is_map(message) and map_size(message) > 0 do\n    # Validate that message has a type field\n    case Map.has_key?(message, \"type\") do\n      true -> {:ok, message}\n      false -> {:error, :missing_type_field, message}\n    end\n  end\n\n  @impl true\n  def validate_message(message) do\n    {:error, :invalid_message_format, message}\n  end\n\n  @impl true\n  def message_type(%{\"type\" => type}) when is_binary(type) do\n    String.to_atom(type)\n  end\n\n  @impl true\n  def message_type(_message) do\n    :unknown\n  end\n\n  @impl true\n  def encode_message({:ack, %{message_id: id}}, _state) do\n    json = Jason.encode!(%{type: \"ack\", message_id: id})\n    {:ok, :text, json}\n  end\n\n  @impl true\n  def encode_message({:error, reason}, _state) do\n    json = Jason.encode!(%{type: \"error\", reason: reason})\n    {:ok, :text, json}\n  end\nend\n```","ref":"WebsockexNova.Behaviors.MessageHandler.html#module-implementation-example"},{"type":"behaviour","title":"Callbacks - WebsockexNova.Behaviors.MessageHandler","doc":"* `message_init/1` - Initialize the handler's state\n* `handle_message/2` - Process an incoming message\n* `validate_message/1` - Validate message format and content\n* `message_type/1` - Extract or determine the message type\n* `encode_message/2` - Encode a message for sending","ref":"WebsockexNova.Behaviors.MessageHandler.html#module-callbacks"},{"type":"callback","title":"WebsockexNova.Behaviors.MessageHandler.encode_message/2","doc":"Encode a message for sending.\n\nCalled to convert a message into a WebSocket frame.","ref":"WebsockexNova.Behaviors.MessageHandler.html#c:encode_message/2"},{"type":"callback","title":"Parameters - WebsockexNova.Behaviors.MessageHandler.encode_message/2","doc":"* `message_type` - The type of message to encode\n* `state` - Current handler state","ref":"WebsockexNova.Behaviors.MessageHandler.html#c:encode_message/2-parameters"},{"type":"callback","title":"Returns - WebsockexNova.Behaviors.MessageHandler.encode_message/2","doc":"* `{:ok, frame_type, data}` - Successfully encoded message\n  * `frame_type` - The WebSocket frame type (:text, :binary, etc.)\n  * `data` - The encoded message data as binary\n* `{:error, reason}` - Failed to encode message\n\nNote: When returning binary data, you may use either:\n* `{:ok, :binary, my_binary_data}` for explicit binary frames\n* `{:ok, :text, my_text_data}` for text frames","ref":"WebsockexNova.Behaviors.MessageHandler.html#c:encode_message/2-returns"},{"type":"type","title":"WebsockexNova.Behaviors.MessageHandler.encode_return/0","doc":"Return values for message encoding\n\n* `{:ok, frame_type, data}` - Successfully encoded message\n  * `frame_type` - The WebSocket frame type (:text, :binary, etc.)\n  * `data` - The encoded message data as binary\n* `{:error, reason}` - Failed to encode message\n\nNote: When returning binary data, you may use either:\n* `{:ok, :binary, my_binary_data}` for explicit binary frames\n* `{:ok, :text, my_text_data}` for text frames","ref":"WebsockexNova.Behaviors.MessageHandler.html#t:encode_return/0"},{"type":"type","title":"WebsockexNova.Behaviors.MessageHandler.frame_type/0","doc":"Frame type","ref":"WebsockexNova.Behaviors.MessageHandler.html#t:frame_type/0"},{"type":"callback","title":"WebsockexNova.Behaviors.MessageHandler.handle_message/2","doc":"Process an incoming message.\n\nCalled when a message is received from the server.","ref":"WebsockexNova.Behaviors.MessageHandler.html#c:handle_message/2"},{"type":"callback","title":"Parameters - WebsockexNova.Behaviors.MessageHandler.handle_message/2","doc":"* `message` - The parsed message (typically a map from decoded JSON)\n* `state` - Current handler state","ref":"WebsockexNova.Behaviors.MessageHandler.html#c:handle_message/2-parameters"},{"type":"callback","title":"Returns - WebsockexNova.Behaviors.MessageHandler.handle_message/2","doc":"* `{:ok, new_state}` - Continue with the updated state\n* `{:reply, message_type, new_state}` - Send a message and continue\n* `{:reply_many, [message_type], new_state}` - Send multiple messages\n* `{:close, code, reason, new_state}` - Close the connection\n* `{:error, reason, new_state}` - Error occurred during processing","ref":"WebsockexNova.Behaviors.MessageHandler.html#c:handle_message/2-returns"},{"type":"type","title":"WebsockexNova.Behaviors.MessageHandler.handler_return/0","doc":"Return values for message handling callbacks\n\n* `{:ok, new_state}` - Continue with the updated state\n* `{:reply, message_type, new_state}` - Send a message and continue\n* `{:reply_many, [message_type], new_state}` - Send multiple messages\n* `{:close, code, reason, new_state}` - Close the connection\n* `{:error, reason, new_state}` - Error occurred during processing","ref":"WebsockexNova.Behaviors.MessageHandler.html#t:handler_return/0"},{"type":"type","title":"WebsockexNova.Behaviors.MessageHandler.message/0","doc":"Message content","ref":"WebsockexNova.Behaviors.MessageHandler.html#t:message/0"},{"type":"callback","title":"WebsockexNova.Behaviors.MessageHandler.message_init/1","doc":"Initialize the handler's state.\n\nCalled when the message handler is started. The return value becomes the initial state.","ref":"WebsockexNova.Behaviors.MessageHandler.html#c:message_init/1"},{"type":"callback","title":"Parameters - WebsockexNova.Behaviors.MessageHandler.message_init/1","doc":"* `opts` - The options passed to the handler","ref":"WebsockexNova.Behaviors.MessageHandler.html#c:message_init/1-parameters"},{"type":"callback","title":"Returns - WebsockexNova.Behaviors.MessageHandler.message_init/1","doc":"* `{:ok, state}` - The initialized state\n* `{:error, reason}` - Initialization failed","ref":"WebsockexNova.Behaviors.MessageHandler.html#c:message_init/1-returns"},{"type":"type","title":"WebsockexNova.Behaviors.MessageHandler.message_type/0","doc":"Message type","ref":"WebsockexNova.Behaviors.MessageHandler.html#t:message_type/0"},{"type":"callback","title":"WebsockexNova.Behaviors.MessageHandler.message_type/1","doc":"Determine the type of a message.\n\nCalled to extract or determine the type/category of a message.","ref":"WebsockexNova.Behaviors.MessageHandler.html#c:message_type/1"},{"type":"callback","title":"Parameters - WebsockexNova.Behaviors.MessageHandler.message_type/1","doc":"* `message` - The message to analyze","ref":"WebsockexNova.Behaviors.MessageHandler.html#c:message_type/1-parameters"},{"type":"callback","title":"Returns - WebsockexNova.Behaviors.MessageHandler.message_type/1","doc":"* The message type (atom, string, or tuple)","ref":"WebsockexNova.Behaviors.MessageHandler.html#c:message_type/1-returns"},{"type":"type","title":"WebsockexNova.Behaviors.MessageHandler.state/0","doc":"Handler state","ref":"WebsockexNova.Behaviors.MessageHandler.html#t:state/0"},{"type":"callback","title":"WebsockexNova.Behaviors.MessageHandler.validate_message/1","doc":"Validate an incoming message.\n\nCalled to validate the format and content of a message.","ref":"WebsockexNova.Behaviors.MessageHandler.html#c:validate_message/1"},{"type":"callback","title":"Parameters - WebsockexNova.Behaviors.MessageHandler.validate_message/1","doc":"* `message` - The message to validate","ref":"WebsockexNova.Behaviors.MessageHandler.html#c:validate_message/1-parameters"},{"type":"callback","title":"Returns - WebsockexNova.Behaviors.MessageHandler.validate_message/1","doc":"* `{:ok, message}` - Message is valid, possibly normalized\n* `{:error, reason, message}` - Message is invalid with reason","ref":"WebsockexNova.Behaviors.MessageHandler.html#c:validate_message/1-returns"},{"type":"type","title":"WebsockexNova.Behaviors.MessageHandler.validate_return/0","doc":"Return values for message validation\n\n* `{:ok, message}` - Message is valid, possibly normalized\n* `{:error, reason, message}` - Message is invalid with reason","ref":"WebsockexNova.Behaviors.MessageHandler.html#t:validate_return/0"},{"type":"behaviour","title":"WebsockexNova.Behaviors.MetricsCollector","doc":"Behaviour for metrics collectors.\nAll state and context are maps. All arguments and return values are explicit and documented.\nSupports both stateless (3-arity, for telemetry) and stateful (4-arity, for handler API) callbacks.","ref":"WebsockexNova.Behaviors.MetricsCollector.html"},{"type":"type","title":"WebsockexNova.Behaviors.MetricsCollector.conn_state/0","doc":"Canonical connection state struct","ref":"WebsockexNova.Behaviors.MetricsCollector.html#t:conn_state/0"},{"type":"type","title":"WebsockexNova.Behaviors.MetricsCollector.event/0","doc":"Event type","ref":"WebsockexNova.Behaviors.MetricsCollector.html#t:event/0"},{"type":"callback","title":"WebsockexNova.Behaviors.MetricsCollector.handle_connection_event/3","doc":"Handles connection-related telemetry events (stateless, for telemetry).\nReturns:\n  - `:ok`","ref":"WebsockexNova.Behaviors.MetricsCollector.html#c:handle_connection_event/3"},{"type":"callback","title":"WebsockexNova.Behaviors.MetricsCollector.handle_connection_event/4","doc":"Handles connection-related events with canonical state struct (for handler API).\nReturns:\n  - `{:ok, conn_state}`","ref":"WebsockexNova.Behaviors.MetricsCollector.html#c:handle_connection_event/4"},{"type":"callback","title":"WebsockexNova.Behaviors.MetricsCollector.handle_error_event/3","doc":"Handles error-related telemetry events (stateless, for telemetry).\nReturns:\n  - `:ok`","ref":"WebsockexNova.Behaviors.MetricsCollector.html#c:handle_error_event/3"},{"type":"callback","title":"WebsockexNova.Behaviors.MetricsCollector.handle_error_event/4","doc":"Handles error-related events with canonical state struct (for handler API).\nReturns:\n  - `{:ok, conn_state}`","ref":"WebsockexNova.Behaviors.MetricsCollector.html#c:handle_error_event/4"},{"type":"callback","title":"WebsockexNova.Behaviors.MetricsCollector.handle_message_event/3","doc":"Handles message-related telemetry events (stateless, for telemetry).\nReturns:\n  - `:ok`","ref":"WebsockexNova.Behaviors.MetricsCollector.html#c:handle_message_event/3"},{"type":"callback","title":"WebsockexNova.Behaviors.MetricsCollector.handle_message_event/4","doc":"Handles message-related events with canonical state struct (for handler API).\nReturns:\n  - `{:ok, conn_state}`","ref":"WebsockexNova.Behaviors.MetricsCollector.html#c:handle_message_event/4"},{"type":"type","title":"WebsockexNova.Behaviors.MetricsCollector.measurements/0","doc":"Measurements map","ref":"WebsockexNova.Behaviors.MetricsCollector.html#t:measurements/0"},{"type":"type","title":"WebsockexNova.Behaviors.MetricsCollector.metadata/0","doc":"Metadata map","ref":"WebsockexNova.Behaviors.MetricsCollector.html#t:metadata/0"},{"type":"type","title":"WebsockexNova.Behaviors.MetricsCollector.metric_type/0","doc":"Metric type","ref":"WebsockexNova.Behaviors.MetricsCollector.html#t:metric_type/0"},{"type":"behaviour","title":"WebsockexNova.Behaviors.RateLimitHandler","doc":"Behaviour for rate limit handlers.\nAll state is a map. All arguments and return values are explicit and documented.","ref":"WebsockexNova.Behaviors.RateLimitHandler.html"},{"type":"callback","title":"WebsockexNova.Behaviors.RateLimitHandler.check_rate_limit/2","doc":"Check if a request can proceed based on current rate limits.\nReturns:\n  - `{:allow, state}`\n  - `{:queue, state}`\n  - `{:reject, reason, state}`","ref":"WebsockexNova.Behaviors.RateLimitHandler.html#c:check_rate_limit/2"},{"type":"callback","title":"WebsockexNova.Behaviors.RateLimitHandler.handle_tick/1","doc":"Process queued requests on a periodic tick.\nReturns:\n  - `{:ok, state}`\n  - `{:process, request, state}`","ref":"WebsockexNova.Behaviors.RateLimitHandler.html#c:handle_tick/1"},{"type":"callback","title":"WebsockexNova.Behaviors.RateLimitHandler.rate_limit_init/1","doc":"Initialize the rate limit handler's state.","ref":"WebsockexNova.Behaviors.RateLimitHandler.html#c:rate_limit_init/1"},{"type":"type","title":"WebsockexNova.Behaviors.RateLimitHandler.request/0","doc":"Request data","ref":"WebsockexNova.Behaviors.RateLimitHandler.html#t:request/0"},{"type":"type","title":"WebsockexNova.Behaviors.RateLimitHandler.state/0","doc":"Handler state","ref":"WebsockexNova.Behaviors.RateLimitHandler.html#t:state/0"},{"type":"behaviour","title":"WebsockexNova.Behaviors.SubscriptionHandler","doc":"Behaviour for managing WebSocket channel subscriptions and topics.\n\nThe SubscriptionHandler provides a consistent interface for managing subscriptions\nacross different WebSocket services. It handles subscription lifecycle, tracking,\nand response processing, allowing platform-specific implementations while\nmaintaining a standard API.","ref":"WebsockexNova.Behaviors.SubscriptionHandler.html"},{"type":"behaviour","title":"Architecture - WebsockexNova.Behaviors.SubscriptionHandler","doc":"SubscriptionHandler is responsible for:\n- Tracking active subscriptions by channel/topic and ID\n- Managing subscription lifecycle (subscribe, unsubscribe)\n- Processing subscription-related responses from the server\n- Building subscription and unsubscription messages\n- Handling subscription errors and recovery\n\nThe handler maintains its state as a map, typically containing subscription\nmappings and metadata.","ref":"WebsockexNova.Behaviors.SubscriptionHandler.html#module-architecture"},{"type":"behaviour","title":"Callback Flow - WebsockexNova.Behaviors.SubscriptionHandler","doc":"1. `subscription_init/1` - Initialize subscription tracking state\n2. `subscribe/3` - Track new subscription request\n3. `handle_subscription/4` - Format and send subscription message\n4. `handle_subscription_response/2` - Process server's subscription response\n5. `unsubscribe/2` - Track unsubscription request\n6. `handle_unsubscription/3` - Format and send unsubscription message\n7. `handle_unsubscription_response/2` - Process server's unsubscription response\n8. `handle_subscription_message/3` - Process incoming subscription data","ref":"WebsockexNova.Behaviors.SubscriptionHandler.html#module-callback-flow"},{"type":"behaviour","title":"Implementation Example - WebsockexNova.Behaviors.SubscriptionHandler","doc":"defmodule MyApp.CustomSubscriptionHandler do\n      @behaviour WebsockexNova.Behaviors.SubscriptionHandler\n      alias WebsockexNova.Message.SubscriptionManager\n      require Logger\n\n      @impl true\n      def subscription_init(opts) do\n        state = %{\n          subscriptions: %{},           # channel -> subscription_id mapping\n          subscription_manager: SubscriptionManager.new(),\n          pending_subscriptions: %{},   # track pending requests\n          subscription_timeout: opts[:subscription_timeout] || 5_000\n        }\n        {:ok, state}\n      end\n\n      @impl true\n      def subscribe(channel, params, state) do\n        # Generate unique subscription ID\n        subscription_id = generate_subscription_id(channel, params)\n        \n        # Track the subscription\n        updated_subs = Map.put(state.subscriptions, channel, subscription_id)\n        updated_pending = Map.put(state.pending_subscriptions, subscription_id, %{\n          channel: channel,\n          params: params,\n          timestamp: System.system_time(:millisecond)\n        })\n        \n        updated_state = state\n        |> Map.put(:subscriptions, updated_subs)\n        |> Map.put(:pending_subscriptions, updated_pending)\n        \n        {:ok, subscription_id, updated_state}\n      end\n\n      @impl true\n      def handle_subscription(channel, params, conn, state) do\n        # Format subscription message for this service\n        subscribe_msg = %{\n          type: \"subscribe\",\n          channel: channel,\n          params: params || %{},\n          id: Map.get(state.subscriptions, channel)\n        }\n        \n        case WebsockexNova.Client.send_json(conn, subscribe_msg) do\n          {:ok, _} ->\n            # Update subscription manager\n            manager = state.subscription_manager\n            updated_manager = SubscriptionManager.add_subscription(manager, channel, params)\n            updated_state = Map.put(state, :subscription_manager, updated_manager)\n            {:ok, %{}, updated_state}\n          \n          error ->\n            # Clean up failed subscription\n            updated_state = cleanup_failed_subscription(state, channel)\n            {:error, error, updated_state}\n        end\n      end\n\n      @impl true\n      def handle_subscription_response(response, state) do\n        # Process server's subscription confirmation\n        case response do\n          %{\"type\" => \"subscribed\", \"channel\" => channel, \"id\" => sub_id} ->\n            # Move from pending to active\n            updated_pending = Map.delete(state.pending_subscriptions, sub_id)\n            updated_state = Map.put(state, :pending_subscriptions, updated_pending)\n            {:ok, updated_state}\n          \n          %{\"type\" => \"subscription_error\", \"channel\" => channel, \"error\" => error_msg} ->\n            # Handle subscription failure\n            updated_state = cleanup_failed_subscription(state, channel)\n            Logger.error(\"Subscription failed for \\#{channel}: \\#{inspect(error_msg)}\")\n            {:ok, updated_state}\n          \n          _ ->\n            # Unknown response format\n            {:ok, state}\n        end\n      end\n\n      @impl true\n      def unsubscribe(subscription_id, state) do\n        # Find channel for this subscription ID\n        channel = find_channel_by_subscription_id(state.subscriptions, subscription_id)\n        \n        if channel do\n          updated_subs = Map.delete(state.subscriptions, channel)\n          updated_state = Map.put(state, :subscriptions, updated_subs)\n          {:ok, updated_state}\n        else\n          {:error, :subscription_not_found, state}\n        end\n      end\n\n      @impl true\n      def handle_unsubscription(subscription_id, conn, state) do\n        # Format unsubscription message\n        unsubscribe_msg = %{\n          type: \"unsubscribe\",\n          id: subscription_id\n        }\n        \n        case WebsockexNova.Client.send_json(conn, unsubscribe_msg) do\n          {:ok, _} ->\n            # Update subscription manager\n            channel = find_channel_by_subscription_id(state.subscriptions, subscription_id)\n            if channel do\n              manager = state.subscription_manager\n              updated_manager = SubscriptionManager.remove_subscription(manager, channel)\n              updated_state = Map.put(state, :subscription_manager, updated_manager)\n              {:ok, updated_state}\n            else\n              {:ok, state}\n            end\n          \n          error ->\n            {:error, error, state}\n        end\n      end\n\n      @impl true\n      def handle_unsubscription_response(response, state) do\n        # Process server's unsubscription confirmation\n        case response do\n          %{\"type\" => \"unsubscribed\", \"id\" => sub_id} ->\n            # Confirmation received, already cleaned up\n            {:ok, state}\n          \n          _ ->\n            {:ok, state}\n        end\n      end\n\n      @impl true\n      def handle_subscription_message(channel, message, state) do\n        # Process incoming data for a subscribed channel\n        # Could include filtering, transformation, or routing\n        processed_message = process_channel_message(channel, message)\n        {:ok, processed_message, state}\n      end\n\n      # Private helpers\n\n      defp generate_subscription_id(channel, params) do\n        # Generate deterministic ID based on channel and params\n        \"\\#{channel}_\\#{:erlang.phash2({channel, params})}\"\n      end\n\n      defp find_channel_by_subscription_id(subscriptions, sub_id) do\n        Enum.find_value(subscriptions, fn {channel, id} ->\n          if id == sub_id, do: channel\n        end)\n      end\n\n      defp cleanup_failed_subscription(state, channel) do\n        sub_id = Map.get(state.subscriptions, channel)\n        \n        state\n        |> Map.update(:subscriptions, %{}, &Map.delete(&1, channel))\n        |> Map.update(:pending_subscriptions, %{}, &Map.delete(&1, sub_id))\n      end\n\n      defp process_channel_message(channel, message) do\n        # Add channel metadata or transform message\n        Map.put(message, :_channel, channel)\n      end\n    end","ref":"WebsockexNova.Behaviors.SubscriptionHandler.html#module-implementation-example"},{"type":"behaviour","title":"Channel and Parameter Structures - WebsockexNova.Behaviors.SubscriptionHandler","doc":"- **Channel**: Typically a string identifying the data stream\n  - Examples: `\"ticker.BTC-USD\"`, `\"orderbook.ETH-USD\"`, `\"trades.*\"`\n  - Can be hierarchical: `\"market.spot.ticker\"`\n  - May support wildcards: `\"trades.*\"` or `\"market.*.ticker\"`\n\n- **Params**: Additional subscription options as a map\n  - Examples: `%{depth: 10}`, `%{interval: \"1m\"}`, `%{symbols: [\"BTC\", \"ETH\"]}`\n  - Service-specific: each platform defines its own parameter structure","ref":"WebsockexNova.Behaviors.SubscriptionHandler.html#module-channel-and-parameter-structures"},{"type":"behaviour","title":"Subscription Response Types - WebsockexNova.Behaviors.SubscriptionHandler","doc":"Common response patterns:\n- Confirmation: `%{\"type\" => \"subscribed\", \"channel\" => \"...\", \"id\" => \"...\"}`\n- Error: `%{\"type\" => \"error\", \"message\" => \"...\", \"channel\" => \"...\"}`\n- Data: `%{\"channel\" => \"...\", \"data\" => %{...}, \"timestamp\" => ...}`\n- Unsubscribe confirmation: `%{\"type\" => \"unsubscribed\", \"id\" => \"...\"}`","ref":"WebsockexNova.Behaviors.SubscriptionHandler.html#module-subscription-response-types"},{"type":"behaviour","title":"Integration with WebsockexNova.Client - WebsockexNova.Behaviors.SubscriptionHandler","doc":"The SubscriptionHandler works with Client functions:\n- `WebsockexNova.Client.subscribe/3` calls the handler's callbacks\n- `WebsockexNova.Client.unsubscribe/2` manages unsubscription\n- Messages are routed through `handle_subscription_message/3`","ref":"WebsockexNova.Behaviors.SubscriptionHandler.html#module-integration-with-websockexnova-client"},{"type":"behaviour","title":"Tips - WebsockexNova.Behaviors.SubscriptionHandler","doc":"1. Use a subscription manager (like `SubscriptionManager`) for complex tracking\n2. Implement timeout handling for pending subscriptions\n3. Consider subscription deduplication for identical requests\n4. Handle reconnection by resubscribing to all active channels\n5. Log subscription events for debugging\n6. Validate channel names and parameters before subscribing\n\nSee `WebsockexNova.Defaults.DefaultSubscriptionHandler` for a reference implementation.","ref":"WebsockexNova.Behaviors.SubscriptionHandler.html#module-tips"},{"type":"callback","title":"WebsockexNova.Behaviors.SubscriptionHandler.active_subscriptions/1","doc":"Get all active subscriptions.\nReturns:\n  - A map of subscription IDs to subscription details","ref":"WebsockexNova.Behaviors.SubscriptionHandler.html#c:active_subscriptions/1"},{"type":"type","title":"WebsockexNova.Behaviors.SubscriptionHandler.channel/0","doc":"Channel name or topic","ref":"WebsockexNova.Behaviors.SubscriptionHandler.html#t:channel/0"},{"type":"callback","title":"WebsockexNova.Behaviors.SubscriptionHandler.find_subscription_by_channel/2","doc":"Find a subscription ID by channel name.\nReturns:\n  - The subscription ID if found, nil otherwise","ref":"WebsockexNova.Behaviors.SubscriptionHandler.html#c:find_subscription_by_channel/2"},{"type":"callback","title":"WebsockexNova.Behaviors.SubscriptionHandler.handle_subscription_response/2","doc":"Process a subscription-related response.\nReturns:\n  - `{:ok, state}`\n  - `{:error, reason, state}`","ref":"WebsockexNova.Behaviors.SubscriptionHandler.html#c:handle_subscription_response/2"},{"type":"type","title":"WebsockexNova.Behaviors.SubscriptionHandler.params/0","doc":"Subscription parameters","ref":"WebsockexNova.Behaviors.SubscriptionHandler.html#t:params/0"},{"type":"type","title":"WebsockexNova.Behaviors.SubscriptionHandler.state/0","doc":"Handler state","ref":"WebsockexNova.Behaviors.SubscriptionHandler.html#t:state/0"},{"type":"callback","title":"WebsockexNova.Behaviors.SubscriptionHandler.subscribe/3","doc":"Track a new subscription to a channel or topic.\nReturns:\n  - `{:ok, subscription_id, state}`\n  - `{:error, reason, state}`","ref":"WebsockexNova.Behaviors.SubscriptionHandler.html#c:subscribe/3"},{"type":"type","title":"WebsockexNova.Behaviors.SubscriptionHandler.subscription_id/0","doc":"Subscription ID","ref":"WebsockexNova.Behaviors.SubscriptionHandler.html#t:subscription_id/0"},{"type":"callback","title":"WebsockexNova.Behaviors.SubscriptionHandler.subscription_init/1","doc":"Initialize the handler's state.","ref":"WebsockexNova.Behaviors.SubscriptionHandler.html#c:subscription_init/1"},{"type":"type","title":"WebsockexNova.Behaviors.SubscriptionHandler.subscription_response/0","doc":"Subscription response","ref":"WebsockexNova.Behaviors.SubscriptionHandler.html#t:subscription_response/0"},{"type":"callback","title":"WebsockexNova.Behaviors.SubscriptionHandler.unsubscribe/2","doc":"Remove an existing subscription.\nReturns:\n  - `{:ok, state}`\n  - `{:error, reason, state}`","ref":"WebsockexNova.Behaviors.SubscriptionHandler.html#c:unsubscribe/2"},{"type":"module","title":"WebsockexNova.Client","doc":"Main client API for WebsockexNova WebSocket connections.\n\nThis module provides a high-level, user-friendly API for interacting with WebSocket connections\nusing WebsockexNova. It delegates operations to the transport layer and adapter implementations\nwhile providing a consistent interface regardless of the underlying transport or adapter.","ref":"WebsockexNova.Client.html"},{"type":"module","title":"Usage - WebsockexNova.Client","doc":"```elixir\n# Connect to a WebSocket server using an adapter\n{:ok, conn} = WebsockexNova.Client.connect(MyApp.WebSocket.Adapter, %{\n  host: \"example.com\",\n  port: 443,\n  path: \"/ws\",\n  transport_opts: [transport: :tls]\n})\n\n# Send a JSON message\n{:ok, response} = WebsockexNova.Client.send_json(conn, %{type: \"ping\"})\n\n# Subscribe to a channel\n{:ok, subscription} = WebsockexNova.Client.subscribe(conn, \"market.updates\")\n\n# Authenticate with credentials\n{:ok, auth_result} = WebsockexNova.Client.authenticate(conn, %{api_key: \"key\", secret: \"secret\"})\n\n# Close the connection\n:ok = WebsockexNova.Client.close(conn)\n```","ref":"WebsockexNova.Client.html#module-usage"},{"type":"module","title":"Comprehensive Example for a Trading/Financial App - WebsockexNova.Client","doc":"```elixir\n{:ok, conn} = WebsockexNova.Client.connect(MyApp.WebSocket.Adapter, %{\n  host: System.get_env(\"EXCHANGE_HOST\") || \"api.exchange.com\",\n  port: 443,\n  path: \"/ws/v1\",\n  headers: [{\"X-API-KEY\", System.get_env(\"EXCHANGE_API_KEY\") || \"demo-key\"}],\n  timeout: 10_000,\n  transport_opts: %{transport: :tls},\n  protocols: [:http],\n  retry: 10,\n  backoff_type: :exponential,\n  base_backoff: 1_000,\n  ws_opts: %{},\n  # Rate limiting\n  rate_limit_handler: WebsockexNova.Defaults.DefaultRateLimitHandler,\n  rate_limit_opts: %{\n    capacity: 120,\n    refill_rate: 10,\n    refill_interval: 1_000,\n    queue_limit: 200,\n    cost_map: %{\n      subscription: 5,\n      auth: 10,\n      query: 1,\n      order: 10\n    }\n  },\n  # Logging\n  log_level: :info,\n  log_format: :plain,\n  # Metrics\n  metrics_collector: MyApp.CustomMetricsCollector,\n  # Authentication\n  credentials: %{\n    api_key: System.get_env(\"EXCHANGE_API_KEY\"),\n    secret: System.get_env(\"EXCHANGE_API_SECRET\")\n  },\n  # Handler overrides (optional)\n  auth_handler: MyApp.CustomAuthHandler,\n  message_handler: MyApp.CustomMessageHandler,\n  subscription_handler: MyApp.CustomSubscriptionHandler,\n  error_handler: MyApp.CustomErrorHandler,\n  logging_handler: MyApp.CustomLoggingHandler,\n  # Connection handler options\n  max_reconnect_attempts: 10,\n  ping_interval: 15_000,\n  auth_refresh_threshold: 120\n})\n```","ref":"WebsockexNova.Client.html#module-comprehensive-example-for-a-trading-financial-app"},{"type":"module","title":"Adapter Integration - WebsockexNova.Client","doc":"The client API works with any adapter that implements the required behaviors:\n\n- `WebsockexNova.Behaviors.ConnectionHandler`\n- `WebsockexNova.Behaviors.MessageHandler`\n- `WebsockexNova.Behaviors.SubscriptionHandler`\n- `WebsockexNova.Behaviors.AuthHandler`\n- `WebsockexNova.Behaviors.ErrorHandler`\n\nIf an adapter doesn't implement a behavior, the client falls back to using default implementations\nfrom the `WebsockexNova.Defaults` namespace.","ref":"WebsockexNova.Client.html#module-adapter-integration"},{"type":"module","title":"Custom Matcher Example - WebsockexNova.Client","doc":"You can pass a custom matcher function to filter or extract specific responses:\n\n    matcher = fn\n      {:websockex_nova, {:websocket_frame, _stream_ref, {:text, \"special\"}}} -> {:ok, \"special\"}\n      _ -> :skip\n    end\n\n    opts = %{matcher: matcher}\n    {:ok, response} = WebsockexNova.Client.send_text(conn, \"ignored\", opts)\n    # response == \"special\"\n\nThe matcher function receives each message and should return:\n- `{:ok, value}` to match and return the value\n- `:skip` to ignore and continue waiting","ref":"WebsockexNova.Client.html#module-custom-matcher-example"},{"type":"type","title":"WebsockexNova.Client.auth_options/0","doc":"Authentication options","ref":"WebsockexNova.Client.html#t:auth_options/0"},{"type":"type","title":"WebsockexNova.Client.auth_result/0","doc":"Authentication response","ref":"WebsockexNova.Client.html#t:auth_result/0"},{"type":"function","title":"WebsockexNova.Client.authenticate/3","doc":"Authenticates with the WebSocket server.","ref":"WebsockexNova.Client.html#authenticate/3"},{"type":"function","title":"Parameters - WebsockexNova.Client.authenticate/3","doc":"* `conn` - Client connection struct\n* `credentials` - Authentication credentials\n* `options` - Authentication options","ref":"WebsockexNova.Client.html#authenticate/3-parameters"},{"type":"function","title":"Options - WebsockexNova.Client.authenticate/3","doc":"* `:timeout` - Response timeout in milliseconds (default: 30,000)\n* `:matcher` - (optional) A function to match/filter responses. The function should accept a message and return `{:ok, response}` to match, or `:skip` to ignore and continue waiting. See module doc for examples.","ref":"WebsockexNova.Client.html#authenticate/3-options"},{"type":"function","title":"Returns - WebsockexNova.Client.authenticate/3","doc":"* `{:ok, conn, auth_result}` on success (with updated connection)\n* `{:error, reason}` on failure","ref":"WebsockexNova.Client.html#authenticate/3-returns"},{"type":"function","title":"WebsockexNova.Client.close/1","doc":"Closes the WebSocket connection.","ref":"WebsockexNova.Client.html#close/1"},{"type":"function","title":"Parameters - WebsockexNova.Client.close/1","doc":"* `conn` - Client connection struct","ref":"WebsockexNova.Client.html#close/1-parameters"},{"type":"function","title":"Returns - WebsockexNova.Client.close/1","doc":"* `:ok`","ref":"WebsockexNova.Client.html#close/1-returns"},{"type":"function","title":"WebsockexNova.Client.connect/2","doc":"Connects to a WebSocket server using the specified adapter.\n\nThis function initializes the adapter, retrieves connection information,\nestablishes a connection using the transport layer, and upgrades to WebSocket.","ref":"WebsockexNova.Client.html#connect/2"},{"type":"function","title":"Parameters - WebsockexNova.Client.connect/2","doc":"* `adapter` - Module implementing adapter behaviors\n* `options` - Connection options","ref":"WebsockexNova.Client.html#connect/2-parameters"},{"type":"function","title":"Options - WebsockexNova.Client.connect/2","doc":"* `:host` - Hostname or IP address of the server (required)\n* `:port` - Port number of the server (required)\n* `:path` - WebSocket endpoint path (required)\n* `:headers` - Additional headers for the upgrade request (optional)\n* `:transport_opts` - Transport-specific options as a map or keyword list (optional)\n* `:timeout` - Connection timeout in milliseconds (default: 30,000)","ref":"WebsockexNova.Client.html#connect/2-options"},{"type":"function","title":"Returns - WebsockexNova.Client.connect/2","doc":"* `{:ok, conn}` on success\n* `{:error, reason}` on failure","ref":"WebsockexNova.Client.html#connect/2-returns"},{"type":"type","title":"WebsockexNova.Client.connect_options/0","doc":"Connection configuration options. Must include at least :host, :port, and :path.\nMay include any additional keys required by handlers, adapters, or transports.","ref":"WebsockexNova.Client.html#t:connect_options/0"},{"type":"type","title":"Required keys - WebsockexNova.Client.connect_options/0","doc":"* :host - Hostname or IP address (string)\n  * :port - Port number (integer)\n  * :path - WebSocket endpoint path (string)","ref":"WebsockexNova.Client.html#t:connect_options/0-required-keys"},{"type":"type","title":"Common optional keys - WebsockexNova.Client.connect_options/0","doc":"* :headers - Additional headers (keyword list or map)\n  * :timeout - Connection timeout in ms (integer)\n  * :transport_opts - Transport-specific options (map or keyword list)\n    Examples: `%{verify: :verify_peer}` or `[verify: :verify_peer]`\n  * :protocols - List of protocols (e.g., [:http])\n  * :retry - Number of connection retries (integer or :infinity)\n  * :backoff_type - :linear | :exponential | :jittered\n  * :base_backoff - Initial backoff in ms (integer)\n  * :ws_opts - WebSocket-specific options (map)\n  * :rate_limit_handler - Module implementing RateLimitHandler\n  * :rate_limit_opts - Map of rate limiting options (see DefaultRateLimitHandler)\n  * :log_level - :debug | :info | :warn | :error\n  * :log_format - :plain | :json\n  * :metrics_collector - Module implementing MetricsCollector\n  * :auth_handler, :message_handler, :subscription_handler, :error_handler, :logging_handler - Custom handler modules\n  * :credentials - Map with :api_key/:secret or :token for authentication\n  * :max_reconnect_attempts, :reconnect_attempts, :ping_interval, :auth_refresh_threshold, etc.\n  * Any other keys required by your custom handlers or adapters\n\nAll additional keys are passed to the relevant handler, adapter, or transport module.","ref":"WebsockexNova.Client.html#t:connect_options/0-common-optional-keys"},{"type":"type","title":"WebsockexNova.Client.connection_result/0","doc":"Connection response","ref":"WebsockexNova.Client.html#t:connection_result/0"},{"type":"type","title":"WebsockexNova.Client.message_options/0","doc":"Message options","ref":"WebsockexNova.Client.html#t:message_options/0"},{"type":"type","title":"WebsockexNova.Client.message_result/0","doc":"Message response","ref":"WebsockexNova.Client.html#t:message_result/0"},{"type":"function","title":"WebsockexNova.Client.ping/2","doc":"Sends a ping message to the WebSocket server.","ref":"WebsockexNova.Client.html#ping/2"},{"type":"function","title":"Parameters - WebsockexNova.Client.ping/2","doc":"* `conn` - Client connection struct\n* `options` - Message options","ref":"WebsockexNova.Client.html#ping/2-parameters"},{"type":"function","title":"Options - WebsockexNova.Client.ping/2","doc":"* `:timeout` - Response timeout in milliseconds (default: 30,000)","ref":"WebsockexNova.Client.html#ping/2-options"},{"type":"function","title":"Returns - WebsockexNova.Client.ping/2","doc":"* `{:ok, :pong}` on success\n* `{:error, reason}` on failure","ref":"WebsockexNova.Client.html#ping/2-returns"},{"type":"function","title":"WebsockexNova.Client.register_callback/2","doc":"Registers a process to receive notifications from the connection.\n\nRegistered processes will receive messages when connection events occur:\n- `{:connection_down, protocol, reason}` - When connection is lost\n- `{:connection_reconnected, updated_conn}` - When connection is automatically reconnected\n                                              (the updated_conn has new transport_pid and stream_ref)\n- `{:ws_message, message}` - When a WebSocket message is received","ref":"WebsockexNova.Client.html#register_callback/2"},{"type":"function","title":"Parameters - WebsockexNova.Client.register_callback/2","doc":"* `conn` - Client connection struct\n* `pid` - Process ID to register","ref":"WebsockexNova.Client.html#register_callback/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Client.register_callback/2","doc":"* `{:ok, ClientConn.t()}` with updated connection\n* `{:error, reason}` if registration fails","ref":"WebsockexNova.Client.html#register_callback/2-returns"},{"type":"function","title":"WebsockexNova.Client.send_frame/2","doc":"Sends a raw WebSocket frame.","ref":"WebsockexNova.Client.html#send_frame/2"},{"type":"function","title":"Parameters - WebsockexNova.Client.send_frame/2","doc":"* `conn` - Client connection struct\n* `frame` - WebSocket frame to send","ref":"WebsockexNova.Client.html#send_frame/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Client.send_frame/2","doc":"* `:ok` on success\n* `{:error, reason}` on failure","ref":"WebsockexNova.Client.html#send_frame/2-returns"},{"type":"function","title":"WebsockexNova.Client.send_json/3","doc":"Sends a JSON message.","ref":"WebsockexNova.Client.html#send_json/3"},{"type":"function","title":"Parameters - WebsockexNova.Client.send_json/3","doc":"* `conn` - Client connection struct\n* `data` - Map to encode as JSON and send\n* `options` - Message options","ref":"WebsockexNova.Client.html#send_json/3-parameters"},{"type":"function","title":"Options - WebsockexNova.Client.send_json/3","doc":"* `:timeout` - Response timeout in milliseconds (default: 30,000)\n* `:matcher` - (optional) A function to match/filter responses. The function should accept a message and return `{:ok, response}` to match, or `:skip` to ignore and continue waiting. See module doc for examples.","ref":"WebsockexNova.Client.html#send_json/3-options"},{"type":"function","title":"Returns - WebsockexNova.Client.send_json/3","doc":"* `{:ok, response}` on success\n* `{:error, reason}` on failure","ref":"WebsockexNova.Client.html#send_json/3-returns"},{"type":"function","title":"WebsockexNova.Client.send_text/3","doc":"Sends a text message.","ref":"WebsockexNova.Client.html#send_text/3"},{"type":"function","title":"Parameters - WebsockexNova.Client.send_text/3","doc":"* `conn` - Client connection struct\n* `text` - Text message to send\n* `options` - Message options","ref":"WebsockexNova.Client.html#send_text/3-parameters"},{"type":"function","title":"Options - WebsockexNova.Client.send_text/3","doc":"* `:timeout` - Response timeout in milliseconds (default: 30,000)\n* `:matcher` - (optional) A function to match/filter responses. The function should accept a message and return `{:ok, response}` to match, or `:skip` to ignore and continue waiting. See module doc for examples.","ref":"WebsockexNova.Client.html#send_text/3-options"},{"type":"function","title":"Returns - WebsockexNova.Client.send_text/3","doc":"* `{:ok, response}` on success\n* `{:error, reason}` on failure","ref":"WebsockexNova.Client.html#send_text/3-returns"},{"type":"function","title":"WebsockexNova.Client.status/2","doc":"Gets the current connection status.","ref":"WebsockexNova.Client.html#status/2"},{"type":"function","title":"Parameters - WebsockexNova.Client.status/2","doc":"* `conn` - Client connection struct\n* `options` - Options","ref":"WebsockexNova.Client.html#status/2-parameters"},{"type":"function","title":"Options - WebsockexNova.Client.status/2","doc":"* `:timeout` - Request timeout in milliseconds (default: 30,000)","ref":"WebsockexNova.Client.html#status/2-options"},{"type":"function","title":"Returns - WebsockexNova.Client.status/2","doc":"* `{:ok, status}` on success\n* `{:error, reason}` on failure","ref":"WebsockexNova.Client.html#status/2-returns"},{"type":"type","title":"WebsockexNova.Client.status_result/0","doc":"Status response","ref":"WebsockexNova.Client.html#t:status_result/0"},{"type":"function","title":"WebsockexNova.Client.subscribe/3","doc":"Subscribes to a channel or topic.","ref":"WebsockexNova.Client.html#subscribe/3"},{"type":"function","title":"Parameters - WebsockexNova.Client.subscribe/3","doc":"* `conn` - Client connection struct\n* `channel` - Channel or topic to subscribe to\n* `options` - Subscription options","ref":"WebsockexNova.Client.html#subscribe/3-parameters"},{"type":"function","title":"Options - WebsockexNova.Client.subscribe/3","doc":"* `:timeout` - Response timeout in milliseconds (default: 30,000)\n* `:matcher` - (optional) A function to match/filter responses. The function should accept a message and return `{:ok, response}` to match, or `:skip` to ignore and continue waiting. See module doc for examples.","ref":"WebsockexNova.Client.html#subscribe/3-options"},{"type":"function","title":"Returns - WebsockexNova.Client.subscribe/3","doc":"* `{:ok, subscription}` on success\n* `{:error, reason}` on failure","ref":"WebsockexNova.Client.html#subscribe/3-returns"},{"type":"type","title":"WebsockexNova.Client.subscribe_options/0","doc":"Subscribe options","ref":"WebsockexNova.Client.html#t:subscribe_options/0"},{"type":"type","title":"WebsockexNova.Client.subscription_result/0","doc":"Subscription response","ref":"WebsockexNova.Client.html#t:subscription_result/0"},{"type":"function","title":"WebsockexNova.Client.unregister_callback/2","doc":"Unregisters a process from receiving notifications.","ref":"WebsockexNova.Client.html#unregister_callback/2"},{"type":"function","title":"Parameters - WebsockexNova.Client.unregister_callback/2","doc":"* `conn` - Client connection struct\n* `pid` - Process ID to unregister","ref":"WebsockexNova.Client.html#unregister_callback/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Client.unregister_callback/2","doc":"* `{:ok, ClientConn.t()}` with updated connection","ref":"WebsockexNova.Client.html#unregister_callback/2-returns"},{"type":"function","title":"WebsockexNova.Client.unsubscribe/3","doc":"Unsubscribes from a channel or topic.","ref":"WebsockexNova.Client.html#unsubscribe/3"},{"type":"function","title":"Parameters - WebsockexNova.Client.unsubscribe/3","doc":"* `conn` - Client connection struct\n* `channel` - Channel or topic to unsubscribe from\n* `options` - Subscription options","ref":"WebsockexNova.Client.html#unsubscribe/3-parameters"},{"type":"function","title":"Options - WebsockexNova.Client.unsubscribe/3","doc":"* `:timeout` - Response timeout in milliseconds (default: 30,000)\n* `:matcher` - (optional) A function to match/filter responses. The function should accept a message and return `{:ok, response}` to match, or `:skip` to ignore and continue waiting. See module doc for examples.","ref":"WebsockexNova.Client.html#unsubscribe/3-options"},{"type":"function","title":"Returns - WebsockexNova.Client.unsubscribe/3","doc":"* `{:ok, result}` on success\n* `{:error, reason}` on failure","ref":"WebsockexNova.Client.html#unsubscribe/3-returns"},{"type":"module","title":"WebsockexNova.Client.Handlers","doc":"Utility functions for registering various handler behaviors with WebsockexNova connections.\n\nThis module provides helper functions to set up behavior handlers for WebSocket connections,\nensuring that appropriate defaults are used when not explicitly provided.","ref":"WebsockexNova.Client.Handlers.html"},{"type":"function","title":"WebsockexNova.Client.Handlers.configure_handlers/2","doc":"Configures handlers for a WebSocket connection based on the adapter module.\n\nThis function examines the adapter module to determine which behaviors it implements,\nand configures the connection options accordingly. For any behavior not implemented\nby the adapter, it falls back to the default implementation.","ref":"WebsockexNova.Client.Handlers.html#configure_handlers/2"},{"type":"function","title":"Parameters - WebsockexNova.Client.Handlers.configure_handlers/2","doc":"* `adapter` - Module implementing one or more WebsockexNova behaviors\n* `options` - Connection options (may already contain handler configurations)","ref":"WebsockexNova.Client.Handlers.html#configure_handlers/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Client.Handlers.configure_handlers/2","doc":"* Updated options map with all handler configurations","ref":"WebsockexNova.Client.Handlers.html#configure_handlers/2-returns"},{"type":"module","title":"WebsockexNova.ClientConn","doc":"Canonical state for a WebSocket client connection.\nAll core application/session state is explicit and top-level.\nHandler/feature-specific state is namespaced in maps (e.g., :rate_limit, :logging, :metrics).\nAdapter-specific state is kept in :adapter_state. :extras is for extensibility.\n\nThis struct contains all the information needed to interact with a WebSocket connection,\nincluding the transport layer implementation, transport process, stream reference,\nadapter module, and adapter state.","ref":"WebsockexNova.ClientConn.html"},{"type":"module","title":"Fields - WebsockexNova.ClientConn","doc":"* `:transport` - The transport module implementing `WebsockexNova.Transport`\n  * `:transport_pid` - PID of the transport process\n  * `:stream_ref` - WebSocket stream reference\n  * `:adapter` - Adapter module implementing various behaviors\n  * `:adapter_state` - State maintained by the adapter (stores auth status, auth tokens, credentials, subscriptions, etc.)\n  * `:callback_pids` - List of PIDs registered to receive event notifications\n  * `:connection_info` - Connection information and initial configuration\n  * `:connection_id` - Stable identifier that persists across reconnections\n  * `:rate_limit` - Rate limit configuration\n  * `:logging` - Logging configuration\n  * `:metrics` - Metrics configuration\n  * `:reconnection` - Reconnection configuration map for error handler\n  * `:connection_handler_settings` - State specific to the connection handler\n  * `:auth_handler_settings` - State specific to the auth handler\n  * `:subscription_handler_settings` - State specific to the subscription handler\n  * `:error_handler_settings` - State specific to the error handler\n  * `:message_handler_settings` - State specific to the message handler\n  * `:extras` - Extensible/optional state","ref":"WebsockexNova.ClientConn.html#module-fields"},{"type":"type","title":"WebsockexNova.ClientConn.adapter/0","doc":"Adapter module implementing behaviors","ref":"WebsockexNova.ClientConn.html#t:adapter/0"},{"type":"function","title":"WebsockexNova.ClientConn.fetch/2","doc":"Implements the Access behaviour to enable bracket access (`conn[:field]`).\n\nAllows retrieving fields from the ClientConn struct using Access syntax:","ref":"WebsockexNova.ClientConn.html#fetch/2"},{"type":"function","title":"Examples - WebsockexNova.ClientConn.fetch/2","doc":"conn[:adapter]\n    Access.get(conn, :connection_info)\n    conn[\"adapter\"]\n    Access.get(conn, \"connection_info\")","ref":"WebsockexNova.ClientConn.html#fetch/2-examples"},{"type":"function","title":"Parameters - WebsockexNova.ClientConn.fetch/2","doc":"- conn: The ClientConn struct\n  - key: The field name to access (atom or string)","ref":"WebsockexNova.ClientConn.html#fetch/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.ClientConn.fetch/2","doc":"- `{:ok, value}` if the key exists\n  - `:error` if the key doesn't exist","ref":"WebsockexNova.ClientConn.html#fetch/2-returns"},{"type":"function","title":"WebsockexNova.ClientConn.get_and_update/3","doc":"Implements the Access behaviour for updating ClientConn fields.\n\nThis enables functions like `Access.get_and_update/3` to work with ClientConn.","ref":"WebsockexNova.ClientConn.html#get_and_update/3"},{"type":"function","title":"Examples - WebsockexNova.ClientConn.get_and_update/3","doc":"{old_value, updated_conn} = Access.get_and_update(conn, :adapter_state, fn current ->\n      {current, Map.put(current, :new_key, :new_value)}\n    end)\n    \n    {old_value, updated_conn} = Access.get_and_update(conn, \"adapter_state\", fn current ->\n      {current, Map.put(current, :new_key, :new_value)}\n    end)","ref":"WebsockexNova.ClientConn.html#get_and_update/3-examples"},{"type":"function","title":"Parameters - WebsockexNova.ClientConn.get_and_update/3","doc":"- conn: The ClientConn struct\n  - key: The field name to update (atom or string)\n  - function: Function that transforms the current value","ref":"WebsockexNova.ClientConn.html#get_and_update/3-parameters"},{"type":"function","title":"Returns - WebsockexNova.ClientConn.get_and_update/3","doc":"- {get_value, updated_conn}","ref":"WebsockexNova.ClientConn.html#get_and_update/3-returns"},{"type":"function","title":"WebsockexNova.ClientConn.get_current_stream_ref/1","doc":"Get the current stream_ref for a connection.\n\nThis is used alongside get_current_transport_pid to ensure operations\nuse the current stream_ref, which may have changed after reconnection.","ref":"WebsockexNova.ClientConn.html#get_current_stream_ref/1"},{"type":"function","title":"Parameters - WebsockexNova.ClientConn.get_current_stream_ref/1","doc":"- conn: The ClientConn struct","ref":"WebsockexNova.ClientConn.html#get_current_stream_ref/1-parameters"},{"type":"function","title":"Returns - WebsockexNova.ClientConn.get_current_stream_ref/1","doc":"- stream_ref: The current stream reference","ref":"WebsockexNova.ClientConn.html#get_current_stream_ref/1-returns"},{"type":"function","title":"WebsockexNova.ClientConn.get_current_transport_pid/1","doc":"Get the current transport_pid for a connection.\n\nThis function checks the ConnectionRegistry first using the connection_id.\nIf found, it returns the current transport PID. This ensures operations\nwork even after reconnection when the transport_pid might have changed.\n\nIf the lookup fails, it falls back to the PID stored in the struct.","ref":"WebsockexNova.ClientConn.html#get_current_transport_pid/1"},{"type":"function","title":"Parameters - WebsockexNova.ClientConn.get_current_transport_pid/1","doc":"- conn: The ClientConn struct","ref":"WebsockexNova.ClientConn.html#get_current_transport_pid/1-parameters"},{"type":"function","title":"Returns - WebsockexNova.ClientConn.get_current_transport_pid/1","doc":"- pid: The current transport process PID","ref":"WebsockexNova.ClientConn.html#get_current_transport_pid/1-returns"},{"type":"function","title":"WebsockexNova.ClientConn.pop/2","doc":"Implements the Access behaviour to pop values from ClientConn fields.\n\nSince ClientConn is a struct with fixed fields, actual removal is not supported.\nFor map-type fields, this can clear the value by setting it to an empty map.","ref":"WebsockexNova.ClientConn.html#pop/2"},{"type":"function","title":"Examples - WebsockexNova.ClientConn.pop/2","doc":"{value, updated_conn} = Access.pop(conn, :extras)\n    {value, updated_conn} = Access.pop(conn, \"extras\")","ref":"WebsockexNova.ClientConn.html#pop/2-examples"},{"type":"function","title":"Parameters - WebsockexNova.ClientConn.pop/2","doc":"- conn: The ClientConn struct\n  - key: The field name to pop (atom or string)","ref":"WebsockexNova.ClientConn.html#pop/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.ClientConn.pop/2","doc":"- {current_value, updated_conn}","ref":"WebsockexNova.ClientConn.html#pop/2-returns"},{"type":"type","title":"WebsockexNova.ClientConn.stream_ref/0","doc":"WebSocket stream reference","ref":"WebsockexNova.ClientConn.html#t:stream_ref/0"},{"type":"type","title":"WebsockexNova.ClientConn.t/0","doc":"Client connection structure (canonical state)","ref":"WebsockexNova.ClientConn.html#t:t/0"},{"type":"type","title":"WebsockexNova.ClientConn.transport/0","doc":"WebSocket transport module","ref":"WebsockexNova.ClientConn.html#t:transport/0"},{"type":"module","title":"WebsockexNova.ClientMacro","doc":"Macro for building WebsockexNova client modules with minimal boilerplate.\n\nThis macro provides a convenient way to create domain-specific WebSocket client modules\nthat wrap the generic WebsockexNova.Client with service-specific functionality.","ref":"WebsockexNova.ClientMacro.html"},{"type":"module","title":"Basic Usage - WebsockexNova.ClientMacro","doc":"defmodule MyApp.MyClient do\n      use WebsockexNova.ClientMacro, adapter: MyApp.MyAdapter\n\n      # Add domain-specific methods:\n      def subscribe_to_custom_channel(conn, instrument_id, opts \\ nil) do\n        channel = \"custom.#{instrument_id}.events\"\n        subscribe(conn, channel, opts)\n      end\n    end","ref":"WebsockexNova.ClientMacro.html#module-basic-usage"},{"type":"module","title":"What this macro does - WebsockexNova.ClientMacro","doc":"- Injects all common client functions (connect, authenticate, send_json, subscribe, etc.)\n- Configures the client to use your specified adapter\n- Provides default connection options that can be overridden\n- Enables dependency injection for testing via application config\n- Allows you to add domain-specific helper methods","ref":"WebsockexNova.ClientMacro.html#module-what-this-macro-does"},{"type":"module","title":"Options - WebsockexNova.ClientMacro","doc":"- `:adapter` (required) - The adapter module to use for connections\n- `:default_options` (optional) - Default connection options as a map","ref":"WebsockexNova.ClientMacro.html#module-options"},{"type":"module","title":"Examples - WebsockexNova.ClientMacro","doc":"","ref":"WebsockexNova.ClientMacro.html#module-examples"},{"type":"module","title":"Trading client with custom methods - WebsockexNova.ClientMacro","doc":"defmodule MyApp.TradingClient do\n      use WebsockexNova.ClientMacro, \n        adapter: MyApp.TradingAdapter,\n        default_options: %{\n          heartbeat_interval: 10_000,\n          reconnect_delay: 1_000\n        }\n\n      def place_order(conn, order_params) do\n        send_json(conn, %{\n          method: \"place_order\",\n          params: order_params\n        })\n      end\n\n      def subscribe_to_orderbook(conn, symbol, depth \\ 10) do\n        subscribe(conn, \"orderbook.#{symbol}\", %{depth: depth})\n      end\n\n      def subscribe_to_trades(conn, symbol) do\n        subscribe(conn, \"trades.#{symbol}\", %{})\n      end\n    end\n\n    # Usage:\n    {:ok, conn} = MyApp.TradingClient.connect()\n    {:ok, _} = MyApp.TradingClient.authenticate(conn, %{api_key: \"key\", api_secret: \"secret\"})\n    {:ok, _} = MyApp.TradingClient.subscribe_to_orderbook(conn, \"BTC-USD\")","ref":"WebsockexNova.ClientMacro.html#module-trading-client-with-custom-methods"},{"type":"module","title":"Chat client with typed message sending - WebsockexNova.ClientMacro","doc":"defmodule MyApp.ChatClient do\n      use WebsockexNova.ClientMacro,\n        adapter: MyApp.ChatAdapter\n\n      def send_chat_message(conn, room_id, message) do\n        send_json(conn, %{\n          type: \"message\",\n          room: room_id,\n          text: message,\n          timestamp: DateTime.utc_now()\n        })\n      end\n\n      def join_room(conn, room_id) do\n        send_json(conn, %{type: \"join\", room: room_id})\n      end\n\n      def leave_room(conn, room_id) do\n        send_json(conn, %{type: \"leave\", room: room_id})\n      end\n\n      def set_typing_status(conn, room_id, is_typing) do\n        send_json(conn, %{\n          type: \"typing\",\n          room: room_id,\n          typing: is_typing\n        })\n      end\n    end","ref":"WebsockexNova.ClientMacro.html#module-chat-client-with-typed-message-sending"},{"type":"module","title":"IoT client with device management - WebsockexNova.ClientMacro","doc":"defmodule MyApp.IoTClient do\n      use WebsockexNova.ClientMacro,\n        adapter: MyApp.IoTAdapter,\n        default_options: %{\n          transport: :tcp,\n          port: 8883,\n          protocol: \"mqtt-websocket\"\n        }\n\n      def register_device(conn, device_id, metadata) do\n        send_json(conn, %{\n          action: \"register\",\n          device_id: device_id,\n          metadata: metadata\n        })\n      end\n\n      def send_telemetry(conn, device_id, data) do\n        send_json(conn, %{\n          action: \"telemetry\",\n          device_id: device_id,\n          data: data,\n          timestamp: System.system_time(:second)\n        })\n      end\n\n      def update_device_shadow(conn, device_id, desired_state) do\n        send_json(conn, %{\n          action: \"update_shadow\",\n          device_id: device_id,\n          state: %{desired: desired_state}\n        })\n      end\n    end","ref":"WebsockexNova.ClientMacro.html#module-iot-client-with-device-management"},{"type":"module","title":"Testing Support - WebsockexNova.ClientMacro","doc":"The macro supports dependency injection for testing by allowing you to configure\nthe underlying client module:\n\n    # In config/test.exs\n    config :websockex_nova, :client_module, MyApp.MockClient\n\n    # In your test\n    defmodule MyApp.TradingClientTest do\n      use ExUnit.Case\n\n      test \"places order successfully\" do\n        {:ok, conn} = MyApp.TradingClient.connect()\n        {:ok, response} = MyApp.TradingClient.place_order(conn, %{symbol: \"BTC-USD\", size: 1})\n        assert response.order_id\n      end\n    end","ref":"WebsockexNova.ClientMacro.html#module-testing-support"},{"type":"module","title":"Available Functions - WebsockexNova.ClientMacro","doc":"The following functions are automatically available in your client module:\n\n- `connect/0`, `connect/1` - Establish WebSocket connection\n- `close/1` - Close connection\n- `ping/1` - Send ping frame\n- `send_text/2` - Send text message\n- `send_json/2` - Send JSON-encoded message\n- `send_binary/2` - Send binary message\n- `send_frame/3` - Send specific frame type\n- `send_message/2` - Send via message handler\n- `subscribe/3` - Subscribe to channel\n- `unsubscribe/2` - Unsubscribe from channel\n- `authenticate/2` - Perform authentication","ref":"WebsockexNova.ClientMacro.html#module-available-functions"},{"type":"module","title":"Tips - WebsockexNova.ClientMacro","doc":"1. Keep domain logic in your client module, not in the adapter\n2. Use typed functions for common operations (like `subscribe_to_trades/2`)\n3. Consider adding error handling in your domain-specific methods\n4. Use default_options for environment-specific configuration\n5. Document your domain-specific methods for better usability\n\nSee `WebsockexNova.Examples.ClientDeribitMacro` for a complete example.","ref":"WebsockexNova.ClientMacro.html#module-tips"},{"type":"module","title":"WebsockexNova.Connection.State","doc":"State struct for WebsockexNova.Connection GenServer.\nEncapsulates all data required for robust Gun/WebSocket lifecycle management,\nbuffering, request/response correlation, and handler module integration.","ref":"WebsockexNova.Connection.State.html"},{"type":"module","title":"WebsockexNova.ConnectionRegistry","doc":"Registry for WebSocket connection information.\n\nThis registry maps connection IDs to transport PIDs, allowing transparent reconnection \nwithout requiring client code to update connection references.\n\nThe registry is used internally by the library to maintain a stable reference to\nconnection processes even when the underlying transport process changes during reconnection.","ref":"WebsockexNova.ConnectionRegistry.html"},{"type":"function","title":"WebsockexNova.ConnectionRegistry.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\nSee `Supervisor.child_spec/2` for details.","ref":"WebsockexNova.ConnectionRegistry.html#child_spec/1"},{"type":"function","title":"WebsockexNova.ConnectionRegistry.get_transport_pid/1","doc":"Get the current transport_pid for a connection_id.\n\nReturns `{:ok, pid}` if found, or `{:error, :not_found}` if not found.","ref":"WebsockexNova.ConnectionRegistry.html#get_transport_pid/1"},{"type":"function","title":"WebsockexNova.ConnectionRegistry.register/2","doc":"Register a connection_id with a transport_pid.\nIf the connection_id is already registered, it updates the association.\n\nReturns `:ok`.","ref":"WebsockexNova.ConnectionRegistry.html#register/2"},{"type":"function","title":"WebsockexNova.ConnectionRegistry.start_link/1","doc":"Start the connection registry.\nShould be called by the application supervisor.","ref":"WebsockexNova.ConnectionRegistry.html#start_link/1"},{"type":"function","title":"WebsockexNova.ConnectionRegistry.unregister/1","doc":"Unregister a connection_id from the registry.\n\nReturns `:ok` regardless of whether the connection_id was registered.","ref":"WebsockexNova.ConnectionRegistry.html#unregister/1"},{"type":"function","title":"WebsockexNova.ConnectionRegistry.update_transport_pid/2","doc":"Updates the transport_pid for an existing connection_id.\nThis is typically used during reconnection when the transport process changes.\n\nReturns `:ok` if the update was successful, or `{:error, :not_found}` if the connection_id was not found.","ref":"WebsockexNova.ConnectionRegistry.html#update_transport_pid/2"},{"type":"module","title":"WebsockexNova.Defaults.DefaultAuthHandler","doc":"Default implementation of the AuthHandler behavior.\n\nThis module provides sensible default implementations for all AuthHandler\ncallbacks, including:\n\n* Basic token-based authentication\n* Automatic token expiration tracking\n* Credential management\n* Authentication status tracking","ref":"WebsockexNova.Defaults.DefaultAuthHandler.html"},{"type":"module","title":"Usage - WebsockexNova.Defaults.DefaultAuthHandler","doc":"You can use this module directly or as a starting point for your own implementation:\n\n    defmodule MyApp.CustomAuthHandler do\n      use WebsockexNova.Defaults.DefaultAuthHandler\n\n      # Override specific callbacks as needed\n      def generate_auth_data(state) do\n        # Custom auth data generation\n        {:ok, auth_data, state}\n      end\n    end","ref":"WebsockexNova.Defaults.DefaultAuthHandler.html#module-usage"},{"type":"module","title":"Configuration - WebsockexNova.Defaults.DefaultAuthHandler","doc":"The default handler expects the following configuration in adapter_state:\n\n* `:credentials` - Map containing authentication credentials (required)\n    * `:api_key` - API key or client ID\n    * `:secret` - API secret or password\n* `:auth_status` - Current authentication status (default: :unauthenticated)\n* `:auth_expires_at` - Unix timestamp when the authentication token expires (optional)\n* `:auth_refresh_threshold` - Seconds before expiry to trigger reauthentication (default: 60)","ref":"WebsockexNova.Defaults.DefaultAuthHandler.html#module-configuration"},{"type":"module","title":"WebsockexNova.Defaults.DefaultConnectionHandler","doc":"Default implementation of the ConnectionHandler behavior.\n\nThis module provides sensible default implementations for all ConnectionHandler\ncallbacks, including:\n\n* Basic connection tracking\n* Automatic ping/pong handling\n* Reconnection attempt management\n* Connection state maintenance","ref":"WebsockexNova.Defaults.DefaultConnectionHandler.html"},{"type":"module","title":"Usage - WebsockexNova.Defaults.DefaultConnectionHandler","doc":"You can use this module directly or as a starting point for your own implementation:\n\n    defmodule MyApp.CustomHandler do\n      use WebsockexNova.Defaults.DefaultConnectionHandler\n\n      # Override specific callbacks as needed\n      def handle_frame(:text, frame_data, conn) do\n        # Custom text frame handling\n        {:ok, conn}\n      end\n    end","ref":"WebsockexNova.Defaults.DefaultConnectionHandler.html#module-usage"},{"type":"module","title":"Configuration - WebsockexNova.Defaults.DefaultConnectionHandler","doc":"The default handler supports the following configuration in the adapter_state:\n\n* `:max_reconnect_attempts` - Maximum number of reconnection attempts (default: 5)\n* `:reconnect_attempts` - Current number of reconnection attempts (default: 0)\n* `:ping_interval` - Interval in milliseconds between ping frames (default: 30000)","ref":"WebsockexNova.Defaults.DefaultConnectionHandler.html#module-configuration"},{"type":"function","title":"WebsockexNova.Defaults.DefaultConnectionHandler.init/1","doc":"Initializes the handler state as a ClientConn struct.\nAny unknown fields are placed in adapter_state.","ref":"WebsockexNova.Defaults.DefaultConnectionHandler.html#init/1"},{"type":"module","title":"WebsockexNova.Defaults.DefaultErrorHandler","doc":"Default implementation of the ErrorHandler behavior.\n\nThis module provides sensible default implementations for all ErrorHandler\ncallbacks, including:\n\n* Error classification by type\n* Reconnection logic with exponential backoff\n* Standardized error logging\n* State tracking of errors and reconnection attempts (single source of truth)","ref":"WebsockexNova.Defaults.DefaultErrorHandler.html"},{"type":"module","title":"Usage - WebsockexNova.Defaults.DefaultErrorHandler","doc":"You can use this module directly or as a starting point for your own implementation:\n\n    defmodule MyApp.CustomErrorHandler do\n      use WebsockexNova.Defaults.DefaultErrorHandler\n\n      # Override specific callbacks as needed\n      def should_reconnect?(error, attempt, state) do\n        # Custom reconnection logic\n        {attempt < 10, attempt * 1000}\n      end\n    end","ref":"WebsockexNova.Defaults.DefaultErrorHandler.html#module-usage"},{"type":"module","title":"Configuration - WebsockexNova.Defaults.DefaultErrorHandler","doc":"The default handler supports configuration via the `:reconnection` key in the state (preferred),\nor legacy keys for backward compatibility:\n\n* `:reconnection` - map or keyword list with keys:\n    * `:max_attempts` or `:max_reconnect_attempts` (default: 5)\n    * `:base_delay` or `:initial_delay` (default: 1000)\n    * `:max_delay` (default: 30000)\n    * `:strategy` (currently only exponential supported)\n* Legacy keys (for backward compatibility):\n    * `:max_reconnect_attempts`, `:base_delay`, `:max_delay`","ref":"WebsockexNova.Defaults.DefaultErrorHandler.html#module-configuration"},{"type":"module","title":"Reconnection Attempt Tracking - WebsockexNova.Defaults.DefaultErrorHandler","doc":"This handler tracks the reconnection attempt count in the adapter_state under the `:reconnect_attempts` key.\nUse `increment_reconnect_attempts/1` and `reset_reconnect_attempts/1` to update this count.","ref":"WebsockexNova.Defaults.DefaultErrorHandler.html#module-reconnection-attempt-tracking"},{"type":"function","title":"WebsockexNova.Defaults.DefaultErrorHandler.error_init/1","doc":"","ref":"WebsockexNova.Defaults.DefaultErrorHandler.html#error_init/1"},{"type":"function","title":"WebsockexNova.Defaults.DefaultErrorHandler.get_reconnect_attempts/1","doc":"Get the reconnection attempt count from the adapter_state.","ref":"WebsockexNova.Defaults.DefaultErrorHandler.html#get_reconnect_attempts/1"},{"type":"function","title":"WebsockexNova.Defaults.DefaultErrorHandler.increment_reconnect_attempts/1","doc":"Increment the reconnection attempt count in the adapter_state.","ref":"WebsockexNova.Defaults.DefaultErrorHandler.html#increment_reconnect_attempts/1"},{"type":"function","title":"WebsockexNova.Defaults.DefaultErrorHandler.reset_reconnect_attempts/1","doc":"Reset the reconnection attempt count in the adapter_state.","ref":"WebsockexNova.Defaults.DefaultErrorHandler.html#reset_reconnect_attempts/1"},{"type":"module","title":"WebsockexNova.Defaults.DefaultLoggingHandler","doc":"Default implementation of the LoggingHandler behavior for WebsockexNova.\n\nThis module provides standardized logging for connection, message, and error events using Elixir's Logger. It supports configurable log levels and formats (plain or JSON).","ref":"WebsockexNova.Defaults.DefaultLoggingHandler.html"},{"type":"module","title":"Configuration - WebsockexNova.Defaults.DefaultLoggingHandler","doc":"The log level and format can be set in the handler state as:\n\n    %{log_level: :info, log_format: :plain}\n\nIf not set, defaults are :info and :plain.\nAll logging state is now stored in the canonical WebsockexNova.ClientConn struct under the :logging field.","ref":"WebsockexNova.Defaults.DefaultLoggingHandler.html#module-configuration"},{"type":"function","title":"WebsockexNova.Defaults.DefaultLoggingHandler.logging_init/1","doc":"Initializes the DefaultLoggingHandler state in the canonical struct.\nReturns {:ok, conn} where conn is the canonical struct with logging config set.","ref":"WebsockexNova.Defaults.DefaultLoggingHandler.html#logging_init/1"},{"type":"module","title":"WebsockexNova.Defaults.DefaultMessageHandler","doc":"Default implementation of the MessageHandler behavior.\n\nThis module provides sensible default implementations for all MessageHandler\ncallbacks, including:\n\n* JSON message parsing and validation\n* Message type detection from common fields\n* JSON encoding for outbound messages\n* Basic subscription tracking","ref":"WebsockexNova.Defaults.DefaultMessageHandler.html"},{"type":"module","title":"Usage - WebsockexNova.Defaults.DefaultMessageHandler","doc":"You can use this module directly or as a starting point for your own implementation:\n\n    defmodule MyApp.CustomMessageHandler do\n      use WebsockexNova.Defaults.DefaultMessageHandler\n\n      # Override specific callbacks as needed\n      def message_type(message) do\n        # Custom message type detection\n        Map.get(message, \"custom_type\", :unknown)\n      end\n    end","ref":"WebsockexNova.Defaults.DefaultMessageHandler.html#module-usage"},{"type":"module","title":"Features - WebsockexNova.Defaults.DefaultMessageHandler","doc":"* Automatically parses JSON text messages\n* Tracks message processing count\n* Handles common subscription responses\n* Provides standardized error handling","ref":"WebsockexNova.Defaults.DefaultMessageHandler.html#module-features"},{"type":"module","title":"WebsockexNova.Defaults.DefaultMetricsCollector","doc":"Default implementation of the MetricsCollector behavior for WebsockexNova.\n\nSubscribes to all relevant telemetry events and aggregates metrics in ETS tables.\nProvides a public API for querying metrics (for testing/demo purposes).\n\nAll metrics handler API now uses the canonical WebsockexNova.ClientConn struct,\nwith metrics-specific state in the :metrics field.","ref":"WebsockexNova.Defaults.DefaultMetricsCollector.html"},{"type":"module","title":"Metrics Tracked - WebsockexNova.Defaults.DefaultMetricsCollector","doc":"* Connection statistics (open/close counts, durations)\n  * Message throughput (sent/received count, size, latency)\n  * Error metrics (count by category)","ref":"WebsockexNova.Defaults.DefaultMetricsCollector.html#module-metrics-tracked"},{"type":"module","title":"Usage - WebsockexNova.Defaults.DefaultMetricsCollector","doc":"# Start the collector (normally done in your supervision tree)\n    WebsockexNova.Defaults.DefaultMetricsCollector.start_link([])\n\n    # Query metrics (for testing/demo)\n    WebsockexNova.Defaults.DefaultMetricsCollector.get_metric(:connections_opened)","ref":"WebsockexNova.Defaults.DefaultMetricsCollector.html#module-usage"},{"type":"function","title":"WebsockexNova.Defaults.DefaultMetricsCollector.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"WebsockexNova.Defaults.DefaultMetricsCollector.html#child_spec/1"},{"type":"function","title":"WebsockexNova.Defaults.DefaultMetricsCollector.do_handle_connection_event/3","doc":"","ref":"WebsockexNova.Defaults.DefaultMetricsCollector.html#do_handle_connection_event/3"},{"type":"function","title":"WebsockexNova.Defaults.DefaultMetricsCollector.do_handle_error_event/3","doc":"","ref":"WebsockexNova.Defaults.DefaultMetricsCollector.html#do_handle_error_event/3"},{"type":"function","title":"WebsockexNova.Defaults.DefaultMetricsCollector.do_handle_message_event/3","doc":"","ref":"WebsockexNova.Defaults.DefaultMetricsCollector.html#do_handle_message_event/3"},{"type":"function","title":"WebsockexNova.Defaults.DefaultMetricsCollector.get_metric/1","doc":"Get a metric by key (for testing/demo).","ref":"WebsockexNova.Defaults.DefaultMetricsCollector.html#get_metric/1"},{"type":"function","title":"WebsockexNova.Defaults.DefaultMetricsCollector.get_metric/2","doc":"","ref":"WebsockexNova.Defaults.DefaultMetricsCollector.html#get_metric/2"},{"type":"function","title":"WebsockexNova.Defaults.DefaultMetricsCollector.handle_telemetry/4","doc":"","ref":"WebsockexNova.Defaults.DefaultMetricsCollector.html#handle_telemetry/4"},{"type":"function","title":"WebsockexNova.Defaults.DefaultMetricsCollector.start_link/1","doc":"","ref":"WebsockexNova.Defaults.DefaultMetricsCollector.html#start_link/1"},{"type":"module","title":"WebsockexNova.Defaults.DefaultRateLimitHandler","doc":"Default implementation of the RateLimitHandler behavior.\nThis module now stores all rate limit state in the canonical WebsockexNova.ClientConn struct,\nunder the :rate_limit field.\n\nThis module provides a standard token bucket algorithm implementation for rate limiting\nWebSocket requests. It supports:\n\n* Configurable token bucket capacity\n* Flexible token refill rates\n* Request queueing with priority handling\n* Burst handling through token accumulation","ref":"WebsockexNova.Defaults.DefaultRateLimitHandler.html"},{"type":"module","title":"Configuration Options - WebsockexNova.Defaults.DefaultRateLimitHandler","doc":"* `:mode` - Rate limiting mode (`:normal`, `:always_allow`, `:always_queue`, `:always_reject`)\n* `:capacity` - Maximum number of tokens in the bucket (default: 60)\n* `:refill_rate` - Number of tokens to add per interval (default: 1)\n* `:refill_interval` - Milliseconds between token refills (default: 1000)\n* `:queue_limit` - Maximum number of queued requests (default: 100)\n* `:cost_map` - Map of request types to their token costs (default: %{})","ref":"WebsockexNova.Defaults.DefaultRateLimitHandler.html#module-configuration-options"},{"type":"module","title":"Examples - WebsockexNova.Defaults.DefaultRateLimitHandler","doc":"```elixir\n# Initialize with custom configuration\n{:ok, state} = WebsockexNova.Defaults.DefaultRateLimitHandler.rate_limit_init(\n  capacity: 100,\n  refill_rate: 5,\n  refill_interval: 1000,\n  queue_limit: 50,\n  mode: :normal,\n  cost_map: %{\n    subscription: 5,\n    auth: 10,\n    query: 1\n  }\n)\n```","ref":"WebsockexNova.Defaults.DefaultRateLimitHandler.html#module-examples"},{"type":"function","title":"WebsockexNova.Defaults.DefaultRateLimitHandler.check_rate_limit/2","doc":"Check if a request can proceed based on current rate limits.\nAccepts and returns the canonical struct.","ref":"WebsockexNova.Defaults.DefaultRateLimitHandler.html#check_rate_limit/2"},{"type":"function","title":"WebsockexNova.Defaults.DefaultRateLimitHandler.handle_tick/1","doc":"Process queued requests on a periodic tick.\nAccepts and returns the canonical struct.","ref":"WebsockexNova.Defaults.DefaultRateLimitHandler.html#handle_tick/1"},{"type":"function","title":"WebsockexNova.Defaults.DefaultRateLimitHandler.rate_limit_init/1","doc":"Initialize the rate limit handler's state in the canonical struct.","ref":"WebsockexNova.Defaults.DefaultRateLimitHandler.html#rate_limit_init/1"},{"type":"module","title":"WebsockexNova.Defaults.DefaultSubscriptionHandler","doc":"Default implementation of the SubscriptionHandler behavior.\n\nThis module provides sensible default implementations for all SubscriptionHandler\ncallbacks, enabling applications to subscribe to data streams via WebSocket.","ref":"WebsockexNova.Defaults.DefaultSubscriptionHandler.html"},{"type":"module","title":"Features - WebsockexNova.Defaults.DefaultSubscriptionHandler","doc":"* Subscription management with unique IDs\n* Automatic subscription status tracking\n* Automatic timeout detection for pending subscriptions\n* Methods to find and filter active subscriptions","ref":"WebsockexNova.Defaults.DefaultSubscriptionHandler.html#module-features"},{"type":"module","title":"Usage - WebsockexNova.Defaults.DefaultSubscriptionHandler","doc":"You can use this module directly or as a starting point for your own implementation:\n\n    defmodule MyApp.CustomSubscriptionHandler do\n      use WebsockexNova.Defaults.DefaultSubscriptionHandler\n\n      # Override specific callbacks as needed\n      def subscribe(channel, params, conn) do\n        # Custom subscription logic\n        # ...\n      end\n    end","ref":"WebsockexNova.Defaults.DefaultSubscriptionHandler.html#module-usage"},{"type":"type","title":"WebsockexNova.Defaults.DefaultSubscriptionHandler.channel/0","doc":"A channel or topic to subscribe to","ref":"WebsockexNova.Defaults.DefaultSubscriptionHandler.html#t:channel/0"},{"type":"function","title":"WebsockexNova.Defaults.DefaultSubscriptionHandler.cleanup_expired_subscriptions/1","doc":"Clean up expired subscriptions.","ref":"WebsockexNova.Defaults.DefaultSubscriptionHandler.html#cleanup_expired_subscriptions/1"},{"type":"function","title":"WebsockexNova.Defaults.DefaultSubscriptionHandler.find_subscriptions_by_status/2","doc":"Find all subscriptions with a specific status.","ref":"WebsockexNova.Defaults.DefaultSubscriptionHandler.html#find_subscriptions_by_status/2"},{"type":"type","title":"WebsockexNova.Defaults.DefaultSubscriptionHandler.params/0","doc":"Parameters for the subscription","ref":"WebsockexNova.Defaults.DefaultSubscriptionHandler.html#t:params/0"},{"type":"type","title":"WebsockexNova.Defaults.DefaultSubscriptionHandler.state/0","doc":"State - either a ClientConn struct or any map with subscription-related fields","ref":"WebsockexNova.Defaults.DefaultSubscriptionHandler.html#t:state/0"},{"type":"type","title":"WebsockexNova.Defaults.DefaultSubscriptionHandler.subscription_id/0","doc":"A unique identifier for a subscription","ref":"WebsockexNova.Defaults.DefaultSubscriptionHandler.html#t:subscription_id/0"},{"type":"function","title":"WebsockexNova.Defaults.DefaultSubscriptionHandler.subscription_init/1","doc":"Initializes a subscription handler.","ref":"WebsockexNova.Defaults.DefaultSubscriptionHandler.html#subscription_init/1"},{"type":"function","title":"Parameters - WebsockexNova.Defaults.DefaultSubscriptionHandler.subscription_init/1","doc":"* `opts` - Options for initializing the subscription handler (optional)","ref":"WebsockexNova.Defaults.DefaultSubscriptionHandler.html#subscription_init/1-parameters"},{"type":"function","title":"Returns - WebsockexNova.Defaults.DefaultSubscriptionHandler.subscription_init/1","doc":"* `{:ok, conn}` - Initial state with subscription handler settings","ref":"WebsockexNova.Defaults.DefaultSubscriptionHandler.html#subscription_init/1-returns"},{"type":"type","title":"WebsockexNova.Defaults.DefaultSubscriptionHandler.subscription_response/0","doc":"Response from a subscription request","ref":"WebsockexNova.Defaults.DefaultSubscriptionHandler.html#t:subscription_response/0"},{"type":"module","title":"WebsockexNova.Examples.AdapterDeribit","doc":"Example Deribit adapter using `use WebsockexNova.Adapter` macro.\n\nThis demonstrates how to build a Deribit WebSocket API adapter with minimal overrides.","ref":"WebsockexNova.Examples.AdapterDeribit.html"},{"type":"function","title":"WebsockexNova.Examples.AdapterDeribit.handle_frame/3","doc":"Handle WebSocket frames directly. This is crucial for proper handling of\nDeribit's heartbeat mechanism, which requires responding to test_request\nframes with a public/test request.","ref":"WebsockexNova.Examples.AdapterDeribit.html#handle_frame/3"},{"type":"module","title":"WebsockexNova.Examples.AdapterWithSubscriptionPreservation","doc":"Example adapter demonstrating subscription preservation across reconnections.\n\nThis adapter uses the SubscriptionManager to track and restore subscriptions\nautomatically when the connection is re-established after a disconnection.","ref":"WebsockexNova.Examples.AdapterWithSubscriptionPreservation.html"},{"type":"function","title":"WebsockexNova.Examples.AdapterWithSubscriptionPreservation.prepare_frame/3","doc":"","ref":"WebsockexNova.Examples.AdapterWithSubscriptionPreservation.html#prepare_frame/3"},{"type":"module","title":"WebsockexNova.Examples.ClientDeribit","doc":"Deribit-specific client API, wrapping WebsockexNova.Client with the Deribit adapter.\n\nProvides a user-friendly, domain-specific interface for connecting, authenticating,\nand subscribing to Deribit WebSocket channels.","ref":"WebsockexNova.Examples.ClientDeribit.html"},{"type":"function","title":"WebsockexNova.Examples.ClientDeribit.authenticate/3","doc":"Authenticate using client credentials (from opts or environment).","ref":"WebsockexNova.Examples.ClientDeribit.html#authenticate/3"},{"type":"function","title":"WebsockexNova.Examples.ClientDeribit.connect/1","doc":"Connect to Deribit WebSocket API with sensible defaults.\nUser-supplied opts override defaults.","ref":"WebsockexNova.Examples.ClientDeribit.html#connect/1"},{"type":"function","title":"WebsockexNova.Examples.ClientDeribit.disconnect/1","doc":"Disconnects from the Deribit WebSocket server.","ref":"WebsockexNova.Examples.ClientDeribit.html#disconnect/1"},{"type":"function","title":"WebsockexNova.Examples.ClientDeribit.send_json/3","doc":"Send a custom JSON-RPC payload to Deribit.","ref":"WebsockexNova.Examples.ClientDeribit.html#send_json/3"},{"type":"function","title":"WebsockexNova.Examples.ClientDeribit.set_heartbeat/3","doc":"Sets the heartbeat interval for the Deribit connection.\nThe server will send a heartbeat message every `interval` seconds.\n\nWhen the heartbeat is set, the server will send:\n1. Regular \"heartbeat\" messages\n2. Periodic \"test_request\" messages that require a response\n\nThe value must be between 10 and 60 seconds for the Deribit API.\nSetting to 0 disables the heartbeat.","ref":"WebsockexNova.Examples.ClientDeribit.html#set_heartbeat/3"},{"type":"function","title":"WebsockexNova.Examples.ClientDeribit.subscribe_to_ticker/3","doc":"Subscribe to a Deribit ticker channel for a given instrument (e.g., \"BTC-PERPETUAL\").","ref":"WebsockexNova.Examples.ClientDeribit.html#subscribe_to_ticker/3"},{"type":"function","title":"WebsockexNova.Examples.ClientDeribit.subscribe_to_trades/3","doc":"Subscribe to a Deribit trades channel for a given instrument (e.g., \"BTC-PERPETUAL\").","ref":"WebsockexNova.Examples.ClientDeribit.html#subscribe_to_trades/3"},{"type":"module","title":"WebsockexNova.Examples.ClientDeribitMacro","doc":"Deribit-specific client API using the ClientMacro.\n\nThis demonstrates how to build a Deribit WebSocket API client with minimal code,\nleveraging the WebsockexNova.ClientMacro.","ref":"WebsockexNova.Examples.ClientDeribitMacro.html"},{"type":"function","title":"WebsockexNova.Examples.ClientDeribitMacro.authenticate/3","doc":"Authenticate using credentials.","ref":"WebsockexNova.Examples.ClientDeribitMacro.html#authenticate/3"},{"type":"function","title":"WebsockexNova.Examples.ClientDeribitMacro.close/1","doc":"Close the connection.","ref":"WebsockexNova.Examples.ClientDeribitMacro.html#close/1"},{"type":"function","title":"WebsockexNova.Examples.ClientDeribitMacro.connect/1","doc":"Connect to WebSocket API with sensible defaults from the adapter.\nUser-supplied opts override defaults.","ref":"WebsockexNova.Examples.ClientDeribitMacro.html#connect/1"},{"type":"function","title":"WebsockexNova.Examples.ClientDeribitMacro.default_opts/0","doc":"","ref":"WebsockexNova.Examples.ClientDeribitMacro.html#default_opts/0"},{"type":"function","title":"WebsockexNova.Examples.ClientDeribitMacro.get_account_summary/3","doc":"Get account summary","ref":"WebsockexNova.Examples.ClientDeribitMacro.html#get_account_summary/3"},{"type":"function","title":"WebsockexNova.Examples.ClientDeribitMacro.get_platform_status/2","doc":"Get information about locked currencies on the platform.","ref":"WebsockexNova.Examples.ClientDeribitMacro.html#get_platform_status/2"},{"type":"function","title":"Example - WebsockexNova.Examples.ClientDeribitMacro.get_platform_status/2","doc":"{:ok, status} = DeribitClient.get_platform_status(conn)\n    # status will include information on locked currencies","ref":"WebsockexNova.Examples.ClientDeribitMacro.html#get_platform_status/2-example"},{"type":"function","title":"WebsockexNova.Examples.ClientDeribitMacro.get_time/2","doc":"Retrieves the current time from Deribit (in milliseconds).\n\nThis is useful for checking clock skew between your software and Deribit's systems.","ref":"WebsockexNova.Examples.ClientDeribitMacro.html#get_time/2"},{"type":"function","title":"Example - WebsockexNova.Examples.ClientDeribitMacro.get_time/2","doc":"{:ok, timestamp} = DeribitClient.get_time(conn)\n    # timestamp will be something like 1550147385946","ref":"WebsockexNova.Examples.ClientDeribitMacro.html#get_time/2-example"},{"type":"function","title":"WebsockexNova.Examples.ClientDeribitMacro.hello/4","doc":"Introduce the client software to Deribit over WebSocket.\n\nDeribit will also introduce itself in the response.","ref":"WebsockexNova.Examples.ClientDeribitMacro.html#hello/4"},{"type":"function","title":"Parameters - WebsockexNova.Examples.ClientDeribitMacro.hello/4","doc":"* `client_name` - Client software name\n* `client_version` - Client software version","ref":"WebsockexNova.Examples.ClientDeribitMacro.html#hello/4-parameters"},{"type":"function","title":"Example - WebsockexNova.Examples.ClientDeribitMacro.hello/4","doc":"{:ok, response} = DeribitClient.hello(conn, \"MyTradingApp\", \"1.0.0\")\n    # response will include Deribit's version information","ref":"WebsockexNova.Examples.ClientDeribitMacro.html#hello/4-example"},{"type":"function","title":"WebsockexNova.Examples.ClientDeribitMacro.ping/2","doc":"Send a ping and wait for pong response.","ref":"WebsockexNova.Examples.ClientDeribitMacro.html#ping/2"},{"type":"function","title":"WebsockexNova.Examples.ClientDeribitMacro.place_market_order/3","doc":"Place a market order (buy/sell at market price)","ref":"WebsockexNova.Examples.ClientDeribitMacro.html#place_market_order/3"},{"type":"function","title":"WebsockexNova.Examples.ClientDeribitMacro.send_json/3","doc":"Send a JSON message.","ref":"WebsockexNova.Examples.ClientDeribitMacro.html#send_json/3"},{"type":"function","title":"WebsockexNova.Examples.ClientDeribitMacro.send_text/3","doc":"Send a text message.","ref":"WebsockexNova.Examples.ClientDeribitMacro.html#send_text/3"},{"type":"function","title":"WebsockexNova.Examples.ClientDeribitMacro.status/2","doc":"Get connection status.","ref":"WebsockexNova.Examples.ClientDeribitMacro.html#status/2"},{"type":"function","title":"WebsockexNova.Examples.ClientDeribitMacro.subscribe/3","doc":"Subscribe to a channel.","ref":"WebsockexNova.Examples.ClientDeribitMacro.html#subscribe/3"},{"type":"function","title":"WebsockexNova.Examples.ClientDeribitMacro.subscribe_to_ticker/3","doc":"Subscribe to a Deribit ticker channel for a given instrument (e.g., \"BTC-PERPETUAL\").","ref":"WebsockexNova.Examples.ClientDeribitMacro.html#subscribe_to_ticker/3"},{"type":"function","title":"WebsockexNova.Examples.ClientDeribitMacro.subscribe_to_trades/3","doc":"Subscribe to a Deribit trades channel for a given instrument (e.g., \"BTC-PERPETUAL\").","ref":"WebsockexNova.Examples.ClientDeribitMacro.html#subscribe_to_trades/3"},{"type":"function","title":"WebsockexNova.Examples.ClientDeribitMacro.test/3","doc":"Tests the connection to the API server and returns its version.\n\nUse to verify API reachability and version.","ref":"WebsockexNova.Examples.ClientDeribitMacro.html#test/3"},{"type":"function","title":"Parameters - WebsockexNova.Examples.ClientDeribitMacro.test/3","doc":"* `expected_result` - (Optional) If set to \"exception\", triggers error","ref":"WebsockexNova.Examples.ClientDeribitMacro.html#test/3-parameters"},{"type":"function","title":"Example - WebsockexNova.Examples.ClientDeribitMacro.test/3","doc":"{:ok, version_info} = DeribitClient.test(conn)\n    # version_info will include Deribit's API version","ref":"WebsockexNova.Examples.ClientDeribitMacro.html#test/3-example"},{"type":"function","title":"WebsockexNova.Examples.ClientDeribitMacro.unsubscribe/3","doc":"Unsubscribe from a channel.","ref":"WebsockexNova.Examples.ClientDeribitMacro.html#unsubscribe/3"},{"type":"module","title":"WebsockexNova.Gun.BehaviorBridge","doc":"Connects Gun events to WebsockexNova behavior callbacks.\n\nThis module serves as the bridge between the Gun transport layer and the WebsockexNova\nbehavior-based architecture. It translates Gun-specific events into standardized callbacks\nto the appropriate behavior implementations.","ref":"WebsockexNova.Gun.BehaviorBridge.html"},{"type":"module","title":"Responsibilities - WebsockexNova.Gun.BehaviorBridge","doc":"The BehaviorBridge has the following key responsibilities:\n\n1. **Event Translation**: Converts Gun-specific messages to behavior-friendly formats\n2. **Routing**: Routes events to the appropriate behavior (ConnectionHandler, MessageHandler, ErrorHandler)\n3. **State Management**: Maintains and updates connection state based on behavior responses\n4. **Response Processing**: Handles behavior return values with appropriate actions\n5. **Error Handling**: Ensures robust error handling throughout the bridge","ref":"WebsockexNova.Gun.BehaviorBridge.html#module-responsibilities"},{"type":"module","title":"Integration Points - WebsockexNova.Gun.BehaviorBridge","doc":"The BehaviorBridge integrates with:\n\n* **Gun Connection Events**: Handles gun_up, gun_down, and other Gun protocol events\n* **WebSocket Frames**: Processes WebSocket frames and routes them to handlers\n* **Connection State**: Updates and manages connection state through transitions\n* **Behavior Callbacks**: Invokes the appropriate callbacks on behavior implementations\n\nThis module is designed to be used internally by the ConnectionWrapper, which acts as\nthe main GenServer responsible for receiving and processing Gun messages.","ref":"WebsockexNova.Gun.BehaviorBridge.html#module-integration-points"},{"type":"function","title":"WebsockexNova.Gun.BehaviorBridge.handle_error/3","doc":"Handles errors encountered during Gun operations.\n\nRoutes the error to the error handler for processing.","ref":"WebsockexNova.Gun.BehaviorBridge.html#handle_error/3"},{"type":"function","title":"Parameters - WebsockexNova.Gun.BehaviorBridge.handle_error/3","doc":"* `error` - The error that occurred\n* `context` - Additional context about the error\n* `state` - Current connection state","ref":"WebsockexNova.Gun.BehaviorBridge.html#handle_error/3-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.BehaviorBridge.handle_error/3","doc":"* `{:noreply, updated_state}` - Continue with the updated state\n* `{:noreply, {:reconnect, updated_state}}` - Attempt reconnection\n* `{:stop, reason, updated_state}` - Stop the process","ref":"WebsockexNova.Gun.BehaviorBridge.html#handle_error/3-returns"},{"type":"function","title":"WebsockexNova.Gun.BehaviorBridge.handle_gun_down/6","doc":"Handles the `:gun_down` message when a connection is lost.\n\nTranslates this event to a call to the connection handler's handle_disconnect callback.","ref":"WebsockexNova.Gun.BehaviorBridge.html#handle_gun_down/6"},{"type":"function","title":"Parameters - WebsockexNova.Gun.BehaviorBridge.handle_gun_down/6","doc":"* `gun_pid` - The Gun connection PID\n* `protocol` - The protocol that was in use\n* `reason` - Reason for the connection loss\n* `state` - Current connection state\n* `killed_streams` - List of stream references that were killed\n* `unprocessed_streams` - List of stream references with unprocessed data","ref":"WebsockexNova.Gun.BehaviorBridge.html#handle_gun_down/6-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.BehaviorBridge.handle_gun_down/6","doc":"* `{:noreply, updated_state}` - Standard response\n* `{:noreply, {:reconnect, updated_state}}` - Request reconnection\n* `{:stop, reason, updated_state}` - Request process termination","ref":"WebsockexNova.Gun.BehaviorBridge.html#handle_gun_down/6-returns"},{"type":"function","title":"WebsockexNova.Gun.BehaviorBridge.handle_gun_up/3","doc":"Handles the `:gun_up` message when a connection is established.\n\nTranslates this event to a call to the connection handler's handle_connect callback.","ref":"WebsockexNova.Gun.BehaviorBridge.html#handle_gun_up/3"},{"type":"function","title":"Parameters - WebsockexNova.Gun.BehaviorBridge.handle_gun_up/3","doc":"* `gun_pid` - The Gun connection PID\n* `protocol` - The protocol that was negotiated\n* `state` - Current connection state","ref":"WebsockexNova.Gun.BehaviorBridge.html#handle_gun_up/3-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.BehaviorBridge.handle_gun_up/3","doc":"`{:noreply, updated_state}` or other appropriate return value","ref":"WebsockexNova.Gun.BehaviorBridge.html#handle_gun_up/3-returns"},{"type":"function","title":"WebsockexNova.Gun.BehaviorBridge.handle_websocket_frame/4","doc":"Handles WebSocket frames received from Gun.\n\nProcesses the frame and routes it to the appropriate handler.","ref":"WebsockexNova.Gun.BehaviorBridge.html#handle_websocket_frame/4"},{"type":"function","title":"Parameters - WebsockexNova.Gun.BehaviorBridge.handle_websocket_frame/4","doc":"* `gun_pid` - The Gun connection PID\n* `stream_ref` - The stream reference for the frame\n* `frame` - The WebSocket frame (as a tuple like `{:text, data}`)\n* `state` - Current connection state","ref":"WebsockexNova.Gun.BehaviorBridge.html#handle_websocket_frame/4-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.BehaviorBridge.handle_websocket_frame/4","doc":"* `{:noreply, updated_state}` - Standard response\n* `{:reply, frame_type, data, updated_state, stream_ref}` - Response to send\n* `{:stop, reason, updated_state}` - Stop the process","ref":"WebsockexNova.Gun.BehaviorBridge.html#handle_websocket_frame/4-returns"},{"type":"function","title":"WebsockexNova.Gun.BehaviorBridge.handle_websocket_upgrade/4","doc":"Handles WebSocket upgrades received from Gun.\n\nTransitions the state to websocket_connected and notifies the connection handler.","ref":"WebsockexNova.Gun.BehaviorBridge.html#handle_websocket_upgrade/4"},{"type":"function","title":"Parameters - WebsockexNova.Gun.BehaviorBridge.handle_websocket_upgrade/4","doc":"* `gun_pid` - The Gun connection PID\n* `stream_ref` - The stream reference for the upgrade\n* `headers` - The response headers received\n* `state` - Current connection state","ref":"WebsockexNova.Gun.BehaviorBridge.html#handle_websocket_upgrade/4-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.BehaviorBridge.handle_websocket_upgrade/4","doc":"* `{:noreply, updated_state}` - Standard response\n* `{:stop, reason, updated_state}` - Stop the process","ref":"WebsockexNova.Gun.BehaviorBridge.html#handle_websocket_upgrade/4-returns"},{"type":"function","title":"WebsockexNova.Gun.BehaviorBridge.should_reconnect?/3","doc":"Determines if reconnection should be attempted after an error.\n\nConsults the error handler to make this decision.","ref":"WebsockexNova.Gun.BehaviorBridge.html#should_reconnect?/3"},{"type":"function","title":"Parameters - WebsockexNova.Gun.BehaviorBridge.should_reconnect?/3","doc":"* `error` - The error that caused the disconnection\n* `attempt` - The current reconnection attempt number\n* `state` - Current connection state","ref":"WebsockexNova.Gun.BehaviorBridge.html#should_reconnect?/3-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.BehaviorBridge.should_reconnect?/3","doc":"* `{true, delay}` - Should reconnect after the specified delay\n* `{false, _}` - Should not reconnect","ref":"WebsockexNova.Gun.BehaviorBridge.html#should_reconnect?/3-returns"},{"type":"module","title":"WebsockexNova.Gun.ConnectionManager","doc":"Manages the WebSocket connection lifecycle using a state machine approach.","ref":"WebsockexNova.Gun.ConnectionManager.html"},{"type":"module","title":"Responsibilities - WebsockexNova.Gun.ConnectionManager","doc":"- Handles connection state transitions.\n- Delegates all reconnection policy decisions to the error handler (single source of truth).\n- On disconnect/error, calls the error handler's `should_reconnect?/3`.\n- If allowed, calls the transport adapter's `schedule_reconnection/2`.\n- Does not track reconnection attempts or delays; this is handled by the error handler.","ref":"WebsockexNova.Gun.ConnectionManager.html#module-responsibilities"},{"type":"function","title":"WebsockexNova.Gun.ConnectionManager.can_transition?/2","doc":"Checks if the connection state can transition to the target state.","ref":"WebsockexNova.Gun.ConnectionManager.html#can_transition?/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionManager.can_transition?/2","doc":"* `from_state` - Current state\n* `to_state` - Target state","ref":"WebsockexNova.Gun.ConnectionManager.html#can_transition?/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionManager.can_transition?/2","doc":"* `true` if the transition is allowed\n* `false` if the transition is not allowed","ref":"WebsockexNova.Gun.ConnectionManager.html#can_transition?/2-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionManager.initiate_connection/1","doc":"Initiates a connection.","ref":"WebsockexNova.Gun.ConnectionManager.html#initiate_connection/1"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionManager.initiate_connection/1","doc":"* `state` - Current connection state","ref":"WebsockexNova.Gun.ConnectionManager.html#initiate_connection/1-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionManager.initiate_connection/1","doc":"* `{:ok, updated_state}` on success\n* `{:error, reason, updated_state}` on failure","ref":"WebsockexNova.Gun.ConnectionManager.html#initiate_connection/1-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionManager.schedule_reconnection/2","doc":"Schedules a reconnection attempt by delegating policy to the error handler.\nThis function is the only place reconnection is scheduled.","ref":"WebsockexNova.Gun.ConnectionManager.html#schedule_reconnection/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionManager.schedule_reconnection/2","doc":"* `state` - Current connection state\n* `callback` - Function to call with (delay, attempt_number) when reconnection should be attempted","ref":"WebsockexNova.Gun.ConnectionManager.html#schedule_reconnection/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionManager.schedule_reconnection/2","doc":"* `updated_state` - The new connection state after scheduling (or not scheduling) reconnection","ref":"WebsockexNova.Gun.ConnectionManager.html#schedule_reconnection/2-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionManager.start_connection/1","doc":"Starts a new connection by transitioning to connecting state and opening a connection.\n\nThis function centralizes the connection establishment process, handling both the\nstate transition and the actual connection attempt.","ref":"WebsockexNova.Gun.ConnectionManager.html#start_connection/1"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionManager.start_connection/1","doc":"* `state` - Current connection state","ref":"WebsockexNova.Gun.ConnectionManager.html#start_connection/1-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionManager.start_connection/1","doc":"* `{:ok, updated_state}` on success\n* `{:error, reason, updated_state}` on failure","ref":"WebsockexNova.Gun.ConnectionManager.html#start_connection/1-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionManager.transition_to/3","doc":"Attempts to transition the state machine to a new state.","ref":"WebsockexNova.Gun.ConnectionManager.html#transition_to/3"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionManager.transition_to/3","doc":"* `state` - Current connection state\n* `to_state` - Desired new state\n* `params` - Optional parameters for the transition","ref":"WebsockexNova.Gun.ConnectionManager.html#transition_to/3-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionManager.transition_to/3","doc":"* `{:ok, new_state}` on successful transition\n* `{:error, :invalid_transition}` if the transition is not allowed","ref":"WebsockexNova.Gun.ConnectionManager.html#transition_to/3-returns"},{"type":"module","title":"WebsockexNova.Gun.ConnectionOptions","doc":"Centralized parsing and validation for Gun connection options.","ref":"WebsockexNova.Gun.ConnectionOptions.html"},{"type":"module","title":"Defaults and Security - WebsockexNova.Gun.ConnectionOptions","doc":"- The default transport is `:tls` (HTTPS/WSS) for secure connections.\n- For most production APIs and public WebSocket endpoints, this is required.\n- If you are connecting to a local or non-TLS endpoint (e.g., localhost:80), you must override:\n\n    %{transport: :tcp, port: 80}\n\n- For self-signed certificates in development, you can use:\n\n    %{transport: :tls, transport_opts: [verify: :verify_none]}\n\n  **Never use `verify: :verify_none` in production!**","ref":"WebsockexNova.Gun.ConnectionOptions.html#module-defaults-and-security"},{"type":"module","title":"Example Usage - WebsockexNova.Gun.ConnectionOptions","doc":"# Production (default)\n    {:ok, conn} = WebsockexNova.Gun.ConnectionWrapper.open(\"api.example.com\", 443)\n\n    # Local development (plain HTTP)\n    {:ok, conn} = WebsockexNova.Gun.ConnectionWrapper.open(\"localhost\", 80, %{transport: :tcp})\n\n    # Local development (self-signed HTTPS)\n    {:ok, conn} = WebsockexNova.Gun.ConnectionWrapper.open(\"localhost\", 443, %{transport_opts: [verify: :verify_none]})\n\nThis module merges user options with defaults, validates required fields,\nand normalizes option values for consistent downstream usage.","ref":"WebsockexNova.Gun.ConnectionOptions.html#module-example-usage"},{"type":"function","title":"WebsockexNova.Gun.ConnectionOptions.parse_and_validate/1","doc":"","ref":"WebsockexNova.Gun.ConnectionOptions.html#parse_and_validate/1"},{"type":"module","title":"WebsockexNova.Gun.ConnectionState","doc":"State structure for ConnectionWrapper.\n\nThis module defines the state structure used by the ConnectionWrapper,\nproviding better type safety and organization of state data.","ref":"WebsockexNova.Gun.ConnectionState.html"},{"type":"module","title":"IMPORTANT: State Layering - WebsockexNova.Gun.ConnectionState","doc":"This struct should ONLY contain transport-layer, process-local, or Gun-specific data.\nIt should NOT store canonical application-level state like:\n- Authentication\n- Subscriptions\n- User credentials\n- Application handler state\n\nThe canonical application state lives in `WebsockexNova.ClientConn`.\nThis struct may reference a ClientConn struct for operations, but should\nnever duplicate the canonical state.","ref":"WebsockexNova.Gun.ConnectionState.html#module-important-state-layering"},{"type":"module","title":"Reconnection Policy - WebsockexNova.Gun.ConnectionState","doc":"- The connection state no longer tracks reconnection attempts or delays.\n- All reconnection policy and tracking is handled by the error handler.","ref":"WebsockexNova.Gun.ConnectionState.html#module-reconnection-policy"},{"type":"function","title":"WebsockexNova.Gun.ConnectionState.add_active_stream/3","doc":"Adds an active stream to the state with the given status and data.","ref":"WebsockexNova.Gun.ConnectionState.html#add_active_stream/3"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionState.add_active_stream/3","doc":"* `state` - Current connection state\n* `stream_ref` - Stream reference to add\n* `stream_data` - Data for the stream, can be either an atom status or a map with data","ref":"WebsockexNova.Gun.ConnectionState.html#add_active_stream/3-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionState.add_active_stream/3","doc":"Updated connection state struct","ref":"WebsockexNova.Gun.ConnectionState.html#add_active_stream/3-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionState.clear_all_streams/1","doc":"Clears all active streams from the state.","ref":"WebsockexNova.Gun.ConnectionState.html#clear_all_streams/1"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionState.clear_all_streams/1","doc":"* `state` - Current connection state","ref":"WebsockexNova.Gun.ConnectionState.html#clear_all_streams/1-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionState.clear_all_streams/1","doc":"Updated connection state struct with empty active_streams","ref":"WebsockexNova.Gun.ConnectionState.html#clear_all_streams/1-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionState.get_callback_pid/1","doc":"Gets the callback PID from the state.","ref":"WebsockexNova.Gun.ConnectionState.html#get_callback_pid/1"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionState.get_callback_pid/1","doc":"* `state` - Current connection state","ref":"WebsockexNova.Gun.ConnectionState.html#get_callback_pid/1-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionState.get_callback_pid/1","doc":"The callback PID, or nil if none exists","ref":"WebsockexNova.Gun.ConnectionState.html#get_callback_pid/1-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionState.new/3","doc":"Creates a new connection state.","ref":"WebsockexNova.Gun.ConnectionState.html#new/3"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionState.new/3","doc":"* `host` - The hostname to connect to\n* `port` - The port to connect to\n* `options` - Connection options (only transport configuration is stored)","ref":"WebsockexNova.Gun.ConnectionState.html#new/3-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionState.new/3","doc":"A new connection state struct","ref":"WebsockexNova.Gun.ConnectionState.html#new/3-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionState.prepare_for_termination/1","doc":"Prepares the state for process termination by clearing references and resources.","ref":"WebsockexNova.Gun.ConnectionState.html#prepare_for_termination/1"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionState.prepare_for_termination/1","doc":"* `state` - Current connection state","ref":"WebsockexNova.Gun.ConnectionState.html#prepare_for_termination/1-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionState.prepare_for_termination/1","doc":"Updated connection state ready for termination","ref":"WebsockexNova.Gun.ConnectionState.html#prepare_for_termination/1-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionState.record_error/2","doc":"Records an error in the state.","ref":"WebsockexNova.Gun.ConnectionState.html#record_error/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionState.record_error/2","doc":"* `state` - Current connection state\n* `error` - Error to record","ref":"WebsockexNova.Gun.ConnectionState.html#record_error/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionState.record_error/2","doc":"Updated connection state struct","ref":"WebsockexNova.Gun.ConnectionState.html#record_error/2-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionState.remove_stream/2","doc":"Removes a stream from the active streams map.","ref":"WebsockexNova.Gun.ConnectionState.html#remove_stream/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionState.remove_stream/2","doc":"* `state` - Current connection state\n* `stream_ref` - Stream reference to remove","ref":"WebsockexNova.Gun.ConnectionState.html#remove_stream/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionState.remove_stream/2","doc":"Updated connection state struct","ref":"WebsockexNova.Gun.ConnectionState.html#remove_stream/2-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionState.remove_streams/2","doc":"Removes multiple streams from the active streams map.","ref":"WebsockexNova.Gun.ConnectionState.html#remove_streams/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionState.remove_streams/2","doc":"* `state` - Current connection state\n* `stream_refs` - List of stream references to remove","ref":"WebsockexNova.Gun.ConnectionState.html#remove_streams/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionState.remove_streams/2","doc":"Updated connection state struct","ref":"WebsockexNova.Gun.ConnectionState.html#remove_streams/2-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionState.setup_auth_handler/3","doc":"Sets up the auth handler module and stores it in the handlers map.","ref":"WebsockexNova.Gun.ConnectionState.html#setup_auth_handler/3"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionState.setup_auth_handler/3","doc":"* `state` - Current connection state\n* `auth_handler` - Auth handler module\n* `options` - Handler options (not stored in ConnectionState)","ref":"WebsockexNova.Gun.ConnectionState.html#setup_auth_handler/3-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionState.setup_auth_handler/3","doc":"Updated connection state struct","ref":"WebsockexNova.Gun.ConnectionState.html#setup_auth_handler/3-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionState.setup_connection_handler/3","doc":"Sets up the connection handler module and stores it in the handlers map.","ref":"WebsockexNova.Gun.ConnectionState.html#setup_connection_handler/3"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionState.setup_connection_handler/3","doc":"* `state` - Current connection state\n* `connection_handler` - Connection handler module\n* `options` - Handler options (not stored in ConnectionState)","ref":"WebsockexNova.Gun.ConnectionState.html#setup_connection_handler/3-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionState.setup_connection_handler/3","doc":"Updated connection state struct","ref":"WebsockexNova.Gun.ConnectionState.html#setup_connection_handler/3-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionState.setup_error_handler/3","doc":"Sets up the error handler module and stores it in the handlers map.","ref":"WebsockexNova.Gun.ConnectionState.html#setup_error_handler/3"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionState.setup_error_handler/3","doc":"* `state` - Current connection state\n* `error_handler` - Error handler module\n* `options` - Handler options (not stored in ConnectionState)","ref":"WebsockexNova.Gun.ConnectionState.html#setup_error_handler/3-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionState.setup_error_handler/3","doc":"Updated connection state struct","ref":"WebsockexNova.Gun.ConnectionState.html#setup_error_handler/3-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionState.setup_logging_handler/3","doc":"Setup the logging handler module.","ref":"WebsockexNova.Gun.ConnectionState.html#setup_logging_handler/3"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionState.setup_logging_handler/3","doc":"* `state` - Current connection state\n* `logging_handler` - Logging handler module\n* `_options` - Logging handler options (not stored in ConnectionState)","ref":"WebsockexNova.Gun.ConnectionState.html#setup_logging_handler/3-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionState.setup_logging_handler/3","doc":"Updated connection state struct","ref":"WebsockexNova.Gun.ConnectionState.html#setup_logging_handler/3-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionState.setup_message_handler/3","doc":"Sets up the message handler module and stores it in the handlers map.","ref":"WebsockexNova.Gun.ConnectionState.html#setup_message_handler/3"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionState.setup_message_handler/3","doc":"* `state` - Current connection state\n* `message_handler` - Message handler module\n* `options` - Handler options (not stored in ConnectionState)","ref":"WebsockexNova.Gun.ConnectionState.html#setup_message_handler/3-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionState.setup_message_handler/3","doc":"Updated connection state struct","ref":"WebsockexNova.Gun.ConnectionState.html#setup_message_handler/3-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionState.setup_subscription_handler/3","doc":"Sets up the subscription handler module and stores it in the handlers map.","ref":"WebsockexNova.Gun.ConnectionState.html#setup_subscription_handler/3"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionState.setup_subscription_handler/3","doc":"* `state` - Current connection state\n* `subscription_handler` - Subscription handler module\n* `options` - Handler options (not stored in ConnectionState)","ref":"WebsockexNova.Gun.ConnectionState.html#setup_subscription_handler/3-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionState.setup_subscription_handler/3","doc":"Updated connection state struct","ref":"WebsockexNova.Gun.ConnectionState.html#setup_subscription_handler/3-returns"},{"type":"type","title":"WebsockexNova.Gun.ConnectionState.status/0","doc":"Connection status","ref":"WebsockexNova.Gun.ConnectionState.html#t:status/0"},{"type":"type","title":"WebsockexNova.Gun.ConnectionState.t/0","doc":"Connection state structure","ref":"WebsockexNova.Gun.ConnectionState.html#t:t/0"},{"type":"function","title":"WebsockexNova.Gun.ConnectionState.update_active_streams/2","doc":"Updates the active streams map.","ref":"WebsockexNova.Gun.ConnectionState.html#update_active_streams/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionState.update_active_streams/2","doc":"* `state` - Current connection state\n* `active_streams` - Map of stream references to stream data","ref":"WebsockexNova.Gun.ConnectionState.html#update_active_streams/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionState.update_active_streams/2","doc":"Updated connection state struct","ref":"WebsockexNova.Gun.ConnectionState.html#update_active_streams/2-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionState.update_callback_pid/2","doc":"Updates the callback PID in the state.","ref":"WebsockexNova.Gun.ConnectionState.html#update_callback_pid/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionState.update_callback_pid/2","doc":"* `state` - Current connection state\n* `pid` - New callback PID","ref":"WebsockexNova.Gun.ConnectionState.html#update_callback_pid/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionState.update_callback_pid/2","doc":"Updated connection state struct","ref":"WebsockexNova.Gun.ConnectionState.html#update_callback_pid/2-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionState.update_connection_handler_state/2","doc":"Updates the connection handler state within the handlers map.","ref":"WebsockexNova.Gun.ConnectionState.html#update_connection_handler_state/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionState.update_connection_handler_state/2","doc":"* `state` - Current connection state\n* `handler_state` - New handler state to store","ref":"WebsockexNova.Gun.ConnectionState.html#update_connection_handler_state/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionState.update_connection_handler_state/2","doc":"Updated connection state struct","ref":"WebsockexNova.Gun.ConnectionState.html#update_connection_handler_state/2-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionState.update_gun_monitor_ref/2","doc":"Updates the Gun connection monitor reference in the state.","ref":"WebsockexNova.Gun.ConnectionState.html#update_gun_monitor_ref/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionState.update_gun_monitor_ref/2","doc":"* `state` - Current connection state\n* `monitor_ref` - Monitor reference for the Gun process","ref":"WebsockexNova.Gun.ConnectionState.html#update_gun_monitor_ref/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionState.update_gun_monitor_ref/2","doc":"Updated connection state struct","ref":"WebsockexNova.Gun.ConnectionState.html#update_gun_monitor_ref/2-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionState.update_gun_pid/2","doc":"Updates the Gun connection PID in the state.","ref":"WebsockexNova.Gun.ConnectionState.html#update_gun_pid/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionState.update_gun_pid/2","doc":"* `state` - Current connection state\n* `gun_pid` - The Gun connection process PID","ref":"WebsockexNova.Gun.ConnectionState.html#update_gun_pid/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionState.update_gun_pid/2","doc":"Updated connection state struct","ref":"WebsockexNova.Gun.ConnectionState.html#update_gun_pid/2-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionState.update_handler/3","doc":"Updates a handler module reference.","ref":"WebsockexNova.Gun.ConnectionState.html#update_handler/3"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionState.update_handler/3","doc":"* `state` - Current connection state\n* `handler_key` - Key for the handler (:connection_handler, :message_handler, etc.)\n* `module` - Handler module","ref":"WebsockexNova.Gun.ConnectionState.html#update_handler/3-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionState.update_handler/3","doc":"Updated connection state struct","ref":"WebsockexNova.Gun.ConnectionState.html#update_handler/3-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionState.update_handlers/2","doc":"Updates multiple handlers at once in the handlers map.","ref":"WebsockexNova.Gun.ConnectionState.html#update_handlers/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionState.update_handlers/2","doc":"* `state` - Current connection state\n* `handlers` - Map of handler keys to handler modules or state","ref":"WebsockexNova.Gun.ConnectionState.html#update_handlers/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionState.update_handlers/2","doc":"Updated connection state struct","ref":"WebsockexNova.Gun.ConnectionState.html#update_handlers/2-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionState.update_status/2","doc":"Updates the connection status in the state.","ref":"WebsockexNova.Gun.ConnectionState.html#update_status/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionState.update_status/2","doc":"* `state` - Current connection state\n* `status` - New status to set","ref":"WebsockexNova.Gun.ConnectionState.html#update_status/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionState.update_status/2","doc":"Updated connection state struct","ref":"WebsockexNova.Gun.ConnectionState.html#update_status/2-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionState.update_stream/3","doc":"Updates or adds a stream in the active streams map.","ref":"WebsockexNova.Gun.ConnectionState.html#update_stream/3"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionState.update_stream/3","doc":"* `state` - Current connection state\n* `stream_ref` - Stream reference\n* `status` - Stream status","ref":"WebsockexNova.Gun.ConnectionState.html#update_stream/3-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionState.update_stream/3","doc":"Updated connection state struct","ref":"WebsockexNova.Gun.ConnectionState.html#update_stream/3-returns"},{"type":"module","title":"WebsockexNova.Gun.ConnectionWrapper","doc":"A thin adapter over Gun's WebSocket implementation, providing a standardized API.","ref":"WebsockexNova.Gun.ConnectionWrapper.html"},{"type":"module","title":"Thin Adapter Pattern - WebsockexNova.Gun.ConnectionWrapper","doc":"This module implements the \"thin adapter\" architectural pattern by:\n\n1. **Abstracting Gun's API**: Provides a simpler, more standardized interface over Gun's\n   lower-level functionality while maintaining full access to Gun's capabilities\n\n2. **Minimizing Logic**: Acts primarily as a pass-through to the underlying Gun library,\n   with minimal logic in the adapter itself\n\n3. **Delegating Business Logic**: Forwards most decisions to specialized modules like\n   ConnectionManager and behavior callbacks\n\n4. **Standardizing Interfaces**: Exposes a consistent API regardless of underlying\n   transport implementation details\n\nThis pattern allows WebsockexNova to potentially support different transport layers\nin the future while maintaining a consistent API for client applications.","ref":"WebsockexNova.Gun.ConnectionWrapper.html#module-thin-adapter-pattern"},{"type":"module","title":"Architecture - WebsockexNova.Gun.ConnectionWrapper","doc":"ConnectionWrapper uses a clean architecture with strict separation of concerns:\n\n* **Core ConnectionWrapper**: Minimal GenServer implementation that routes messages\n* **ConnectionState**: Immutable state management with structured updates\n* **ConnectionManager**: Business logic for connection lifecycle and state transitions\n* **MessageHandlers**: Specialized handlers for different Gun message types\n* **BehaviorHelpers**: Consistent delegation to behavior callbacks\n* **ErrorHandler**: Standardized error handling patterns","ref":"WebsockexNova.Gun.ConnectionWrapper.html#module-architecture"},{"type":"module","title":"Delegation Pattern - WebsockexNova.Gun.ConnectionWrapper","doc":"The module employs a standardized multi-level delegation pattern:\n\n1. **Layer 1**: GenServer callbacks receive Gun messages\n   ```elixir\n   def handle_info({:gun_ws, gun_pid, stream_ref, frame}, %{gun_pid: gun_pid} = state) do\n     MessageHandlers.handle_websocket_frame(gun_pid, stream_ref, frame, state)\n   end\n   ```\n\n2. **Layer 2**: Messages are delegated to specialized MessageHandlers\n   ```elixir\n   # In MessageHandlers module\n   def handle_websocket_frame(gun_pid, stream_ref, frame, state) do\n     # Process frame, then call behavior callbacks through BehaviorHelpers\n     BehaviorHelpers.call_handle_frame(state, frame_type, frame_data, stream_ref)\n   end\n   ```\n\n3. **Layer 3**: MessageHandlers call behavior callbacks through BehaviorHelpers\n   ```elixir\n   # In BehaviorHelpers module\n   def call_handle_frame(state, frame_type, frame_data, stream_ref) do\n     handler_module = Map.get(state.handlers, :connection_handler)\n     handler_state = Map.get(state.handlers, :connection_handler_state)\n     handler_module.handle_frame(frame_type, frame_data, handler_state)\n   end\n   ```\n\n4. **Layer 4**: Results are processed through a consistent handler\n   ```elixir\n   # Back in ConnectionWrapper\n   def process_handler_result({:reply, frame_type, data, state, stream_ref}) do\n     :gun.ws_send(state.gun_pid, stream_ref, {frame_type, data})\n     {:noreply, state}\n   end\n   ```","ref":"WebsockexNova.Gun.ConnectionWrapper.html#module-delegation-pattern"},{"type":"module","title":"Ownership Model - WebsockexNova.Gun.ConnectionWrapper","doc":"Gun connections have a specific ownership model where only one process receives\nmessages from a Gun connection. This module provides a complete ownership\ntransfer protocol:\n\n```elixir\n# Process A - Current owner of Gun connection\nWebsockexNova.Gun.ConnectionWrapper.transfer_ownership(wrapper_pid, target_pid)\n\n# Process B - Receiving ownership\nWebsockexNova.Gun.ConnectionWrapper.receive_ownership(wrapper_pid, gun_pid)\n```\n\nThe transfer protocol carefully manages process monitors, message routing, and state\nsynchronization to ensure reliable handoff between processes.","ref":"WebsockexNova.Gun.ConnectionWrapper.html#module-ownership-model"},{"type":"module","title":"Usage Examples - WebsockexNova.Gun.ConnectionWrapper","doc":"","ref":"WebsockexNova.Gun.ConnectionWrapper.html#module-usage-examples"},{"type":"module","title":"Basic Connection - WebsockexNova.Gun.ConnectionWrapper","doc":"```elixir\n# Open a connection\n{:ok, conn} = WebsockexNova.Gun.ConnectionWrapper.open(\"example.com\", 443, %{\n  transport: :tls,\n  callback_handler: MyApp.WebSocketHandler\n})\n\n# Upgrade to WebSocket\n{:ok, stream_ref} = WebsockexNova.Gun.ConnectionWrapper.upgrade_to_websocket(conn, \"/ws\")\n\n# Send a frame\nWebsockexNova.Gun.ConnectionWrapper.send_frame(conn, stream_ref, {:text, ~s({\"type\": \"ping\"})})\n```","ref":"WebsockexNova.Gun.ConnectionWrapper.html#module-basic-connection"},{"type":"module","title":"With Custom Handlers - WebsockexNova.Gun.ConnectionWrapper","doc":"```elixir\n# Configure with custom handlers\n{:ok, conn} = WebsockexNova.Gun.ConnectionWrapper.open(\"example.com\", 443, %{\n  transport: :tls,\n  callback_handler: MyApp.ConnectionHandler,\n  message_handler: MyApp.MessageHandler,\n  error_handler: MyApp.ErrorHandler\n})\n```","ref":"WebsockexNova.Gun.ConnectionWrapper.html#module-with-custom-handlers"},{"type":"module","title":"Process Transfer - WebsockexNova.Gun.ConnectionWrapper","doc":"```elixir\n# In process A (current owner)\n{:ok, conn} = WebsockexNova.Gun.ConnectionWrapper.open(\"example.com\", 443)\nWebsockexNova.Gun.ConnectionWrapper.transfer_ownership(conn, process_b_pid)\n\n# In process B (new owner)\nWebsockexNova.Gun.ConnectionWrapper.receive_ownership(my_wrapper_pid, gun_pid)\n```","ref":"WebsockexNova.Gun.ConnectionWrapper.html#module-process-transfer"},{"type":"function","title":"WebsockexNova.Gun.ConnectionWrapper.authenticate/3","doc":"Authenticates using the configured auth handler.","ref":"WebsockexNova.Gun.ConnectionWrapper.html#authenticate/3"},{"type":"function","title":"WebsockexNova.Gun.ConnectionWrapper.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"WebsockexNova.Gun.ConnectionWrapper.html#child_spec/1"},{"type":"function","title":"WebsockexNova.Gun.ConnectionWrapper.close/1","doc":"Closes a WebSocket connection.","ref":"WebsockexNova.Gun.ConnectionWrapper.html#close/1"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionWrapper.close/1","doc":"* `conn` - The client connection struct","ref":"WebsockexNova.Gun.ConnectionWrapper.html#close/1-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionWrapper.close/1","doc":"* `:ok`","ref":"WebsockexNova.Gun.ConnectionWrapper.html#close/1-returns"},{"type":"type","title":"WebsockexNova.Gun.ConnectionWrapper.frame/0","doc":"WebSocket frame types","ref":"WebsockexNova.Gun.ConnectionWrapper.html#t:frame/0"},{"type":"function","title":"WebsockexNova.Gun.ConnectionWrapper.get_state/1","doc":"Gets the current connection state.","ref":"WebsockexNova.Gun.ConnectionWrapper.html#get_state/1"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionWrapper.get_state/1","doc":"* `conn` - The client connection struct","ref":"WebsockexNova.Gun.ConnectionWrapper.html#get_state/1-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionWrapper.get_state/1","doc":"* The current state struct","ref":"WebsockexNova.Gun.ConnectionWrapper.html#get_state/1-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionWrapper.open/4","doc":"Opens a connection to a WebSocket server and upgrades to WebSocket in one step.","ref":"WebsockexNova.Gun.ConnectionWrapper.html#open/4"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionWrapper.open/4","doc":"* `host` - Hostname or IP address of the server\n* `port` - Port number of the server (default: 80, or 443 for TLS)\n* `path` - WebSocket endpoint path\n* `options` - Connection options (see `t:options/0`)","ref":"WebsockexNova.Gun.ConnectionWrapper.html#open/4-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionWrapper.open/4","doc":"* `{:ok, %WebsockexNova.ClientConn{}}` on success\n* `{:error, reason}` on failure","ref":"WebsockexNova.Gun.ConnectionWrapper.html#open/4-returns"},{"type":"type","title":"WebsockexNova.Gun.ConnectionWrapper.options/0","doc":"Options for connection wrapper","ref":"WebsockexNova.Gun.ConnectionWrapper.html#t:options/0"},{"type":"type","title":"Possible error atoms returned by API functions: - WebsockexNova.Gun.ConnectionWrapper.options/0","doc":"- :not_connected — The connection is not established or Gun process is missing\n- :stream_not_found — The provided stream reference does not exist or is closed\n- :no_gun_pid — No Gun process is available for ownership transfer\n- :invalid_target_process — The target process for ownership transfer is invalid or dead\n- :gun_process_not_alive — The Gun process is no longer alive\n- :invalid_gun_pid — The provided Gun PID is invalid or dead\n- :http_error — HTTP upgrade or response error (see tuple for details)\n- :invalid_stream_status — The stream is not in a valid state for the requested operation\n- :terminal_error — A terminal error occurred, preventing reconnection\n- :transition_error — State machine transition failed\n- :reconnect_failed — Reconnection attempt failed","ref":"WebsockexNova.Gun.ConnectionWrapper.html#t:options/0-possible-error-atoms-returned-by-api-functions"},{"type":"function","title":"WebsockexNova.Gun.ConnectionWrapper.ping/2","doc":"Sends a ping using the configured connection handler.","ref":"WebsockexNova.Gun.ConnectionWrapper.html#ping/2"},{"type":"function","title":"WebsockexNova.Gun.ConnectionWrapper.process_transport_message/2","doc":"Process a Gun message.","ref":"WebsockexNova.Gun.ConnectionWrapper.html#process_transport_message/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionWrapper.process_transport_message/2","doc":"* `conn` - The client connection struct\n* `message` - The Gun message to process","ref":"WebsockexNova.Gun.ConnectionWrapper.html#process_transport_message/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionWrapper.process_transport_message/2","doc":"* `:ok`","ref":"WebsockexNova.Gun.ConnectionWrapper.html#process_transport_message/2-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionWrapper.receive_ownership/2","doc":"Receives ownership of a Gun connection from another process.\n\nThis function implements the receiving side of the ownership transfer protocol.\nIt's designed to be used in conjunction with `transfer_ownership/2` but can also\nbe used independently to take ownership of any Gun process.\n\nThe receive protocol:\n1. Validates that the provided Gun PID exists and is alive\n2. Creates a monitor for the Gun process\n3. Retrieves information about the Gun connection using `:gun.info/1`\n4. Sets the current process as the Gun process owner with `:gun.set_owner/2`\n5. Updates the ConnectionWrapper state with the Gun PID, monitor reference, and status\n\nThis function is particularly useful when implementing systems where connections\nneed to be dynamically reassigned between processes, such as in worker pools or\nduring process handoffs.","ref":"WebsockexNova.Gun.ConnectionWrapper.html#receive_ownership/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionWrapper.receive_ownership/2","doc":"* `conn` - The client connection struct\n* `gun_pid` - PID of the Gun process being transferred","ref":"WebsockexNova.Gun.ConnectionWrapper.html#receive_ownership/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionWrapper.receive_ownership/2","doc":"* `:ok` on success\n* `{:error, :invalid_gun_pid}` if the Gun process is invalid or dead\n* `{:error, reason}` for Gun-specific errors","ref":"WebsockexNova.Gun.ConnectionWrapper.html#receive_ownership/2-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionWrapper.send_frame/3","doc":"Sends a WebSocket frame.","ref":"WebsockexNova.Gun.ConnectionWrapper.html#send_frame/3"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionWrapper.send_frame/3","doc":"* `conn` - The client connection struct\n* `stream_ref` - The stream reference from the upgrade\n* `frame` - WebSocket frame to send","ref":"WebsockexNova.Gun.ConnectionWrapper.html#send_frame/3-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionWrapper.send_frame/3","doc":"* `:ok` on success\n* `{:error, reason}` on failure","ref":"WebsockexNova.Gun.ConnectionWrapper.html#send_frame/3-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionWrapper.set_status/2","doc":"Sets the connection status (mainly for testing).","ref":"WebsockexNova.Gun.ConnectionWrapper.html#set_status/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionWrapper.set_status/2","doc":"* `conn` - The client connection struct\n* `status` - The new status to set","ref":"WebsockexNova.Gun.ConnectionWrapper.html#set_status/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionWrapper.set_status/2","doc":"* `:ok`","ref":"WebsockexNova.Gun.ConnectionWrapper.html#set_status/2-returns"},{"type":"type","title":"WebsockexNova.Gun.ConnectionWrapper.status/0","doc":"Connection status","ref":"WebsockexNova.Gun.ConnectionWrapper.html#t:status/0"},{"type":"function","title":"WebsockexNova.Gun.ConnectionWrapper.status/2","doc":"Gets the status using the configured connection handler.","ref":"WebsockexNova.Gun.ConnectionWrapper.html#status/2"},{"type":"function","title":"WebsockexNova.Gun.ConnectionWrapper.subscribe/4","doc":"Subscribes to a channel using the configured subscription handler.","ref":"WebsockexNova.Gun.ConnectionWrapper.html#subscribe/4"},{"type":"function","title":"WebsockexNova.Gun.ConnectionWrapper.transfer_ownership/2","doc":"Transfers ownership of the Gun connection to another process.\n\nGun connections have a specific ownership model where only one process receives messages\nfrom a Gun connection. This function implements a safe transfer protocol that ensures\nproper message routing after ownership changes.\n\nThe transfer protocol:\n1. Validates that the Gun process exists and is alive\n2. Validates that the target process exists and is alive\n3. Demonitors the current Gun process monitor\n4. Creates a new monitor for the Gun process\n5. Uses `:gun.set_owner/2` to redirect Gun messages to the new owner\n6. Sends a `:gun_info` message with connection state to the new owner\n7. Updates the local state with the new monitor reference\n\nAfter the transfer completes:\n- The target process will receive all Gun messages\n- The original process maintains its ConnectionWrapper state\n- The ConnectionWrapper maintains its monitor of the Gun process\n\nThis function is useful for load balancing, process migration, or implementing\nmore complex ownership strategies.","ref":"WebsockexNova.Gun.ConnectionWrapper.html#transfer_ownership/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionWrapper.transfer_ownership/2","doc":"* `conn` - The client connection struct\n* `new_owner_pid` - PID of the process that should become the new owner","ref":"WebsockexNova.Gun.ConnectionWrapper.html#transfer_ownership/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionWrapper.transfer_ownership/2","doc":"* `:ok` on success\n* `{:error, :no_gun_pid}` if no Gun process exists\n* `{:error, :invalid_target_process}` if the target process is invalid or dead\n* `{:error, :gun_process_not_alive}` if the Gun process died\n* `{:error, reason}` for other Gun-specific errors","ref":"WebsockexNova.Gun.ConnectionWrapper.html#transfer_ownership/2-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionWrapper.unsubscribe/3","doc":"Unsubscribes from a channel using the configured subscription handler.","ref":"WebsockexNova.Gun.ConnectionWrapper.html#unsubscribe/3"},{"type":"function","title":"WebsockexNova.Gun.ConnectionWrapper.upgrade_to_websocket/3","doc":"Upgrades an HTTP connection to WebSocket.","ref":"WebsockexNova.Gun.ConnectionWrapper.html#upgrade_to_websocket/3"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionWrapper.upgrade_to_websocket/3","doc":"* `conn` - The client connection struct\n* `path` - The WebSocket endpoint path\n* `headers` - Additional headers for the upgrade request","ref":"WebsockexNova.Gun.ConnectionWrapper.html#upgrade_to_websocket/3-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionWrapper.upgrade_to_websocket/3","doc":"* `{:ok, reference()}` on success\n* `{:error, reason}` on failure","ref":"WebsockexNova.Gun.ConnectionWrapper.html#upgrade_to_websocket/3-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionWrapper.wait_for_websocket_upgrade/3","doc":"Waits for the WebSocket upgrade to complete.","ref":"WebsockexNova.Gun.ConnectionWrapper.html#wait_for_websocket_upgrade/3"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionWrapper.wait_for_websocket_upgrade/3","doc":"* `conn` - The client connection struct\n* `stream_ref` - Stream reference from the upgrade\n* `timeout` - Timeout in milliseconds (default: 5000)","ref":"WebsockexNova.Gun.ConnectionWrapper.html#wait_for_websocket_upgrade/3-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionWrapper.wait_for_websocket_upgrade/3","doc":"* `{:ok, headers}` on successful upgrade\n* `{:error, reason}` on failure","ref":"WebsockexNova.Gun.ConnectionWrapper.html#wait_for_websocket_upgrade/3-returns"},{"type":"module","title":"WebsockexNova.Gun.ConnectionWrapper.ErrorHandler","doc":"Standardized error handling for the Gun ConnectionWrapper.\n\nThis module provides consistent error handling patterns for different\ntypes of errors encountered in the ConnectionWrapper. It ensures that:\n\n1. Errors are logged with appropriate context\n2. State is properly updated with error information\n3. Streams are cleaned up as needed\n4. Callbacks are notified with consistent error messages\n5. Proper return values are maintained","ref":"WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.html"},{"type":"function","title":"WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.handle_async_error/3","doc":"Handles async error response in handle_info callbacks.","ref":"WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.html#handle_async_error/3"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.handle_async_error/3","doc":"* `stream_ref` - Reference to the stream with the error (or nil)\n* `reason` - The error reason\n* `state` - Current connection state","ref":"WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.html#handle_async_error/3-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.handle_async_error/3","doc":"`{:noreply, updated_state}` - For asynchronous contexts","ref":"WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.html#handle_async_error/3-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.handle_connection_error/2","doc":"Handles Gun-related connection errors.","ref":"WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.html#handle_connection_error/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.handle_connection_error/2","doc":"* `reason` - The error reason\n* `state` - Current connection state","ref":"WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.html#handle_connection_error/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.handle_connection_error/2","doc":"`{:reply, {:error, reason}, state}` - For synchronous contexts","ref":"WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.html#handle_connection_error/2-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.handle_stream_error/3","doc":"Handles stream-related errors.","ref":"WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.html#handle_stream_error/3"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.handle_stream_error/3","doc":"* `stream_ref` - Reference to the stream with the error\n* `reason` - The error reason\n* `state` - Current connection state","ref":"WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.html#handle_stream_error/3-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.handle_stream_error/3","doc":"`{:reply, {:error, reason}, state}` - For synchronous contexts","ref":"WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.html#handle_stream_error/3-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.handle_transition_error/4","doc":"Handles transition errors when state machine transitions fail.","ref":"WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.html#handle_transition_error/4"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.handle_transition_error/4","doc":"* `current_state` - Current state name\n* `target_state` - Target state name\n* `reason` - The error reason\n* `state` - Current connection state","ref":"WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.html#handle_transition_error/4-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.handle_transition_error/4","doc":"`{:noreply, state}` - For asynchronous contexts","ref":"WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.html#handle_transition_error/4-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.handle_upgrade_error/3","doc":"Handles wait_for_websocket_upgrade errors.","ref":"WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.html#handle_upgrade_error/3"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.handle_upgrade_error/3","doc":"* `stream_ref` - Reference to the stream with the error\n* `reason` - The error reason\n* `state` - Current connection state","ref":"WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.html#handle_upgrade_error/3-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.handle_upgrade_error/3","doc":"`{:reply, {:error, reason}, state}` - For synchronous contexts","ref":"WebsockexNova.Gun.ConnectionWrapper.ErrorHandler.html#handle_upgrade_error/3-returns"},{"type":"module","title":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers","doc":"Handles Gun message processing for the ConnectionWrapper.\n\nThis module provides a set of functions for handling different types of Gun messages,\norganized by category (connection lifecycle, WebSocket, HTTP, error messages).\nEach handler receives the relevant parameters and the current state, and returns\nan updated state.","ref":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.html"},{"type":"function","title":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.handle_connection_down/6","doc":"Handles the `:gun_down` message when a connection is lost.","ref":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.html#handle_connection_down/6"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.handle_connection_down/6","doc":"* `gun_pid` - The Gun connection PID\n* `protocol` - The protocol that was in use\n* `reason` - Reason for the connection loss\n* `state` - Current connection state\n* `killed_streams` - List of stream references that were killed\n* `unprocessed_streams` - List of stream references with unprocessed data","ref":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.html#handle_connection_down/6-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.handle_connection_down/6","doc":"* `{:noreply, updated_state}` - Standard response\n* `{:noreply, {:reconnect, updated_state}}` - Request reconnection\n* `{:stop, reason, updated_state}` - Request process termination","ref":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.html#handle_connection_down/6-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.handle_connection_up/3","doc":"Handles the `:gun_up` message when a connection is established.","ref":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.html#handle_connection_up/3"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.handle_connection_up/3","doc":"* `gun_pid` - The Gun connection PID\n* `protocol` - The protocol that was negotiated\n* `state` - Current connection state","ref":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.html#handle_connection_up/3-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.handle_connection_up/3","doc":"`{:noreply, updated_state}`","ref":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.html#handle_connection_up/3-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.handle_error/4","doc":"Handles the `:gun_error` message when an error occurs.","ref":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.html#handle_error/4"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.handle_error/4","doc":"* `gun_pid` - The Gun connection PID\n* `stream_ref` - Reference to the stream with the error, or nil\n* `reason` - The error reason\n* `state` - Current connection state","ref":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.html#handle_error/4-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.handle_error/4","doc":"`{:noreply, updated_state}`","ref":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.html#handle_error/4-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.handle_http_data/5","doc":"Handles the `:gun_data` message when HTTP data is received.","ref":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.html#handle_http_data/5"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.handle_http_data/5","doc":"* `gun_pid` - The Gun connection PID\n* `stream_ref` - Reference to the stream\n* `is_fin` - Whether this is the final message\n* `data` - The response data\n* `state` - Current connection state","ref":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.html#handle_http_data/5-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.handle_http_data/5","doc":"`{:noreply, updated_state}`","ref":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.html#handle_http_data/5-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.handle_http_response/6","doc":"Handles the `:gun_response` message when an HTTP response is received.","ref":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.html#handle_http_response/6"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.handle_http_response/6","doc":"* `gun_pid` - The Gun connection PID\n* `stream_ref` - Reference to the stream\n* `is_fin` - Whether this is the final message\n* `status` - HTTP status code\n* `headers` - HTTP response headers\n* `state` - Current connection state","ref":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.html#handle_http_response/6-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.handle_http_response/6","doc":"`{:noreply, updated_state}`","ref":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.html#handle_http_response/6-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.handle_websocket_frame/4","doc":"Handles the `:gun_ws` message when a WebSocket frame is received.","ref":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.html#handle_websocket_frame/4"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.handle_websocket_frame/4","doc":"* `gun_pid` - The Gun connection PID\n* `stream_ref` - Reference to the stream that received the frame\n* `frame` - The WebSocket frame\n* `state` - Current connection state","ref":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.html#handle_websocket_frame/4-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.handle_websocket_frame/4","doc":"`{:noreply, updated_state}`","ref":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.html#handle_websocket_frame/4-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.handle_websocket_upgrade/4","doc":"Handles the `:gun_upgrade` message when a WebSocket upgrade succeeds.","ref":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.html#handle_websocket_upgrade/4"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.handle_websocket_upgrade/4","doc":"* `gun_pid` - The Gun connection PID\n* `stream_ref` - Reference to the stream that was upgraded\n* `headers` - Response headers from the upgrade\n* `state` - Current connection state","ref":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.html#handle_websocket_upgrade/4-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.handle_websocket_upgrade/4","doc":"`{:noreply, updated_state}`","ref":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.html#handle_websocket_upgrade/4-returns"},{"type":"function","title":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.notify/2","doc":"Notifies a callback process of an event.","ref":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.html#notify/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.notify/2","doc":"* `callback_pid` - PID of the process to notify (or nil)\n* `message` - Message to send","ref":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.html#notify/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.notify/2","doc":"* `:ok`","ref":"WebsockexNova.Gun.ConnectionWrapper.MessageHandlers.html#notify/2-returns"},{"type":"module","title":"WebsockexNova.Gun.FrameCodec","doc":"Handles encoding and decoding of WebSocket frames.\n\nThis module provides functionality for working with WebSocket frames in Gun.\nIt handles various frame types (text, binary, ping, pong, close) and provides\nutilities for validating frames and working with close codes.\n\nThe module uses a pluggable handler system that allows for custom frame handlers\nto be registered, supporting extensions like permessage-deflate and other WebSocket\nextensions.\n\nGun WebSocket frames are represented as:\n- `{:text, binary()}` - Text frames\n- `{:binary, binary()}` - Binary frames\n- `:ping` or `{:ping, binary()}` - Ping frames\n- `:pong` or `{:pong, binary()}` - Pong frames\n- `:close` or `{:close, code()}` or `{:close, code(), binary()}` - Close frames","ref":"WebsockexNova.Gun.FrameCodec.html"},{"type":"function","title":"WebsockexNova.Gun.FrameCodec.close_code_meaning/1","doc":"Returns the meaning of a WebSocket close code.","ref":"WebsockexNova.Gun.FrameCodec.html#close_code_meaning/1"},{"type":"function","title":"Examples - WebsockexNova.Gun.FrameCodec.close_code_meaning/1","doc":"iex> WebsockexNova.Gun.FrameCodec.close_code_meaning(1000)\n    \"Normal closure\"\n\n    iex> WebsockexNova.Gun.FrameCodec.close_code_meaning(3000)\n    \"Unknown close code\"","ref":"WebsockexNova.Gun.FrameCodec.html#close_code_meaning/1-examples"},{"type":"function","title":"WebsockexNova.Gun.FrameCodec.decode_frame/1","doc":"Decodes a WebSocket frame received from Gun.\n\nTakes a frame in the Gun format and converts it to the internal format.\nUses the appropriate frame handler based on the frame type.","ref":"WebsockexNova.Gun.FrameCodec.html#decode_frame/1"},{"type":"function","title":"Examples - WebsockexNova.Gun.FrameCodec.decode_frame/1","doc":"iex> WebsockexNova.Gun.FrameCodec.decode_frame({:text, \"Hello\"})\n    {:ok, {:text, \"Hello\"}}\n\n    iex> WebsockexNova.Gun.FrameCodec.decode_frame(:ping)\n    {:ok, :ping}\n\n    iex> WebsockexNova.Gun.FrameCodec.decode_frame({:close, 1000, \"Normal closure\"})\n    {:ok, {:close, 1000, \"Normal closure\"}}","ref":"WebsockexNova.Gun.FrameCodec.html#decode_frame/1-examples"},{"type":"type","title":"WebsockexNova.Gun.FrameCodec.decode_result/0","doc":"","ref":"WebsockexNova.Gun.FrameCodec.html#t:decode_result/0"},{"type":"function","title":"WebsockexNova.Gun.FrameCodec.encode_frame/1","doc":"Encodes a WebSocket frame for sending via Gun.\n\nTakes a frame in the internal format and converts it to the format expected by Gun.\nUses the appropriate frame handler based on the frame type.","ref":"WebsockexNova.Gun.FrameCodec.html#encode_frame/1"},{"type":"function","title":"Examples - WebsockexNova.Gun.FrameCodec.encode_frame/1","doc":"iex> WebsockexNova.Gun.FrameCodec.encode_frame({:text, \"Hello\"})\n    {:text, \"Hello\"}\n\n    iex> WebsockexNova.Gun.FrameCodec.encode_frame(:ping)\n    :ping\n\n    iex> WebsockexNova.Gun.FrameCodec.encode_frame({:close, 1000, \"Normal closure\"})\n    {:close, 1000, \"Normal closure\"}","ref":"WebsockexNova.Gun.FrameCodec.html#encode_frame/1-examples"},{"type":"type","title":"WebsockexNova.Gun.FrameCodec.frame/0","doc":"","ref":"WebsockexNova.Gun.FrameCodec.html#t:frame/0"},{"type":"function","title":"WebsockexNova.Gun.FrameCodec.frame_handler_for/1","doc":"Gets the handler module for a specific frame type.\n\nUses the built-in handler registry, which can be extended at runtime\nwith `register_frame_handler/2`.","ref":"WebsockexNova.Gun.FrameCodec.html#frame_handler_for/1"},{"type":"function","title":"Parameters - WebsockexNova.Gun.FrameCodec.frame_handler_for/1","doc":"* `frame_type` - The type of frame to get a handler for","ref":"WebsockexNova.Gun.FrameCodec.html#frame_handler_for/1-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.FrameCodec.frame_handler_for/1","doc":"The handler module for the specified frame type","ref":"WebsockexNova.Gun.FrameCodec.html#frame_handler_for/1-returns"},{"type":"function","title":"WebsockexNova.Gun.FrameCodec.frame_type/1","doc":"Determines the frame type of a WebSocket frame.","ref":"WebsockexNova.Gun.FrameCodec.html#frame_type/1"},{"type":"function","title":"Parameters - WebsockexNova.Gun.FrameCodec.frame_type/1","doc":"* `frame` - The WebSocket frame","ref":"WebsockexNova.Gun.FrameCodec.html#frame_type/1-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.FrameCodec.frame_type/1","doc":"The frame type as an atom (`:text`, `:binary`, etc.)","ref":"WebsockexNova.Gun.FrameCodec.html#frame_type/1-returns"},{"type":"function","title":"WebsockexNova.Gun.FrameCodec.init_handlers_table/0","doc":"Initializes the ETS table for frame handlers.\n\nThis function creates the ETS table for storing frame handlers if it doesn't exist yet.\nIt is called once at application startup to ensure the table exists before it's accessed.","ref":"WebsockexNova.Gun.FrameCodec.html#init_handlers_table/0"},{"type":"function","title":"Returns - WebsockexNova.Gun.FrameCodec.init_handlers_table/0","doc":"* `:ok` - When the table is successfully initialized\n* `{:error, :table_exists}` - When the table already exists","ref":"WebsockexNova.Gun.FrameCodec.html#init_handlers_table/0-returns"},{"type":"function","title":"WebsockexNova.Gun.FrameCodec.register_frame_handler/2","doc":"Registers a custom frame handler for a specific frame type.\n\nThis allows for extending the WebSocket frame handling with custom\nimplementations, such as for handling extensions.","ref":"WebsockexNova.Gun.FrameCodec.html#register_frame_handler/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.FrameCodec.register_frame_handler/2","doc":"* `frame_type` - The type of frame to register a handler for\n* `handler_module` - The module that implements the handler behavior","ref":"WebsockexNova.Gun.FrameCodec.html#register_frame_handler/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.FrameCodec.register_frame_handler/2","doc":"* `:ok` - If the handler was registered successfully\n* `{:error, :table_missing}` - If the handler registry table doesn't exist","ref":"WebsockexNova.Gun.FrameCodec.html#register_frame_handler/2-returns"},{"type":"function","title":"WebsockexNova.Gun.FrameCodec.valid_close_code?/1","doc":"Checks if a WebSocket close code is valid.","ref":"WebsockexNova.Gun.FrameCodec.html#valid_close_code?/1"},{"type":"function","title":"Examples - WebsockexNova.Gun.FrameCodec.valid_close_code?/1","doc":"iex> WebsockexNova.Gun.FrameCodec.valid_close_code?(1000)\n    true\n\n    iex> WebsockexNova.Gun.FrameCodec.valid_close_code?(999)\n    false","ref":"WebsockexNova.Gun.FrameCodec.html#valid_close_code?/1-examples"},{"type":"function","title":"WebsockexNova.Gun.FrameCodec.validate_close_code/1","doc":"Validates a WebSocket close code.","ref":"WebsockexNova.Gun.FrameCodec.html#validate_close_code/1"},{"type":"function","title":"Examples - WebsockexNova.Gun.FrameCodec.validate_close_code/1","doc":"iex> WebsockexNova.Gun.FrameCodec.validate_close_code(1000)\n    :ok\n\n    iex> WebsockexNova.Gun.FrameCodec.validate_close_code(999)\n    {:error, :invalid_close_code}\n\n    iex> WebsockexNova.Gun.FrameCodec.validate_close_code(1005)\n    {:error, :reserved_close_code}","ref":"WebsockexNova.Gun.FrameCodec.html#validate_close_code/1-examples"},{"type":"function","title":"WebsockexNova.Gun.FrameCodec.validate_control_frame_size/1","doc":"Validates the size of a control frame payload.\n\nWebSocket protocol limits control frame payloads to 125 bytes.","ref":"WebsockexNova.Gun.FrameCodec.html#validate_control_frame_size/1"},{"type":"function","title":"Parameters - WebsockexNova.Gun.FrameCodec.validate_control_frame_size/1","doc":"* `data` - Binary payload data to validate","ref":"WebsockexNova.Gun.FrameCodec.html#validate_control_frame_size/1-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.FrameCodec.validate_control_frame_size/1","doc":"* `:ok` - If the payload size is valid\n* `{:error, :control_frame_too_large}` - If payload exceeds 125 bytes","ref":"WebsockexNova.Gun.FrameCodec.html#validate_control_frame_size/1-returns"},{"type":"function","title":"WebsockexNova.Gun.FrameCodec.validate_frame/1","doc":"Validates a WebSocket frame.\n\nChecks if a frame is valid according to the WebSocket spec.\nUses the appropriate frame handler based on the frame type.","ref":"WebsockexNova.Gun.FrameCodec.html#validate_frame/1"},{"type":"function","title":"Examples - WebsockexNova.Gun.FrameCodec.validate_frame/1","doc":"iex> WebsockexNova.Gun.FrameCodec.validate_frame({:text, \"Hello\"})\n    :ok\n\n    iex> WebsockexNova.Gun.FrameCodec.validate_frame({:text, nil})\n    {:error, :invalid_text_data}\n\n    iex> WebsockexNova.Gun.FrameCodec.validate_frame({:close, 1000})\n    :ok","ref":"WebsockexNova.Gun.FrameCodec.html#validate_frame/1-examples"},{"type":"type","title":"WebsockexNova.Gun.FrameCodec.validate_result/0","doc":"","ref":"WebsockexNova.Gun.FrameCodec.html#t:validate_result/0"},{"type":"module","title":"WebsockexNova.Gun.FrameHandlers.BinaryFrameHandler","doc":"Handler for WebSocket binary frames.\n\nImplements encoding, decoding, and validation specific to binary frames.","ref":"WebsockexNova.Gun.FrameHandlers.BinaryFrameHandler.html"},{"type":"module","title":"WebsockexNova.Gun.FrameHandlers.ControlFrameHandler","doc":"Handler for WebSocket control frames (ping, pong, close).\n\nImplements encoding, decoding, and validation specific to control frames.","ref":"WebsockexNova.Gun.FrameHandlers.ControlFrameHandler.html"},{"type":"function","title":"WebsockexNova.Gun.FrameHandlers.ControlFrameHandler.validate_close_code/1","doc":"Validates a WebSocket close code according to the protocol.","ref":"WebsockexNova.Gun.FrameHandlers.ControlFrameHandler.html#validate_close_code/1"},{"type":"function","title":"Parameters - WebsockexNova.Gun.FrameHandlers.ControlFrameHandler.validate_close_code/1","doc":"* `code` - The close code to validate","ref":"WebsockexNova.Gun.FrameHandlers.ControlFrameHandler.html#validate_close_code/1-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.FrameHandlers.ControlFrameHandler.validate_close_code/1","doc":"* `:ok` - If the close code is valid\n* `{:error, reason}` - If the close code is invalid or reserved","ref":"WebsockexNova.Gun.FrameHandlers.ControlFrameHandler.html#validate_close_code/1-returns"},{"type":"function","title":"WebsockexNova.Gun.FrameHandlers.ControlFrameHandler.validate_control_frame_size/1","doc":"Validates that a control frame payload isn't too large.\n\nWebSocket protocol limits control frame payloads to 125 bytes.","ref":"WebsockexNova.Gun.FrameHandlers.ControlFrameHandler.html#validate_control_frame_size/1"},{"type":"function","title":"Parameters - WebsockexNova.Gun.FrameHandlers.ControlFrameHandler.validate_control_frame_size/1","doc":"* `data` - The binary payload to check","ref":"WebsockexNova.Gun.FrameHandlers.ControlFrameHandler.html#validate_control_frame_size/1-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.FrameHandlers.ControlFrameHandler.validate_control_frame_size/1","doc":"* `:ok` - If the payload size is valid\n* `{:error, :control_frame_too_large}` - If the payload is too large","ref":"WebsockexNova.Gun.FrameHandlers.ControlFrameHandler.html#validate_control_frame_size/1-returns"},{"type":"behaviour","title":"WebsockexNova.Gun.FrameHandlers.FrameHandler","doc":"Behavior defining how to handle different types of WebSocket frames.\n\nThis behavior allows for pluggable frame handling, making it easy to\nextend with custom frame types or specialized processing.","ref":"WebsockexNova.Gun.FrameHandlers.FrameHandler.html"},{"type":"callback","title":"WebsockexNova.Gun.FrameHandlers.FrameHandler.decode_frame/1","doc":"Decodes a frame received from Gun.","ref":"WebsockexNova.Gun.FrameHandlers.FrameHandler.html#c:decode_frame/1"},{"type":"callback","title":"Parameters - WebsockexNova.Gun.FrameHandlers.FrameHandler.decode_frame/1","doc":"* `frame` - The frame received from Gun","ref":"WebsockexNova.Gun.FrameHandlers.FrameHandler.html#c:decode_frame/1-parameters"},{"type":"callback","title":"Returns - WebsockexNova.Gun.FrameHandlers.FrameHandler.decode_frame/1","doc":"* `{:ok, decoded_frame}` - Successfully decoded frame\n* `{:error, reason}` - Error decoding the frame","ref":"WebsockexNova.Gun.FrameHandlers.FrameHandler.html#c:decode_frame/1-returns"},{"type":"callback","title":"WebsockexNova.Gun.FrameHandlers.FrameHandler.encode_frame/1","doc":"Encodes a frame for sending via Gun.","ref":"WebsockexNova.Gun.FrameHandlers.FrameHandler.html#c:encode_frame/1"},{"type":"callback","title":"Parameters - WebsockexNova.Gun.FrameHandlers.FrameHandler.encode_frame/1","doc":"* `frame` - The frame to encode","ref":"WebsockexNova.Gun.FrameHandlers.FrameHandler.html#c:encode_frame/1-parameters"},{"type":"callback","title":"Returns - WebsockexNova.Gun.FrameHandlers.FrameHandler.encode_frame/1","doc":"The encoded frame ready for sending through Gun.","ref":"WebsockexNova.Gun.FrameHandlers.FrameHandler.html#c:encode_frame/1-returns"},{"type":"callback","title":"WebsockexNova.Gun.FrameHandlers.FrameHandler.validate_frame/1","doc":"Validates a frame of a specific type.","ref":"WebsockexNova.Gun.FrameHandlers.FrameHandler.html#c:validate_frame/1"},{"type":"callback","title":"Parameters - WebsockexNova.Gun.FrameHandlers.FrameHandler.validate_frame/1","doc":"* `frame` - The frame to validate","ref":"WebsockexNova.Gun.FrameHandlers.FrameHandler.html#c:validate_frame/1-parameters"},{"type":"callback","title":"Returns - WebsockexNova.Gun.FrameHandlers.FrameHandler.validate_frame/1","doc":"* `:ok` - If the frame is valid\n* `{:error, reason}` - If the frame is invalid","ref":"WebsockexNova.Gun.FrameHandlers.FrameHandler.html#c:validate_frame/1-returns"},{"type":"module","title":"WebsockexNova.Gun.FrameHandlers.TextFrameHandler","doc":"Handler for WebSocket text frames.\n\nImplements encoding, decoding, and validation specific to text frames.","ref":"WebsockexNova.Gun.FrameHandlers.TextFrameHandler.html"},{"type":"module","title":"WebsockexNova.Gun.Helpers.BehaviorHelpers","doc":"Helper functions for calling behavior callbacks consistently.\n\nThis module provides a standardized way to call behavior callbacks,\nhandle their responses, and update state accordingly. It ensures\nproper error handling and consistent behavior throughout the application.","ref":"WebsockexNova.Gun.Helpers.BehaviorHelpers.html"},{"type":"function","title":"WebsockexNova.Gun.Helpers.BehaviorHelpers.call_handle_connect/2","doc":"Calls the connection handler's handle_connect callback.\n\nAssembles a connection info map from the state, calls the handler,\nand processes the response.","ref":"WebsockexNova.Gun.Helpers.BehaviorHelpers.html#call_handle_connect/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.Helpers.BehaviorHelpers.call_handle_connect/2","doc":"* `state` - The current connection state\n* `extra_info` - Optional extra information to include in the conn_info map","ref":"WebsockexNova.Gun.Helpers.BehaviorHelpers.html#call_handle_connect/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.Helpers.BehaviorHelpers.call_handle_connect/2","doc":"`{:ok, updated_state}` or error tuple","ref":"WebsockexNova.Gun.Helpers.BehaviorHelpers.html#call_handle_connect/2-returns"},{"type":"function","title":"WebsockexNova.Gun.Helpers.BehaviorHelpers.call_handle_disconnect/2","doc":"Calls the connection handler's handle_disconnect callback.","ref":"WebsockexNova.Gun.Helpers.BehaviorHelpers.html#call_handle_disconnect/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.Helpers.BehaviorHelpers.call_handle_disconnect/2","doc":"* `state` - The current connection state\n* `reason` - The disconnect reason","ref":"WebsockexNova.Gun.Helpers.BehaviorHelpers.html#call_handle_disconnect/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.Helpers.BehaviorHelpers.call_handle_disconnect/2","doc":"`{:ok, updated_state}`, `{:reconnect, updated_state}`, or `{:stop, reason, updated_state}`","ref":"WebsockexNova.Gun.Helpers.BehaviorHelpers.html#call_handle_disconnect/2-returns"},{"type":"function","title":"WebsockexNova.Gun.Helpers.BehaviorHelpers.call_handle_frame/4","doc":"Calls the connection handler's handle_frame callback.","ref":"WebsockexNova.Gun.Helpers.BehaviorHelpers.html#call_handle_frame/4"},{"type":"function","title":"Parameters - WebsockexNova.Gun.Helpers.BehaviorHelpers.call_handle_frame/4","doc":"* `state` - The current connection state\n* `frame_type` - Type of the frame (:text, :binary, etc.)\n* `frame_data` - Data contained in the frame\n* `stream_ref` - Reference to the stream that received the frame","ref":"WebsockexNova.Gun.Helpers.BehaviorHelpers.html#call_handle_frame/4-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.Helpers.BehaviorHelpers.call_handle_frame/4","doc":"`{:ok, updated_state}` or other handler return value with updated state","ref":"WebsockexNova.Gun.Helpers.BehaviorHelpers.html#call_handle_frame/4-returns"},{"type":"function","title":"WebsockexNova.Gun.Helpers.BehaviorHelpers.call_handle_timeout/1","doc":"Calls the connection handler's handle_timeout callback.","ref":"WebsockexNova.Gun.Helpers.BehaviorHelpers.html#call_handle_timeout/1"},{"type":"function","title":"Parameters - WebsockexNova.Gun.Helpers.BehaviorHelpers.call_handle_timeout/1","doc":"* `state` - The current connection state","ref":"WebsockexNova.Gun.Helpers.BehaviorHelpers.html#call_handle_timeout/1-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.Helpers.BehaviorHelpers.call_handle_timeout/1","doc":"`{:ok, updated_state}`, `{:reconnect, updated_state}`, or `{:stop, reason, updated_state}`","ref":"WebsockexNova.Gun.Helpers.BehaviorHelpers.html#call_handle_timeout/1-returns"},{"type":"module","title":"WebsockexNova.Gun.Helpers.StateHelpers","doc":"Helper functions for consistent state operations across the codebase.\n\nThis module provides a single source of truth for common state update operations,\nensuring that state mutations are consistent throughout the application.\n\nIt also includes standardized logging for state transitions to maintain\nconsistent log messages and log levels across the application.","ref":"WebsockexNova.Gun.Helpers.StateHelpers.html"},{"type":"function","title":"WebsockexNova.Gun.Helpers.StateHelpers.handle_connection_established/2","doc":"Updates state for a successful connection.\n\nPerforms standard updates when a connection is established:\n- Updates the Gun PID\n- Sets status to :connected\n- Resets reconnection attempts\n- Logs the connection establishment","ref":"WebsockexNova.Gun.Helpers.StateHelpers.html#handle_connection_established/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.Helpers.StateHelpers.handle_connection_established/2","doc":"* `state` - The current connection state\n* `gun_pid` - The Gun connection process PID","ref":"WebsockexNova.Gun.Helpers.StateHelpers.html#handle_connection_established/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.Helpers.StateHelpers.handle_connection_established/2","doc":"Updated connection state","ref":"WebsockexNova.Gun.Helpers.StateHelpers.html#handle_connection_established/2-returns"},{"type":"function","title":"WebsockexNova.Gun.Helpers.StateHelpers.handle_connection_failure/2","doc":"Updates state for a connection failure.\n\nPerforms standard updates when a connection fails:\n- Records the error reason\n- Sets status to :error\n- Logs the failure with the reason","ref":"WebsockexNova.Gun.Helpers.StateHelpers.html#handle_connection_failure/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.Helpers.StateHelpers.handle_connection_failure/2","doc":"* `state` - The current connection state\n* `reason` - The failure reason","ref":"WebsockexNova.Gun.Helpers.StateHelpers.html#handle_connection_failure/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.Helpers.StateHelpers.handle_connection_failure/2","doc":"Updated connection state","ref":"WebsockexNova.Gun.Helpers.StateHelpers.html#handle_connection_failure/2-returns"},{"type":"function","title":"WebsockexNova.Gun.Helpers.StateHelpers.handle_disconnection/2","doc":"Updates state for a disconnection event.\n\nPerforms standard updates when a connection is lost:\n- Records the disconnect reason\n- Sets status to :disconnected\n- Logs the disconnection with the reason","ref":"WebsockexNova.Gun.Helpers.StateHelpers.html#handle_disconnection/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.Helpers.StateHelpers.handle_disconnection/2","doc":"* `state` - The current connection state\n* `reason` - The disconnect reason","ref":"WebsockexNova.Gun.Helpers.StateHelpers.html#handle_disconnection/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.Helpers.StateHelpers.handle_disconnection/2","doc":"Updated connection state","ref":"WebsockexNova.Gun.Helpers.StateHelpers.html#handle_disconnection/2-returns"},{"type":"function","title":"WebsockexNova.Gun.Helpers.StateHelpers.handle_ownership_transfer/2","doc":"Updates state when receiving connection ownership info from another process.\n\nThis function is critical for proper Gun process ownership transfer between\nprocesses. When a process transfers Gun ownership, it sends a `:gun_info`\nmessage containing state details to help rebuild the state in the new owner.\n\nThe function:\n1. Validates that required information is present in the info map\n2. Updates the connection state with Gun PID and connection status\n3. Ensures a proper process monitor is established\n4. Migrates active streams information if available","ref":"WebsockexNova.Gun.Helpers.StateHelpers.html#handle_ownership_transfer/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.Helpers.StateHelpers.handle_ownership_transfer/2","doc":"* `state` - The current connection state\n* `info` - Connection info map from previous owner with the following keys:\n  * `:gun_pid` - (required) The Gun process PID\n  * `:status` - (required) Current connection status\n  * `:host` - Hostname of connection\n  * `:port` - Port of connection\n  * `:path` - Path for WebSocket endpoint\n  * `:active_streams` - Map of active stream references","ref":"WebsockexNova.Gun.Helpers.StateHelpers.html#handle_ownership_transfer/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.Helpers.StateHelpers.handle_ownership_transfer/2","doc":"Updated connection state","ref":"WebsockexNova.Gun.Helpers.StateHelpers.html#handle_ownership_transfer/2-returns"},{"type":"function","title":"Examples - WebsockexNova.Gun.Helpers.StateHelpers.handle_ownership_transfer/2","doc":"iex> handle_ownership_transfer(state, %{gun_pid: pid, status: :connected})\n    %ConnectionState{gun_pid: pid, status: :connected, ...}","ref":"WebsockexNova.Gun.Helpers.StateHelpers.html#handle_ownership_transfer/2-examples"},{"type":"function","title":"WebsockexNova.Gun.Helpers.StateHelpers.handle_reconnection_attempt/1","doc":"Updates state for a reconnection attempt.\n\nPerforms standard updates when attempting to reconnect:\n- Increments the reconnection attempt counter\n- Sets status to :reconnecting\n- Logs the reconnection attempt with the attempt number","ref":"WebsockexNova.Gun.Helpers.StateHelpers.html#handle_reconnection_attempt/1"},{"type":"function","title":"Parameters - WebsockexNova.Gun.Helpers.StateHelpers.handle_reconnection_attempt/1","doc":"* `state` - The current connection state","ref":"WebsockexNova.Gun.Helpers.StateHelpers.html#handle_reconnection_attempt/1-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.Helpers.StateHelpers.handle_reconnection_attempt/1","doc":"Updated connection state","ref":"WebsockexNova.Gun.Helpers.StateHelpers.html#handle_reconnection_attempt/1-returns"},{"type":"function","title":"WebsockexNova.Gun.Helpers.StateHelpers.handle_websocket_upgrade/2","doc":"Updates state for a successful WebSocket upgrade.\n\nPerforms standard updates when a WebSocket connection is established:\n- Updates the stream status to :websocket\n- Sets overall connection status to :websocket_connected\n- Logs the WebSocket upgrade","ref":"WebsockexNova.Gun.Helpers.StateHelpers.html#handle_websocket_upgrade/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.Helpers.StateHelpers.handle_websocket_upgrade/2","doc":"* `state` - The current connection state\n* `stream_ref` - The stream reference for the WebSocket connection","ref":"WebsockexNova.Gun.Helpers.StateHelpers.html#handle_websocket_upgrade/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.Helpers.StateHelpers.handle_websocket_upgrade/2","doc":"Updated connection state","ref":"WebsockexNova.Gun.Helpers.StateHelpers.html#handle_websocket_upgrade/2-returns"},{"type":"module","title":"WebsockexNova.Gun.Helpers.StateSyncHelpers","doc":"Helpers for synchronizing state between ClientConn and ConnectionState.\n\nThis module provides functions to ensure proper state synchronization between\nthe canonical client state (`WebsockexNova.ClientConn`) and the transport-local\nstate (`WebsockexNova.Gun.ConnectionState`).\n\nIt implements the state layering principles:\n- ClientConn is the canonical source of truth for session state\n- ConnectionState only contains transport-local/process-local data\n- During transitions (reconnection, ownership transfer), state is properly synchronized","ref":"WebsockexNova.Gun.Helpers.StateSyncHelpers.html"},{"type":"module","title":"Usage - WebsockexNova.Gun.Helpers.StateSyncHelpers","doc":"# During ownership transfer:\n    updated_client_conn = StateSyncHelpers.sync_from_connection_state(client_conn, connection_state)\n\n    # When extracting transport-only state:\n    transport_state = StateSyncHelpers.extract_transport_state(client_conn)\n\n    # When updating client conn with transport state changes:\n    updated_client_conn = StateSyncHelpers.update_client_conn_from_transport(client_conn, connection_state)","ref":"WebsockexNova.Gun.Helpers.StateSyncHelpers.html#module-usage"},{"type":"function","title":"WebsockexNova.Gun.Helpers.StateSyncHelpers.create_client_conn/2","doc":"Creates a ClientConn struct from a ConnectionState and existing ClientConn.\n\nThis is used when you need to call a behavior callback that expects a ClientConn\nbut you only have ConnectionState. It preserves session data from the existing\nClientConn while updating transport-specific fields from ConnectionState.","ref":"WebsockexNova.Gun.Helpers.StateSyncHelpers.html#create_client_conn/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.Helpers.StateSyncHelpers.create_client_conn/2","doc":"* `client_conn` - The existing ClientConn struct (can be nil if creating fresh)\n* `conn_state` - The ConnectionState with transport information","ref":"WebsockexNova.Gun.Helpers.StateSyncHelpers.html#create_client_conn/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.Helpers.StateSyncHelpers.create_client_conn/2","doc":"A new or updated ClientConn struct","ref":"WebsockexNova.Gun.Helpers.StateSyncHelpers.html#create_client_conn/2-returns"},{"type":"function","title":"WebsockexNova.Gun.Helpers.StateSyncHelpers.extract_transport_state/1","doc":"Extracts transport-level information from a ClientConn struct.\n\nUse this when you need to populate a ConnectionState with the configuration\nfrom a ClientConn, but don't want to copy user/session state.","ref":"WebsockexNova.Gun.Helpers.StateSyncHelpers.html#extract_transport_state/1"},{"type":"function","title":"Parameters - WebsockexNova.Gun.Helpers.StateSyncHelpers.extract_transport_state/1","doc":"* `client_conn` - The ClientConn struct to extract transport info from","ref":"WebsockexNova.Gun.Helpers.StateSyncHelpers.html#extract_transport_state/1-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.Helpers.StateSyncHelpers.extract_transport_state/1","doc":"Map with transport-only fields that can be used to build a ConnectionState","ref":"WebsockexNova.Gun.Helpers.StateSyncHelpers.html#extract_transport_state/1-returns"},{"type":"function","title":"WebsockexNova.Gun.Helpers.StateSyncHelpers.register_callback/3","doc":"Registers a callback PID in both the ConnectionState and ClientConn.","ref":"WebsockexNova.Gun.Helpers.StateSyncHelpers.html#register_callback/3"},{"type":"function","title":"Parameters - WebsockexNova.Gun.Helpers.StateSyncHelpers.register_callback/3","doc":"* `client_conn` - The ClientConn to update\n* `conn_state` - The ConnectionState to update\n* `pid` - The PID to register","ref":"WebsockexNova.Gun.Helpers.StateSyncHelpers.html#register_callback/3-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.Helpers.StateSyncHelpers.register_callback/3","doc":"{updated_client_conn, updated_conn_state}","ref":"WebsockexNova.Gun.Helpers.StateSyncHelpers.html#register_callback/3-returns"},{"type":"function","title":"WebsockexNova.Gun.Helpers.StateSyncHelpers.sync_client_conn_from_connection/2","doc":"Synchronizes client connection information from ConnectionState to ClientConn.\n\nThis updates the ClientConn with the latest transport state from ConnectionState.\nUsed during ownership transfer and status changes.","ref":"WebsockexNova.Gun.Helpers.StateSyncHelpers.html#sync_client_conn_from_connection/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.Helpers.StateSyncHelpers.sync_client_conn_from_connection/2","doc":"* `client_conn` - The ClientConn to update\n* `conn_state` - The ConnectionState with updated transport information","ref":"WebsockexNova.Gun.Helpers.StateSyncHelpers.html#sync_client_conn_from_connection/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.Helpers.StateSyncHelpers.sync_client_conn_from_connection/2","doc":"Updated ClientConn struct","ref":"WebsockexNova.Gun.Helpers.StateSyncHelpers.html#sync_client_conn_from_connection/2-returns"},{"type":"function","title":"WebsockexNova.Gun.Helpers.StateSyncHelpers.sync_connection_state_from_client/2","doc":"Synchronizes connection state information from ClientConn to ConnectionState.\n\nThis ensures the ConnectionState has the latest configuration from ClientConn\nwithout copying session state. Used during reconnection and initialization.","ref":"WebsockexNova.Gun.Helpers.StateSyncHelpers.html#sync_connection_state_from_client/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.Helpers.StateSyncHelpers.sync_connection_state_from_client/2","doc":"* `conn_state` - The ConnectionState to update\n* `client_conn` - The ClientConn with canonical configuration","ref":"WebsockexNova.Gun.Helpers.StateSyncHelpers.html#sync_connection_state_from_client/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.Helpers.StateSyncHelpers.sync_connection_state_from_client/2","doc":"Updated ConnectionState","ref":"WebsockexNova.Gun.Helpers.StateSyncHelpers.html#sync_connection_state_from_client/2-returns"},{"type":"function","title":"WebsockexNova.Gun.Helpers.StateSyncHelpers.sync_handler_modules/2","doc":"Synchronizes handler modules between ClientConn connection_info and ConnectionState handlers.","ref":"WebsockexNova.Gun.Helpers.StateSyncHelpers.html#sync_handler_modules/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.Helpers.StateSyncHelpers.sync_handler_modules/2","doc":"* `client_conn` - The ClientConn to update\n* `conn_state` - The ConnectionState with handler module information","ref":"WebsockexNova.Gun.Helpers.StateSyncHelpers.html#sync_handler_modules/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.Helpers.StateSyncHelpers.sync_handler_modules/2","doc":"Updated ClientConn struct","ref":"WebsockexNova.Gun.Helpers.StateSyncHelpers.html#sync_handler_modules/2-returns"},{"type":"function","title":"WebsockexNova.Gun.Helpers.StateSyncHelpers.unregister_callback/3","doc":"Unregisters a callback PID from both the ConnectionState and ClientConn.","ref":"WebsockexNova.Gun.Helpers.StateSyncHelpers.html#unregister_callback/3"},{"type":"function","title":"Parameters - WebsockexNova.Gun.Helpers.StateSyncHelpers.unregister_callback/3","doc":"* `client_conn` - The ClientConn to update\n* `conn_state` - The ConnectionState to update\n* `pid` - The PID to unregister","ref":"WebsockexNova.Gun.Helpers.StateSyncHelpers.html#unregister_callback/3-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.Helpers.StateSyncHelpers.unregister_callback/3","doc":"{updated_client_conn, updated_conn_state} - ConnectionState will have nil callback_pid if the removed PID\nwas the current one","ref":"WebsockexNova.Gun.Helpers.StateSyncHelpers.html#unregister_callback/3-returns"},{"type":"function","title":"WebsockexNova.Gun.Helpers.StateSyncHelpers.update_client_conn_from_transport/2","doc":"Updates a ClientConn struct with transport state from a ConnectionState.\n\nThis should be used when the transport layer has information that needs to be\nreflected in the canonical client state (e.g., connection status, error).","ref":"WebsockexNova.Gun.Helpers.StateSyncHelpers.html#update_client_conn_from_transport/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.Helpers.StateSyncHelpers.update_client_conn_from_transport/2","doc":"* `client_conn` - The ClientConn struct to update\n* `conn_state` - The ConnectionState with updated transport information","ref":"WebsockexNova.Gun.Helpers.StateSyncHelpers.html#update_client_conn_from_transport/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.Helpers.StateSyncHelpers.update_client_conn_from_transport/2","doc":"Updated ClientConn struct","ref":"WebsockexNova.Gun.Helpers.StateSyncHelpers.html#update_client_conn_from_transport/2-returns"},{"type":"module","title":"WebsockexNova.Gun.Helpers.StateTracer","doc":"Provides detailed tracing capabilities for connection state transitions.\n\nThis module implements event tracing for WebSocket connection state changes,\nallowing for easier debugging and monitoring of the connection lifecycle.\n\nFeatures:\n- Records all state transitions with timestamps\n- Tracks connection statistics (uptime, reconnection frequency)\n- Provides a searchable history of connection events\n- Can output trace events to a file or monitoring system\n- Supports optional correlation IDs for distributed tracing","ref":"WebsockexNova.Gun.Helpers.StateTracer.html"},{"type":"function","title":"WebsockexNova.Gun.Helpers.StateTracer.export_trace/2","doc":"Exports the trace history to a file.","ref":"WebsockexNova.Gun.Helpers.StateTracer.html#export_trace/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.Helpers.StateTracer.export_trace/2","doc":"* `state` - The connection state\n* `path` - File path for the export","ref":"WebsockexNova.Gun.Helpers.StateTracer.html#export_trace/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.Helpers.StateTracer.export_trace/2","doc":"`:ok` on success, `{:error, reason}` on failure","ref":"WebsockexNova.Gun.Helpers.StateTracer.html#export_trace/2-returns"},{"type":"function","title":"WebsockexNova.Gun.Helpers.StateTracer.get_statistics/1","doc":"Gets connection statistics from the trace context.","ref":"WebsockexNova.Gun.Helpers.StateTracer.html#get_statistics/1"},{"type":"function","title":"Parameters - WebsockexNova.Gun.Helpers.StateTracer.get_statistics/1","doc":"* `state` - The connection state","ref":"WebsockexNova.Gun.Helpers.StateTracer.html#get_statistics/1-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.Helpers.StateTracer.get_statistics/1","doc":"Map of connection statistics","ref":"WebsockexNova.Gun.Helpers.StateTracer.html#get_statistics/1-returns"},{"type":"function","title":"WebsockexNova.Gun.Helpers.StateTracer.get_trace_history/1","doc":"Gets the full trace history from the connection state.","ref":"WebsockexNova.Gun.Helpers.StateTracer.html#get_trace_history/1"},{"type":"function","title":"Parameters - WebsockexNova.Gun.Helpers.StateTracer.get_trace_history/1","doc":"* `state` - The connection state","ref":"WebsockexNova.Gun.Helpers.StateTracer.html#get_trace_history/1-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.Helpers.StateTracer.get_trace_history/1","doc":"List of trace events in chronological order (oldest first)","ref":"WebsockexNova.Gun.Helpers.StateTracer.html#get_trace_history/1-returns"},{"type":"function","title":"WebsockexNova.Gun.Helpers.StateTracer.init_trace/2","doc":"Initializes a new trace context in the connection state.","ref":"WebsockexNova.Gun.Helpers.StateTracer.html#init_trace/2"},{"type":"function","title":"Parameters - WebsockexNova.Gun.Helpers.StateTracer.init_trace/2","doc":"* `state` - The current connection state\n* `trace_id` - Optional trace ID for distributed tracing (generates one if nil)","ref":"WebsockexNova.Gun.Helpers.StateTracer.html#init_trace/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.Helpers.StateTracer.init_trace/2","doc":"Updated connection state with trace context","ref":"WebsockexNova.Gun.Helpers.StateTracer.html#init_trace/2-returns"},{"type":"function","title":"WebsockexNova.Gun.Helpers.StateTracer.trace_transition/5","doc":"Traces a state transition event.","ref":"WebsockexNova.Gun.Helpers.StateTracer.html#trace_transition/5"},{"type":"function","title":"Parameters - WebsockexNova.Gun.Helpers.StateTracer.trace_transition/5","doc":"* `state` - The current connection state\n* `event_type` - The type of event (:connect, :disconnect, :upgrade, etc.)\n* `from_status` - Previous connection status\n* `to_status` - New connection status\n* `metadata` - Additional contextual information about the event","ref":"WebsockexNova.Gun.Helpers.StateTracer.html#trace_transition/5-parameters"},{"type":"function","title":"Returns - WebsockexNova.Gun.Helpers.StateTracer.trace_transition/5","doc":"Updated connection state with the event recorded","ref":"WebsockexNova.Gun.Helpers.StateTracer.html#trace_transition/5-returns"},{"type":"module","title":"WebsockexNova.HandlerInvoker","doc":"Centralized dispatcher for handler module invocation.\n\nThis module provides a single entry point for invoking any handler function\n(connection, message, subscription, auth, error, etc.) in a uniform way.\nIt enables modular, testable, and DRY handler invocation logic.","ref":"WebsockexNova.HandlerInvoker.html"},{"type":"function","title":"WebsockexNova.HandlerInvoker.invoke/4","doc":"Invokes the given function on the specified handler module with the provided args.","ref":"WebsockexNova.HandlerInvoker.html#invoke/4"},{"type":"function","title":"Parameters - WebsockexNova.HandlerInvoker.invoke/4","doc":"- handler_type: atom identifying the handler (e.g., :connection_handler)\n  - fun: atom function name to call (e.g., :handle_connect)\n  - args: list of arguments to pass to the function\n  - handlers: map of handler_type => module","ref":"WebsockexNova.HandlerInvoker.html#invoke/4-parameters"},{"type":"function","title":"Returns - WebsockexNova.HandlerInvoker.invoke/4","doc":"- The result of the handler function, or :no_handler if handler is missing,\n    or :no_function if the function is not exported by the handler module.","ref":"WebsockexNova.HandlerInvoker.html#invoke/4-returns"},{"type":"module","title":"WebsockexNova.Helpers.StateHelpers","doc":"Helper functions for working with application state.\n\nThis module provides standardized methods for updating and managing\ndifferent types of handler states within the application.","ref":"WebsockexNova.Helpers.StateHelpers.html"},{"type":"function","title":"WebsockexNova.Helpers.StateHelpers.buffer_request/4","doc":"Adds a request to the request buffer.\nReturns the updated state.","ref":"WebsockexNova.Helpers.StateHelpers.html#buffer_request/4"},{"type":"function","title":"WebsockexNova.Helpers.StateHelpers.cancel_timeout/3","doc":"Removes and cancels a timeout by id using the provided cancel_fun.\nReturns the updated state.","ref":"WebsockexNova.Helpers.StateHelpers.html#cancel_timeout/3"},{"type":"function","title":"WebsockexNova.Helpers.StateHelpers.flush_buffer/2","doc":"Moves all buffered requests to pending_requests and sets timeouts using the provided make_timer fun.\nReturns {new_state, sent_requests} where sent_requests is the list of flushed requests.","ref":"WebsockexNova.Helpers.StateHelpers.html#flush_buffer/2"},{"type":"function","title":"WebsockexNova.Helpers.StateHelpers.get_host/1","doc":"Retrieves the host from the state, adapter_state, or config, in that order.\nReturns nil if not found.","ref":"WebsockexNova.Helpers.StateHelpers.html#get_host/1"},{"type":"function","title":"WebsockexNova.Helpers.StateHelpers.get_port/1","doc":"Retrieves the port from the state, adapter_state, or config, in that order.\nReturns nil if not found.","ref":"WebsockexNova.Helpers.StateHelpers.html#get_port/1"},{"type":"function","title":"WebsockexNova.Helpers.StateHelpers.get_status/1","doc":"Retrieves the status from the state, adapter_state, or config, in that order.\nReturns nil if not found.","ref":"WebsockexNova.Helpers.StateHelpers.html#get_status/1"},{"type":"function","title":"WebsockexNova.Helpers.StateHelpers.handle_ownership_transfer/2","doc":"Stub for handle_ownership_transfer/2. Not yet implemented.","ref":"WebsockexNova.Helpers.StateHelpers.html#handle_ownership_transfer/2"},{"type":"function","title":"WebsockexNova.Helpers.StateHelpers.pop_pending_request/2","doc":"Removes a pending request and its timeout by id.\nReturns {from, new_state} where from is the original from pid (or nil).","ref":"WebsockexNova.Helpers.StateHelpers.html#pop_pending_request/2"},{"type":"function","title":"WebsockexNova.Helpers.StateHelpers.setup_handler/3","doc":"","ref":"WebsockexNova.Helpers.StateHelpers.html#setup_handler/3"},{"type":"function","title":"WebsockexNova.Helpers.StateHelpers.setup_handler/4","doc":"","ref":"WebsockexNova.Helpers.StateHelpers.html#setup_handler/4"},{"type":"function","title":"WebsockexNova.Helpers.StateHelpers.setup_handler/5","doc":"Sets up a handler in the state with its initial state.","ref":"WebsockexNova.Helpers.StateHelpers.html#setup_handler/5"},{"type":"function","title":"Parameters - WebsockexNova.Helpers.StateHelpers.setup_handler/5","doc":"* `state` - The current state map\n* `handler_type` - The type of handler (must be an atom, e.g., :auth_handler, :error_handler)\n* `handler_module` - The module that implements the handler behavior\n* `handler_options` - Options to pass to the handler's init function (if it has one)\n* `callback_name` - The name of the callback function to call for initializing the handler","ref":"WebsockexNova.Helpers.StateHelpers.html#setup_handler/5-parameters"},{"type":"function","title":"Returns - WebsockexNova.Helpers.StateHelpers.setup_handler/5","doc":"The updated state map with the new handler and its state.","ref":"WebsockexNova.Helpers.StateHelpers.html#setup_handler/5-returns"},{"type":"function","title":"Note - WebsockexNova.Helpers.StateHelpers.setup_handler/5","doc":"`handler_type` must be an atom. The handler state will be stored under the key `{handler_type, :state}` in the handlers map (e.g., `{:auth_handler, :state}`).\nThis avoids dynamic atom creation and is safe for all inputs.","ref":"WebsockexNova.Helpers.StateHelpers.html#setup_handler/5-note"},{"type":"function","title":"WebsockexNova.Helpers.StateHelpers.update_auth_handler_state/2","doc":"Updates the auth handler state within the main state map.\n\nThe handler state is stored under the key `{:auth_handler, :state}` in the handlers map.","ref":"WebsockexNova.Helpers.StateHelpers.html#update_auth_handler_state/2"},{"type":"function","title":"WebsockexNova.Helpers.StateHelpers.update_connection_handler_state/2","doc":"Updates the connection handler state within the main state map.\n\nThe handler state is stored under the key `{:connection_handler, :state}` in the handlers map.","ref":"WebsockexNova.Helpers.StateHelpers.html#update_connection_handler_state/2"},{"type":"function","title":"WebsockexNova.Helpers.StateHelpers.update_error_handler_state/2","doc":"Updates the error handler state within the main state map.\n\nThe handler state is stored under the key `{:error_handler, :state}` in the handlers map.","ref":"WebsockexNova.Helpers.StateHelpers.html#update_error_handler_state/2"},{"type":"function","title":"WebsockexNova.Helpers.StateHelpers.update_message_handler_state/2","doc":"Updates the message handler state within the main state map.\n\nThe handler state is stored under the key `{:message_handler, :state}` in the handlers map.","ref":"WebsockexNova.Helpers.StateHelpers.html#update_message_handler_state/2"},{"type":"function","title":"WebsockexNova.Helpers.StateHelpers.update_subscription_handler_state/2","doc":"Updates the subscription handler state within the main state map.\n\nThe handler state is stored under the key `{:subscription_handler, :state}` in the handlers map.","ref":"WebsockexNova.Helpers.StateHelpers.html#update_subscription_handler_state/2"},{"type":"module","title":"WebsockexNova.Message.SubscriptionManager","doc":"Manages WebSocket subscriptions with support for persistence and automatic resubscription.\n\nThe SubscriptionManager provides a higher-level interface for working with subscriptions,\ndelegating the actual subscription tracking to a `SubscriptionHandler` behavior implementation.\nIt adds features like:\n\n* Centralized subscription management\n* Automatic resubscription after reconnection\n* Subscription state persistence\n* Helper functions for monitoring subscriptions","ref":"WebsockexNova.Message.SubscriptionManager.html"},{"type":"module","title":"Usage Example - WebsockexNova.Message.SubscriptionManager","doc":"```elixir\nalias WebsockexNova.Message.SubscriptionManager\nalias MyApp.Crypto.DeribitSubscriptionHandler\n\n# Initialize a subscription manager\n{:ok, manager} = SubscriptionManager.new(DeribitSubscriptionHandler)\n\n# Subscribe to channels\n{:ok, sub_id1, manager} = SubscriptionManager.subscribe(manager, \"market.btcusd.trades\", %{frequency: \"100ms\"})\n{:ok, sub_id2, manager} = SubscriptionManager.subscribe(manager, \"market.btcusd.orderbook\", %{depth: 10})\n\n# Handle subscription responses\nresponse = %{\"type\" => \"subscribed\", \"id\" => sub_id1}\n{:ok, manager} = SubscriptionManager.handle_response(manager, response)\n\n# When connection is lost, prepare for reconnection\n{:ok, manager} = SubscriptionManager.prepare_for_reconnect(manager)\n\n# After reconnection, resubscribe to channels\nresults = SubscriptionManager.resubscribe_after_reconnect(manager)\n# Process results and get the updated manager\n[{:ok, new_sub_id1, manager} | _] = results\n\n# Check active subscriptions\nactive = SubscriptionManager.active_subscriptions(manager)\n```","ref":"WebsockexNova.Message.SubscriptionManager.html#module-usage-example"},{"type":"module","title":"State Persistence - WebsockexNova.Message.SubscriptionManager","doc":"To persist subscriptions across process restarts:\n\n```elixir\n# Export state before process terminates\nstate_to_save = SubscriptionManager.export_state(manager)\n\n# Later, when restarting:\n{:ok, manager} = SubscriptionManager.new(DeribitSubscriptionHandler)\n{:ok, restored_manager} = SubscriptionManager.import_state(manager, state_to_save)\n```","ref":"WebsockexNova.Message.SubscriptionManager.html#module-state-persistence"},{"type":"function","title":"WebsockexNova.Message.SubscriptionManager.active_subscriptions/1","doc":"Retrieves the active (confirmed) subscriptions.","ref":"WebsockexNova.Message.SubscriptionManager.html#active_subscriptions/1"},{"type":"function","title":"Parameters - WebsockexNova.Message.SubscriptionManager.active_subscriptions/1","doc":"* `manager` - The subscription manager","ref":"WebsockexNova.Message.SubscriptionManager.html#active_subscriptions/1-parameters"},{"type":"function","title":"Returns - WebsockexNova.Message.SubscriptionManager.active_subscriptions/1","doc":"* A map of subscription IDs to subscription details","ref":"WebsockexNova.Message.SubscriptionManager.html#active_subscriptions/1-returns"},{"type":"function","title":"WebsockexNova.Message.SubscriptionManager.export_state/1","doc":"Exports the subscription state for persistence.\n\nThis is useful for saving the state before a process terminates, so it can be\nrestored later.","ref":"WebsockexNova.Message.SubscriptionManager.html#export_state/1"},{"type":"function","title":"Parameters - WebsockexNova.Message.SubscriptionManager.export_state/1","doc":"* `manager` - The subscription manager","ref":"WebsockexNova.Message.SubscriptionManager.html#export_state/1-parameters"},{"type":"function","title":"Returns - WebsockexNova.Message.SubscriptionManager.export_state/1","doc":"* A map containing the serializable state","ref":"WebsockexNova.Message.SubscriptionManager.html#export_state/1-returns"},{"type":"function","title":"WebsockexNova.Message.SubscriptionManager.find_subscription_by_channel/2","doc":"Finds a subscription ID by channel name.","ref":"WebsockexNova.Message.SubscriptionManager.html#find_subscription_by_channel/2"},{"type":"function","title":"Parameters - WebsockexNova.Message.SubscriptionManager.find_subscription_by_channel/2","doc":"* `manager` - The subscription manager\n* `channel` - The channel name to look up","ref":"WebsockexNova.Message.SubscriptionManager.html#find_subscription_by_channel/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Message.SubscriptionManager.find_subscription_by_channel/2","doc":"* The subscription ID if found, nil otherwise","ref":"WebsockexNova.Message.SubscriptionManager.html#find_subscription_by_channel/2-returns"},{"type":"function","title":"WebsockexNova.Message.SubscriptionManager.handle_response/2","doc":"Processes a subscription-related response message.","ref":"WebsockexNova.Message.SubscriptionManager.html#handle_response/2"},{"type":"function","title":"Parameters - WebsockexNova.Message.SubscriptionManager.handle_response/2","doc":"* `manager` - The subscription manager\n* `response` - The response message from the server","ref":"WebsockexNova.Message.SubscriptionManager.html#handle_response/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Message.SubscriptionManager.handle_response/2","doc":"* `{:ok, updated_manager}` - Response processed successfully\n* `{:error, reason, updated_manager}` - Error in response","ref":"WebsockexNova.Message.SubscriptionManager.html#handle_response/2-returns"},{"type":"function","title":"WebsockexNova.Message.SubscriptionManager.import_state/2","doc":"Imports previously exported subscription state.\n\nThis is useful for restoring state after process restart.","ref":"WebsockexNova.Message.SubscriptionManager.html#import_state/2"},{"type":"function","title":"Parameters - WebsockexNova.Message.SubscriptionManager.import_state/2","doc":"* `manager` - The subscription manager\n* `exported_state` - The state previously exported with `export_state/1`","ref":"WebsockexNova.Message.SubscriptionManager.html#import_state/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Message.SubscriptionManager.import_state/2","doc":"* `{:ok, updated_manager}` - Successfully imported state","ref":"WebsockexNova.Message.SubscriptionManager.html#import_state/2-returns"},{"type":"function","title":"WebsockexNova.Message.SubscriptionManager.new/2","doc":"Creates a new subscription manager with the specified handler module.","ref":"WebsockexNova.Message.SubscriptionManager.html#new/2"},{"type":"function","title":"Parameters - WebsockexNova.Message.SubscriptionManager.new/2","doc":"* `handler_module` - Module that implements the `SubscriptionHandler` behavior\n* `initial_state` - Optional initial state for the handler (default: `%{}`)","ref":"WebsockexNova.Message.SubscriptionManager.html#new/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Message.SubscriptionManager.new/2","doc":"* `{:ok, manager}` - A new subscription manager","ref":"WebsockexNova.Message.SubscriptionManager.html#new/2-returns"},{"type":"function","title":"WebsockexNova.Message.SubscriptionManager.prepare_for_reconnect/1","doc":"Prepares for reconnection by storing active subscriptions in state.\n\nThis should be called before a reconnection attempt to ensure subscriptions can be\nrestored after reconnection.","ref":"WebsockexNova.Message.SubscriptionManager.html#prepare_for_reconnect/1"},{"type":"function","title":"Parameters - WebsockexNova.Message.SubscriptionManager.prepare_for_reconnect/1","doc":"* `manager` - The subscription manager","ref":"WebsockexNova.Message.SubscriptionManager.html#prepare_for_reconnect/1-parameters"},{"type":"function","title":"Returns - WebsockexNova.Message.SubscriptionManager.prepare_for_reconnect/1","doc":"* `{:ok, updated_manager}` - Successfully prepared for reconnect","ref":"WebsockexNova.Message.SubscriptionManager.html#prepare_for_reconnect/1-returns"},{"type":"function","title":"WebsockexNova.Message.SubscriptionManager.resubscribe_after_reconnect/1","doc":"Resubscribes to all stored subscriptions after a successful reconnect.\n\nThis should be called after a successful reconnection to restore previously\nactive subscriptions.","ref":"WebsockexNova.Message.SubscriptionManager.html#resubscribe_after_reconnect/1"},{"type":"function","title":"Parameters - WebsockexNova.Message.SubscriptionManager.resubscribe_after_reconnect/1","doc":"* `manager` - The subscription manager with stored subscriptions","ref":"WebsockexNova.Message.SubscriptionManager.html#resubscribe_after_reconnect/1-parameters"},{"type":"function","title":"Returns - WebsockexNova.Message.SubscriptionManager.resubscribe_after_reconnect/1","doc":"* A list of subscription results, each being `{:ok, subscription_id, updated_manager}`\n  or `{:error, reason, updated_manager}`","ref":"WebsockexNova.Message.SubscriptionManager.html#resubscribe_after_reconnect/1-returns"},{"type":"function","title":"WebsockexNova.Message.SubscriptionManager.subscribe/3","doc":"Subscribes to a channel using the configured handler.","ref":"WebsockexNova.Message.SubscriptionManager.html#subscribe/3"},{"type":"function","title":"Parameters - WebsockexNova.Message.SubscriptionManager.subscribe/3","doc":"* `manager` - The subscription manager\n* `channel` - The channel to subscribe to\n* `params` - Optional parameters for the subscription","ref":"WebsockexNova.Message.SubscriptionManager.html#subscribe/3-parameters"},{"type":"function","title":"Returns - WebsockexNova.Message.SubscriptionManager.subscribe/3","doc":"* `{:ok, subscription_id, updated_manager}` - Successful subscription with ID\n* `{:error, reason, updated_manager}` - Failed to subscribe","ref":"WebsockexNova.Message.SubscriptionManager.html#subscribe/3-returns"},{"type":"type","title":"WebsockexNova.Message.SubscriptionManager.t/0","doc":"","ref":"WebsockexNova.Message.SubscriptionManager.html#t:t/0"},{"type":"function","title":"WebsockexNova.Message.SubscriptionManager.unsubscribe/2","doc":"Unsubscribes from a channel using the configured handler.","ref":"WebsockexNova.Message.SubscriptionManager.html#unsubscribe/2"},{"type":"function","title":"Parameters - WebsockexNova.Message.SubscriptionManager.unsubscribe/2","doc":"* `manager` - The subscription manager\n* `subscription_id` - The ID of the subscription to remove","ref":"WebsockexNova.Message.SubscriptionManager.html#unsubscribe/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Message.SubscriptionManager.unsubscribe/2","doc":"* `{:ok, updated_manager}` - Successfully unsubscribed\n* `{:error, reason, updated_manager}` - Failed to unsubscribe","ref":"WebsockexNova.Message.SubscriptionManager.html#unsubscribe/2-returns"},{"type":"module","title":"WebsockexNova.MockTransportImpl","doc":"A simple mock implementation of the WebsockexNova.Transport behaviour for testing and development.\n\nThis module can be used in place of a real transport (e.g., Gun) to simulate transport operations\nin tests or development environments where a real network connection is not desired.","ref":"WebsockexNova.MockTransportImpl.html"},{"type":"module","title":"WebsockexNova.Telemetry.TelemetryEvents","doc":"Central registry and documentation for all telemetry events emitted by WebsockexNova.","ref":"WebsockexNova.Telemetry.TelemetryEvents.html"},{"type":"module","title":"Event Names and Payloads - WebsockexNova.Telemetry.TelemetryEvents","doc":"","ref":"WebsockexNova.Telemetry.TelemetryEvents.html#module-event-names-and-payloads"},{"type":"module","title":"Connection Events - WebsockexNova.Telemetry.TelemetryEvents","doc":"* `[:websockex_nova, :connection, :open]`\n    - measurements: `%{duration: integer()}` (ms, optional)\n    - metadata: `%{connection_id: term(), host: String.t(), port: integer()}`\n\n  * `[:websockex_nova, :connection, :close]`\n    - measurements: `%{duration: integer()}` (ms, optional)\n    - metadata: `%{connection_id: term(), host: String.t(), port: integer(), reason: term()}`\n\n  * `[:websockex_nova, :connection, :websocket_upgrade]`\n    - measurements: `%{duration: integer()}` (ms, optional)\n    - metadata: `%{connection_id: term(), stream_ref: reference(), headers: list()}`","ref":"WebsockexNova.Telemetry.TelemetryEvents.html#module-connection-events"},{"type":"module","title":"Message Events - WebsockexNova.Telemetry.TelemetryEvents","doc":"* `[:websockex_nova, :message, :sent]`\n    - measurements: `%{size: integer(), latency: integer()}` (bytes, ms)\n    - metadata: `%{connection_id: term(), stream_ref: reference(), frame_type: atom()}`\n\n  * `[:websockex_nova, :message, :received]`\n    - measurements: `%{size: integer(), latency: integer()}` (bytes, ms)\n    - metadata: `%{connection_id: term(), stream_ref: reference(), frame_type: atom()}`","ref":"WebsockexNova.Telemetry.TelemetryEvents.html#module-message-events"},{"type":"module","title":"Error Events - WebsockexNova.Telemetry.TelemetryEvents","doc":"* `[:websockex_nova, :error, :occurred]`\n    - measurements: `%{}`\n    - metadata: `%{connection_id: term(), stream_ref: reference() | nil, reason: term(), context: map()}`","ref":"WebsockexNova.Telemetry.TelemetryEvents.html#module-error-events"},{"type":"module","title":"Usage - WebsockexNova.Telemetry.TelemetryEvents","doc":"Use these event names and payloads when emitting or subscribing to telemetry events.","ref":"WebsockexNova.Telemetry.TelemetryEvents.html#module-usage"},{"type":"function","title":"WebsockexNova.Telemetry.TelemetryEvents.connection_close/0","doc":"","ref":"WebsockexNova.Telemetry.TelemetryEvents.html#connection_close/0"},{"type":"function","title":"WebsockexNova.Telemetry.TelemetryEvents.connection_open/0","doc":"","ref":"WebsockexNova.Telemetry.TelemetryEvents.html#connection_open/0"},{"type":"function","title":"WebsockexNova.Telemetry.TelemetryEvents.connection_websocket_upgrade/0","doc":"","ref":"WebsockexNova.Telemetry.TelemetryEvents.html#connection_websocket_upgrade/0"},{"type":"function","title":"WebsockexNova.Telemetry.TelemetryEvents.error_occurred/0","doc":"","ref":"WebsockexNova.Telemetry.TelemetryEvents.html#error_occurred/0"},{"type":"function","title":"WebsockexNova.Telemetry.TelemetryEvents.message_received/0","doc":"","ref":"WebsockexNova.Telemetry.TelemetryEvents.html#message_received/0"},{"type":"function","title":"WebsockexNova.Telemetry.TelemetryEvents.message_sent/0","doc":"","ref":"WebsockexNova.Telemetry.TelemetryEvents.html#message_sent/0"},{"type":"function","title":"WebsockexNova.Telemetry.TelemetryEvents.ownership_transfer_received/0","doc":"Event emitted when an ownership transfer is received from another process.","ref":"WebsockexNova.Telemetry.TelemetryEvents.html#ownership_transfer_received/0"},{"type":"function","title":"Measurements - WebsockexNova.Telemetry.TelemetryEvents.ownership_transfer_received/0","doc":"Empty map (timing is not relevant for this event)","ref":"WebsockexNova.Telemetry.TelemetryEvents.html#ownership_transfer_received/0-measurements"},{"type":"function","title":"Metadata - WebsockexNova.Telemetry.TelemetryEvents.ownership_transfer_received/0","doc":"* `:gun_pid` - The Gun process PID\n* `:host` - Hostname of the connection\n* `:port` - Port number of the connection\n* `:stream_count` - Number of active streams transferred","ref":"WebsockexNova.Telemetry.TelemetryEvents.html#ownership_transfer_received/0-metadata"},{"type":"function","title":"WebsockexNova.Telemetry.TelemetryEvents.subscription_restoration_failed/0","doc":"Event emitted when a subscription restoration fails after reconnection.","ref":"WebsockexNova.Telemetry.TelemetryEvents.html#subscription_restoration_failed/0"},{"type":"function","title":"Measurements - WebsockexNova.Telemetry.TelemetryEvents.subscription_restoration_failed/0","doc":"* `:duration` - Time taken before failure in milliseconds","ref":"WebsockexNova.Telemetry.TelemetryEvents.html#subscription_restoration_failed/0-measurements"},{"type":"function","title":"Metadata - WebsockexNova.Telemetry.TelemetryEvents.subscription_restoration_failed/0","doc":"* `:connection_id` - The connection identifier\n* `:subscription_id` - The subscription identifier\n* `:channel` - The channel that failed to subscribe\n* `:reason` - The failure reason","ref":"WebsockexNova.Telemetry.TelemetryEvents.html#subscription_restoration_failed/0-metadata"},{"type":"function","title":"WebsockexNova.Telemetry.TelemetryEvents.subscription_restored/0","doc":"Event emitted when a subscription is successfully restored after reconnection.","ref":"WebsockexNova.Telemetry.TelemetryEvents.html#subscription_restored/0"},{"type":"function","title":"Measurements - WebsockexNova.Telemetry.TelemetryEvents.subscription_restored/0","doc":"* `:duration` - Time taken to restore the subscription in milliseconds","ref":"WebsockexNova.Telemetry.TelemetryEvents.html#subscription_restored/0-measurements"},{"type":"function","title":"Metadata - WebsockexNova.Telemetry.TelemetryEvents.subscription_restored/0","doc":"* `:connection_id` - The connection identifier\n* `:subscription_id` - The subscription identifier\n* `:channel` - The channel being subscribed to","ref":"WebsockexNova.Telemetry.TelemetryEvents.html#subscription_restored/0-metadata"},{"type":"behaviour","title":"WebsockexNova.Transport","doc":"Behaviour for pluggable WebSocket transport layers in WebsockexNova.\n\nThis abstraction allows the connection process to interact with any transport (e.g., Gun, test mocks)\nvia a consistent, testable interface.","ref":"WebsockexNova.Transport.html"},{"type":"behaviour","title":"Callbacks - WebsockexNova.Transport","doc":"* `send_frame/3` — Send a WebSocket frame on a given stream\n  * `upgrade_to_websocket/3` — Upgrade an HTTP connection to WebSocket\n  * `close/1` — Close the transport connection\n  * `process_transport_message/2` — (Optional) Process a transport-specific message (e.g., Gun event)\n  * `get_state/1` — (Optional) Retrieve the current transport state (for testability)\n  * `open/4` — Open a new transport connection\n  * `schedule_reconnection/2` — (Optional) Schedule a reconnection attempt after a disconnect\n  * `start_connection/1` — (Optional) Start a new connection or reconnection attempt for the transport","ref":"WebsockexNova.Transport.html#module-callbacks"},{"type":"callback","title":"WebsockexNova.Transport.close/1","doc":"Close the transport connection.\nReturns :ok.","ref":"WebsockexNova.Transport.html#c:close/1"},{"type":"type","title":"WebsockexNova.Transport.frame/0","doc":"WebSocket frame type","ref":"WebsockexNova.Transport.html#t:frame/0"},{"type":"callback","title":"WebsockexNova.Transport.get_state/1","doc":"(Optional) Retrieve the current transport state (for testability).","ref":"WebsockexNova.Transport.html#c:get_state/1"},{"type":"callback","title":"WebsockexNova.Transport.open/4","doc":"Open a new transport connection.\nReturns {:ok, state} or {:error, reason}.","ref":"WebsockexNova.Transport.html#c:open/4"},{"type":"callback","title":"Parameters - WebsockexNova.Transport.open/4","doc":"* host - Hostname or IP address to connect to\n  * port - Port number\n  * path - WebSocket endpoint path\n  * options - Transport-specific options (map or keyword)","ref":"WebsockexNova.Transport.html#c:open/4-parameters"},{"type":"callback","title":"WebsockexNova.Transport.process_transport_message/2","doc":"(Optional) Process a transport-specific message (e.g., Gun event).\nReturns updated state or other result as needed.","ref":"WebsockexNova.Transport.html#c:process_transport_message/2"},{"type":"callback","title":"WebsockexNova.Transport.schedule_reconnection/2","doc":"(Optional) Schedule a reconnection attempt after a disconnect.\nReturns the updated transport state.","ref":"WebsockexNova.Transport.html#c:schedule_reconnection/2"},{"type":"callback","title":"WebsockexNova.Transport.send_frame/3","doc":"Send a WebSocket frame on the given stream.\nReturns :ok or {:error, reason}.","ref":"WebsockexNova.Transport.html#c:send_frame/3"},{"type":"callback","title":"WebsockexNova.Transport.start_connection/1","doc":"(Optional) Start a new connection or reconnection attempt for the transport.\nReturns the updated transport state.","ref":"WebsockexNova.Transport.html#c:start_connection/1"},{"type":"type","title":"WebsockexNova.Transport.state/0","doc":"Opaque transport state (implementation-defined)","ref":"WebsockexNova.Transport.html#t:state/0"},{"type":"type","title":"WebsockexNova.Transport.stream_ref/0","doc":"Stream reference (opaque)","ref":"WebsockexNova.Transport.html#t:stream_ref/0"},{"type":"callback","title":"WebsockexNova.Transport.upgrade_to_websocket/3","doc":"Upgrade the connection to WebSocket on the given path and headers.\nReturns {:ok, stream_ref} or {:error, reason}.","ref":"WebsockexNova.Transport.html#c:upgrade_to_websocket/3"},{"type":"module","title":"WebsockexNova.Transport.RateLimiting","doc":"Provides rate limiting functionality for WebSocket requests.\n\nThis module serves as a centralized interface for rate limiting across the application,\ndelegating to the configured RateLimitHandler implementation. It includes:\n\n* An Agent process for persistent state management\n* Functions for checking rate limits before sending requests\n* Automatic processing of queued requests\n* Configuration via application environment","ref":"WebsockexNova.Transport.RateLimiting.html"},{"type":"module","title":"Configuration - WebsockexNova.Transport.RateLimiting","doc":"Configure rate limiting in your application config:\n\n```elixir\nconfig :websockex_nova, :rate_limiting,\n  handler: MyApp.CustomRateLimitHandler,\n  capacity: 100,\n  refill_rate: 5,\n  refill_interval: 1000,\n  queue_limit: 200,\n  process_interval: 100,\n  cost_map: %{\n    subscription: 5,\n    auth: 10,\n    query: 1\n  }\n```","ref":"WebsockexNova.Transport.RateLimiting.html#module-configuration"},{"type":"module","title":"Usage - WebsockexNova.Transport.RateLimiting","doc":"```elixir\nalias WebsockexNova.Transport.RateLimiting\n\n# Start the rate limiter\n{:ok, _pid} = RateLimiting.start_link()\n\n# Check if a request can proceed\nrequest = %{type: :query, method: \"get_data\", data: %{id: 123}}\n\ncase RateLimiting.check(request) do\n  :allow ->\n    # Send the request immediately\n    send_request(request)\n\n  {:queue, request_id} ->\n    # Request is queued, will be processed automatically\n    Logger.info(\"Request queued with id #{request_id}\")\n\n  {:reject, reason} ->\n    # Request was rejected\n    Logger.warning(\"Request rejected: #{reason}\")\nend\n```","ref":"WebsockexNova.Transport.RateLimiting.html#module-usage"},{"type":"function","title":"WebsockexNova.Transport.RateLimiting.check/2","doc":"Checks if a request can proceed based on rate limits.","ref":"WebsockexNova.Transport.RateLimiting.html#check/2"},{"type":"function","title":"Parameters - WebsockexNova.Transport.RateLimiting.check/2","doc":"* `request` - The request to check\n* `server` - The server to check with (default: __MODULE__)","ref":"WebsockexNova.Transport.RateLimiting.html#check/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Transport.RateLimiting.check/2","doc":"* `{:allow, request_id}` - Request can proceed immediately\n* `{:queue, request_id}` - Request is queued and will be processed later\n* `{:reject, reason}` - Request is rejected with the given reason","ref":"WebsockexNova.Transport.RateLimiting.html#check/2-returns"},{"type":"function","title":"WebsockexNova.Transport.RateLimiting.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"WebsockexNova.Transport.RateLimiting.html#child_spec/1"},{"type":"function","title":"WebsockexNova.Transport.RateLimiting.force_process_queue/1","doc":"Forces processing of the queue, regardless of rate limits.\n\nThis is primarily for testing and debugging.","ref":"WebsockexNova.Transport.RateLimiting.html#force_process_queue/1"},{"type":"function","title":"Parameters - WebsockexNova.Transport.RateLimiting.force_process_queue/1","doc":"* `server` - The server to process queue for (default: __MODULE__)","ref":"WebsockexNova.Transport.RateLimiting.html#force_process_queue/1-parameters"},{"type":"function","title":"Returns - WebsockexNova.Transport.RateLimiting.force_process_queue/1","doc":"* `{:ok, count}` - Number of requests processed","ref":"WebsockexNova.Transport.RateLimiting.html#force_process_queue/1-returns"},{"type":"function","title":"WebsockexNova.Transport.RateLimiting.on_process/3","doc":"Adds a callback to be executed when a queued request is processed.","ref":"WebsockexNova.Transport.RateLimiting.html#on_process/3"},{"type":"function","title":"Parameters - WebsockexNova.Transport.RateLimiting.on_process/3","doc":"* `request_id` - The ID of the queued request\n* `callback` - Function to call when the request is processed\n* `server` - The server to register with (default: __MODULE__)","ref":"WebsockexNova.Transport.RateLimiting.html#on_process/3-parameters"},{"type":"function","title":"Returns - WebsockexNova.Transport.RateLimiting.on_process/3","doc":"* `:ok` - Callback registered\n* `{:error, :not_found}` - Request ID not found","ref":"WebsockexNova.Transport.RateLimiting.html#on_process/3-returns"},{"type":"function","title":"WebsockexNova.Transport.RateLimiting.start_link/1","doc":"Starts the rate limiting GenServer.","ref":"WebsockexNova.Transport.RateLimiting.html#start_link/1"},{"type":"function","title":"Options - WebsockexNova.Transport.RateLimiting.start_link/1","doc":"* `:name` - The name to register the server under\n* `:handler` - The module implementing the `WebsockexNova.Behaviors.RateLimitHandler` behavior\n* `:handler_opts` - Options to pass to the handler module\n* `:process_interval` - Interval in milliseconds to process the queue\n* `:mode` - Rate limiting mode (`:normal`, `:always_allow`, `:always_queue`, `:always_reject`)","ref":"WebsockexNova.Transport.RateLimiting.html#start_link/1-options"},{"type":"module","title":"WebsockexNova.Transport.Reconnection","doc":"Provides various reconnection strategies for WebSocket connections.\n\nThis module contains implementations of different backoff algorithms for reconnection:\n\n* `LinearBackoff` - Constant delay between reconnection attempts\n* `ExponentialBackoff` - Exponentially increasing delay (with jitter)\n* `JitteredBackoff` - Linear increase with random jitter","ref":"WebsockexNova.Transport.Reconnection.html"},{"type":"module","title":"Usage - WebsockexNova.Transport.Reconnection","doc":"```elixir\n# Get a strategy\nstrategy = WebsockexNova.Transport.Reconnection.get_strategy(:exponential,\n  initial_delay: 1000,\n  max_delay: 30_000,\n  max_retries: 10\n)\n\n# Calculate delay for attempt number\ndelay = WebsockexNova.Transport.Reconnection.calculate_delay(strategy, attempt)\n\n# Check if we should try again\nif WebsockexNova.Transport.Reconnection.should_retry?(strategy, attempt) do\n  # Schedule reconnect after delay\nend\n```","ref":"WebsockexNova.Transport.Reconnection.html#module-usage"},{"type":"function","title":"WebsockexNova.Transport.Reconnection.calculate_delay/2","doc":"Calculates the delay for a reconnection attempt.\n\nDelegates to the appropriate strategy's calculate_delay function.","ref":"WebsockexNova.Transport.Reconnection.html#calculate_delay/2"},{"type":"function","title":"Parameters - WebsockexNova.Transport.Reconnection.calculate_delay/2","doc":"* `strategy` - The reconnection strategy struct\n* `attempt` - The current attempt number (1-based)","ref":"WebsockexNova.Transport.Reconnection.html#calculate_delay/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Transport.Reconnection.calculate_delay/2","doc":"* Delay in milliseconds","ref":"WebsockexNova.Transport.Reconnection.html#calculate_delay/2-returns"},{"type":"function","title":"WebsockexNova.Transport.Reconnection.get_strategy/2","doc":"Returns a reconnection strategy based on the given type and options.","ref":"WebsockexNova.Transport.Reconnection.html#get_strategy/2"},{"type":"function","title":"Parameters - WebsockexNova.Transport.Reconnection.get_strategy/2","doc":"* `type` - The type of backoff strategy (:linear, :exponential, or :jittered)\n* `opts` - Options specific to the chosen strategy","ref":"WebsockexNova.Transport.Reconnection.html#get_strategy/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Transport.Reconnection.get_strategy/2","doc":"* A strategy struct of the corresponding type","ref":"WebsockexNova.Transport.Reconnection.html#get_strategy/2-returns"},{"type":"function","title":"WebsockexNova.Transport.Reconnection.should_retry?/2","doc":"Determines if a reconnection should be attempted.\n\nDelegates to the appropriate strategy's should_retry? function.","ref":"WebsockexNova.Transport.Reconnection.html#should_retry?/2"},{"type":"function","title":"Parameters - WebsockexNova.Transport.Reconnection.should_retry?/2","doc":"* `strategy` - The reconnection strategy struct\n* `attempt` - The current attempt number (1-based)","ref":"WebsockexNova.Transport.Reconnection.html#should_retry?/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Transport.Reconnection.should_retry?/2","doc":"* `true` if reconnection should be attempted\n* `false` otherwise","ref":"WebsockexNova.Transport.Reconnection.html#should_retry?/2-returns"},{"type":"module","title":"WebsockexNova.Transport.Reconnection.ExponentialBackoff","doc":"Implements an exponential backoff reconnection strategy.\n\nThis strategy increases the delay exponentially with each attempt (2^n),\nwith optional jitter to prevent thundering herd problems. It's well-suited\nfor handling temporary network issues and server overload.","ref":"WebsockexNova.Transport.Reconnection.ExponentialBackoff.html"},{"type":"module","title":"Options - WebsockexNova.Transport.Reconnection.ExponentialBackoff","doc":"* `:initial_delay` - Base delay in milliseconds (default: 1000)\n* `:max_delay` - Maximum delay in milliseconds (default: 30_000)\n* `:jitter_factor` - Random factor to apply (0.0-1.0, default: 0.1)\n* `:max_retries` - Maximum number of retry attempts (default: 5, use :infinity for unlimited)","ref":"WebsockexNova.Transport.Reconnection.ExponentialBackoff.html#module-options"},{"type":"function","title":"WebsockexNova.Transport.Reconnection.ExponentialBackoff.calculate_delay/2","doc":"Calculate the delay for a reconnection attempt.\n\nFor ExponentialBackoff, this increases as 2^(attempt-1) * initial_delay,\nwith optional jitter, and capped at max_delay.","ref":"WebsockexNova.Transport.Reconnection.ExponentialBackoff.html#calculate_delay/2"},{"type":"function","title":"Parameters - WebsockexNova.Transport.Reconnection.ExponentialBackoff.calculate_delay/2","doc":"* `strategy` - The ExponentialBackoff strategy struct\n* `attempt` - The current attempt number (1-based)","ref":"WebsockexNova.Transport.Reconnection.ExponentialBackoff.html#calculate_delay/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Transport.Reconnection.ExponentialBackoff.calculate_delay/2","doc":"* Calculated delay in milliseconds","ref":"WebsockexNova.Transport.Reconnection.ExponentialBackoff.html#calculate_delay/2-returns"},{"type":"function","title":"WebsockexNova.Transport.Reconnection.ExponentialBackoff.should_retry?/2","doc":"Determines if a reconnection should be attempted.","ref":"WebsockexNova.Transport.Reconnection.ExponentialBackoff.html#should_retry?/2"},{"type":"function","title":"Parameters - WebsockexNova.Transport.Reconnection.ExponentialBackoff.should_retry?/2","doc":"* `strategy` - The ExponentialBackoff strategy struct\n* `attempt` - The current attempt number (1-based)","ref":"WebsockexNova.Transport.Reconnection.ExponentialBackoff.html#should_retry?/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Transport.Reconnection.ExponentialBackoff.should_retry?/2","doc":"* `true` if the attempt number is within the maximum retries\n* `false` otherwise","ref":"WebsockexNova.Transport.Reconnection.ExponentialBackoff.html#should_retry?/2-returns"},{"type":"type","title":"WebsockexNova.Transport.Reconnection.ExponentialBackoff.t/0","doc":"","ref":"WebsockexNova.Transport.Reconnection.ExponentialBackoff.html#t:t/0"},{"type":"module","title":"WebsockexNova.Transport.Reconnection.JitteredBackoff","doc":"Implements a linear backoff with jitter reconnection strategy.\n\nThis strategy increases the delay linearly with each attempt and adds\na random jitter to prevent thundering herd problems. It provides a more\ngradual increase than exponential backoff while still spreading out\nreconnections.","ref":"WebsockexNova.Transport.Reconnection.JitteredBackoff.html"},{"type":"module","title":"Options - WebsockexNova.Transport.Reconnection.JitteredBackoff","doc":"* `:base_delay` - Base delay in milliseconds (default: 1000)\n* `:jitter_factor` - Random factor to apply (0.0-1.0, default: 0.2)\n* `:max_retries` - Maximum number of retry attempts (default: 5, use :infinity for unlimited)","ref":"WebsockexNova.Transport.Reconnection.JitteredBackoff.html#module-options"},{"type":"function","title":"WebsockexNova.Transport.Reconnection.JitteredBackoff.calculate_delay/2","doc":"Calculate the delay for a reconnection attempt.\n\nFor JitteredBackoff, this increases linearly as attempt * base_delay,\nwith a random jitter applied.","ref":"WebsockexNova.Transport.Reconnection.JitteredBackoff.html#calculate_delay/2"},{"type":"function","title":"Parameters - WebsockexNova.Transport.Reconnection.JitteredBackoff.calculate_delay/2","doc":"* `strategy` - The JitteredBackoff strategy struct\n* `attempt` - The current attempt number (1-based)","ref":"WebsockexNova.Transport.Reconnection.JitteredBackoff.html#calculate_delay/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Transport.Reconnection.JitteredBackoff.calculate_delay/2","doc":"* Calculated delay in milliseconds","ref":"WebsockexNova.Transport.Reconnection.JitteredBackoff.html#calculate_delay/2-returns"},{"type":"function","title":"WebsockexNova.Transport.Reconnection.JitteredBackoff.should_retry?/2","doc":"Determines if a reconnection should be attempted.","ref":"WebsockexNova.Transport.Reconnection.JitteredBackoff.html#should_retry?/2"},{"type":"function","title":"Parameters - WebsockexNova.Transport.Reconnection.JitteredBackoff.should_retry?/2","doc":"* `strategy` - The JitteredBackoff strategy struct\n* `attempt` - The current attempt number (1-based)","ref":"WebsockexNova.Transport.Reconnection.JitteredBackoff.html#should_retry?/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Transport.Reconnection.JitteredBackoff.should_retry?/2","doc":"* `true` if the attempt number is within the maximum retries\n* `false` otherwise","ref":"WebsockexNova.Transport.Reconnection.JitteredBackoff.html#should_retry?/2-returns"},{"type":"type","title":"WebsockexNova.Transport.Reconnection.JitteredBackoff.t/0","doc":"","ref":"WebsockexNova.Transport.Reconnection.JitteredBackoff.html#t:t/0"},{"type":"module","title":"WebsockexNova.Transport.Reconnection.LinearBackoff","doc":"Implements a constant delay reconnection strategy.\n\nThis strategy always waits the same amount of time between reconnection attempts.\nIt's simple and predictable but doesn't account for network congestion or\nother scaling concerns.","ref":"WebsockexNova.Transport.Reconnection.LinearBackoff.html"},{"type":"module","title":"Options - WebsockexNova.Transport.Reconnection.LinearBackoff","doc":"* `:delay` - Constant delay in milliseconds (default: 1000)\n* `:max_retries` - Maximum number of retry attempts (default: 5, use :infinity for unlimited)","ref":"WebsockexNova.Transport.Reconnection.LinearBackoff.html#module-options"},{"type":"function","title":"WebsockexNova.Transport.Reconnection.LinearBackoff.calculate_delay/2","doc":"Calculate the delay for a reconnection attempt.\n\nFor LinearBackoff, this always returns the same delay regardless of attempt number.","ref":"WebsockexNova.Transport.Reconnection.LinearBackoff.html#calculate_delay/2"},{"type":"function","title":"Parameters - WebsockexNova.Transport.Reconnection.LinearBackoff.calculate_delay/2","doc":"* `strategy` - The LinearBackoff strategy struct\n* `_attempt` - The current attempt number (ignored in this strategy)","ref":"WebsockexNova.Transport.Reconnection.LinearBackoff.html#calculate_delay/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Transport.Reconnection.LinearBackoff.calculate_delay/2","doc":"* Constant delay in milliseconds","ref":"WebsockexNova.Transport.Reconnection.LinearBackoff.html#calculate_delay/2-returns"},{"type":"function","title":"WebsockexNova.Transport.Reconnection.LinearBackoff.should_retry?/2","doc":"Determines if a reconnection should be attempted.","ref":"WebsockexNova.Transport.Reconnection.LinearBackoff.html#should_retry?/2"},{"type":"function","title":"Parameters - WebsockexNova.Transport.Reconnection.LinearBackoff.should_retry?/2","doc":"* `strategy` - The LinearBackoff strategy struct\n* `attempt` - The current attempt number (1-based)","ref":"WebsockexNova.Transport.Reconnection.LinearBackoff.html#should_retry?/2-parameters"},{"type":"function","title":"Returns - WebsockexNova.Transport.Reconnection.LinearBackoff.should_retry?/2","doc":"* `true` if the attempt number is within the maximum retries\n* `false` otherwise","ref":"WebsockexNova.Transport.Reconnection.LinearBackoff.html#should_retry?/2-returns"},{"type":"type","title":"WebsockexNova.Transport.Reconnection.LinearBackoff.t/0","doc":"","ref":"WebsockexNova.Transport.Reconnection.LinearBackoff.html#t:t/0"},{"type":"extras","title":"WebsockexNova","doc":"# WebsockexNova\n\nA robust, highly extensible WebSocket client library for Elixir with elegant abstractions for building specialized WebSocket clients.\n\nThis library is based on the excellent [Websockex](https://github.com/Azolo/websockex) library by [Azolo](https://github.com/Azolo) and has been heavily extended with additional capabilities including behavior-based extensibility, adapter patterns, Gun transport integration, automatic reconnection, subscription management, rate limiting, and more.","ref":"readme.html"},{"type":"extras","title":"Features - WebsockexNova","doc":"- **Behavior-Based Architecture**: Easily extendable through behavior interfaces\n- **Adapter Pattern**: Customize specific aspects of WebSocket communication\n- **Default Implementations**: Sensible defaults provided for all behaviors\n- **Gun Integration**: Uses the battle-tested Gun library as the transport layer\n- **Client Macro**: Simplify creation of service-specific WebSocket clients\n- **Adapter Macro**: Build service-specific adapters with minimal boilerplate\n- **Rate Limiting**: Built-in rate limiting capabilities\n- **Observability**: Rich telemetry and logging support","ref":"readme.html#features"},{"type":"extras","title":"Installation - WebsockexNova","doc":"Add `websockex_nova` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:websockex_nova, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - WebsockexNova","doc":"","ref":"readme.html#usage"},{"type":"extras","title":"Basic Usage - WebsockexNova","doc":"```elixir\n# Connect to a WebSocket server\n{:ok, conn} = WebsockexNova.Client.connect(WebsockexNova.Defaults.DefaultAdapter, %{\n  host: \"echo.websocket.org\",\n  port: 443,\n  path: \"/\",\n  transport: :tls\n})\n\n# Send a message\n{:ok, response} = WebsockexNova.Client.send_text(conn, \"Hello, WebSocket!\")\n\n# Close the connection\nWebsockexNova.Client.close(conn)\n```","ref":"readme.html#basic-usage"},{"type":"extras","title":"Using ClientMacro - WebsockexNova","doc":"Create service-specific clients with minimal boilerplate using `WebsockexNova.ClientMacro`:\n\n```elixir\ndefmodule MyApp.DeribitClient do\n  use WebsockexNova.ClientMacro, adapter: MyApp.DeribitAdapter\n\n  # Add domain-specific methods:\n  def subscribe_to_trades(conn, instrument, opts \\\\ nil) do\n    channel = \"trades.#{instrument}.raw\"\n    subscribe(conn, channel, opts)\n  end\n\n  def subscribe_to_ticker(conn, instrument, opts \\\\ nil) do\n    channel = \"ticker.#{instrument}.raw\"\n    subscribe(conn, channel, opts)\n  end\nend\n\n# Usage\n{:ok, conn} = MyApp.DeribitClient.connect()\n{:ok, _} = MyApp.DeribitClient.subscribe_to_trades(conn, \"BTC-PERPETUAL\")\n```","ref":"readme.html#using-clientmacro"},{"type":"extras","title":"Using AdapterMacro - WebsockexNova","doc":"Create service-specific adapters with minimal boilerplate using `WebsockexNova.Adapter`:\n\n```elixir\ndefmodule MyApp.DeribitAdapter do\n  use WebsockexNova.Adapter\n\n  # Override only what you need:\n  @impl WebsockexNova.Behaviors.ConnectionHandler\n  def connection_info(opts) do\n    defaults = %{\n      host: \"www.deribit.com\",\n      port: 443,\n      path: \"/ws/api/v2\",\n      transport: :tls\n    }\n\n    {:ok, Map.merge(defaults, opts)}\n  end\n\n  @impl WebsockexNova.Behaviors.MessageHandler\n  def handle_message(message, state) do\n    # Custom message handling...\n    {:ok, decoded_message, updated_state}\n  end\nend\n```","ref":"readme.html#using-adaptermacro"},{"type":"extras","title":"Documentation - WebsockexNova","doc":"Comprehensive documentation is available in the `docs/` directory:\n\n- **Architecture Overview**: `docs/architecture.md`\n- **Client Macro Guide**: `docs/client_macro.md`\n- **Implementation Plan**: `docs/plan.md`","ref":"readme.html#documentation"},{"type":"extras","title":"License - WebsockexNova","doc":"This project is licensed under the MIT License - see the LICENSE file for details.","ref":"readme.html#license"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.1"}}